@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,7 +61,7 @@ import org.hsqldb.types.Collation;
  * It holds the data structures that form an HSQLDB database instance.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class Database {
@@ -723,9 +723,7 @@ public class Database {
             sb.setLength(0);
         }
 
-        if (!collation.isDefaultCollation() || !collation.isPadSpace()) {
-            list.add(collation.getDatabaseCollationSQL());
-        }
+        list.add(collation.getDatabaseCollationSQL());
 
         HashMappedList lobTables =
             schemaManager.getTables(SqlInvariants.LOBS_SCHEMA);
@@ -776,11 +774,7 @@ public class Database {
 
         r.addRows(list);
 
-        // optional comments on tables etc.
-        list = schemaManager.getCommentsSQL();
-
-        r.addRows(list);
-
+        // table spaces
         list = schemaManager.getTableSpaceSQL();
 
         r.addRows(list);
@@ -860,10 +854,6 @@ public class Database {
 
         public void start() {
 
-            if (!logger.isLogged()) {
-                return;
-            }
-
             synchronized (this) {
                 if (waiting) {
                     return;
