@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,6 +48,7 @@ import org.hsqldb.lib.Set;
 import org.hsqldb.navigator.RowSetNavigatorData;
 import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.result.Result;
+import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.types.ArrayType;
 import org.hsqldb.types.CharacterType;
 import org.hsqldb.types.Collation;
@@ -61,7 +62,7 @@ import org.hsqldb.types.Types;
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class Expression implements Cloneable {
@@ -78,8 +79,8 @@ public class Expression implements Cloneable {
     static final Expression[] emptyArray = new Expression[]{};
 
     //
-    static final Expression EXPR_TRUE  = new ExpressionLogical(true);
-    static final Expression EXPR_FALSE = new ExpressionLogical(false);
+    public static final Expression EXPR_TRUE  = new ExpressionBoolean(true);
+    public static final Expression EXPR_FALSE = new ExpressionBoolean(false);
 
     // type
     protected int opType;
@@ -110,6 +111,7 @@ public class Expression implements Cloneable {
 
     // data type
     protected Type dataType;
+    protected int  groupingType;
 
     //
     int queryTableColumnIndex = -1;    // >= 0 when it is used for order by
@@ -540,7 +542,7 @@ public class Expression implements Cloneable {
             }
         }
 
-        if (OpTypes.aggregateFunctionSet.contains(opType)) {
+        if (isSelfAggregate()) {
             return false;
         }
 
@@ -622,7 +624,7 @@ public class Expression implements Cloneable {
         case OpCodes.TABLE_SUBQUERY :
         case OpCodes.ROW_SUBQUERY :
 */
-        if (OpTypes.aggregateFunctionSet.contains(opType)) {
+        if (isSelfAggregate()) {
             return false;
         }
 
@@ -1117,8 +1119,6 @@ public class Expression implements Cloneable {
                 break;
 
             case OpTypes.VALUELIST :
-
-                /** @todo - should it fall through */
                 break;
 
             case OpTypes.ROW :
@@ -1564,7 +1564,12 @@ public class Expression implements Cloneable {
                 RowSetNavigatorData navigator = table.getNavigator(session);
                 Result              result    = Result.newResult(navigator);
 
-                result.metaData = table.queryExpression.getMetaData();
+                if (table.queryExpression == null) {
+                    result.metaData = ResultMetaData.newResultMetaData(
+                        table.getColumnTypes(), table.getColumnLabels());
+                } else {
+                    result.metaData = table.queryExpression.getMetaData();
+                }
 
                 return result;
             }
@@ -1869,7 +1874,7 @@ public class Expression implements Cloneable {
         nodes[RIGHT] = temp;
     }
 
-    void setAttributesAsColumn(ColumnSchema column, boolean isWritable) {
+    void setAttributesAsColumn(ColumnSchema column) {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
 
@@ -1981,6 +1986,36 @@ public class Expression implements Cloneable {
         }
     }
 
+    public void resolveGrantFilter(Session session, Table table) {
+
+        RangeGroup ranges = new RangeGroupSimple(table.getDefaultRanges(),
+            false);
+        HsqlList set = resolveColumnReferences(session, ranges,
+                                               RangeGroup.emptyArray, null);
+
+        if (set != null && !set.isEmpty()) {
+            Expression e = (Expression) set.get(0);
+
+            e.getColumnName();
+
+            throw Error.error(ErrorCode.X_42501, e.getColumnName());
+        }
+
+        set = collectAllSubqueries(null);
+
+        if (set != null) {
+            if (set.size() > 0) {
+                if (set.size() > 1) {}
+
+                TableDerived subquery = (TableDerived) set.get(0);
+
+                if (subquery.isCorrelated()) {
+                    throw Error.error(ErrorCode.X_42501);
+                }
+            }
+        }
+    }
+
     static HsqlList resolveColumnSet(Session session,
                                      RangeVariable[] rangeVars,
                                      RangeGroup[] rangeGroups,
@@ -2064,6 +2099,11 @@ public class Expression implements Cloneable {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
 
+    public SetFunction updateAggregatingValue(Session session,
+            SetFunction currValue, SetFunction value) {
+        throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
+    }
+
     public Object getAggregatedValue(Session session, SetFunction currValue) {
         throw Error.runtimeError(ErrorCode.U_S0500, "Expression");
     }
