@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.types.Type;
  * Implementation of column, variable, parameter, etc. access operations.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ExpressionColumn extends Expression {
@@ -98,7 +98,7 @@ public class ExpressionColumn extends Expression {
 
     //
     NumberSequence sequence;
-    boolean        isWritable;    // = false; true if column of writable table
+    boolean        isUpdateColumn;
 
     //
     boolean isParam;
@@ -117,6 +117,17 @@ public class ExpressionColumn extends Expression {
         this.columnName = column;
     }
 
+    /**
+     * Column reference in MySQL INSERT
+     */
+    ExpressionColumn(String column) {
+
+        super(OpTypes.COLUMN);
+
+        this.columnName     = column;
+        this.isUpdateColumn = true;
+    }
+
     ExpressionColumn(ColumnSchema column) {
 
         super(OpTypes.COLUMN);
@@ -170,9 +181,7 @@ public class ExpressionColumn extends Expression {
 
         super(type);
 
-        if (type == OpTypes.DYNAMIC_PARAM) {
-            isParam = true;
-        } else if (type == OpTypes.ROWNUM) {
+        if (type == OpTypes.ROWNUM) {
             columnName = rownumName.name;
             dataType   = Type.SQL_INTEGER;
         }
@@ -185,6 +194,13 @@ public class ExpressionColumn extends Expression {
 
         super(type);
 
+        if (type == OpTypes.DYNAMIC_PARAM) {
+            isParam        = true;
+            parameterIndex = columnIndex;
+
+            return;
+        }
+
         this.column      = (ColumnSchema) diagnosticsList.get(columnIndex);
         this.columnIndex = columnIndex;
         this.dataType    = column.dataType;
@@ -198,6 +214,24 @@ public class ExpressionColumn extends Expression {
         this.columnName = name;
     }
 
+    /**
+     * for GROUPING function
+     */
+    ExpressionColumn(Expression groups) {
+
+        super(OpTypes.GROUPING);
+
+        Expression[] exprs = groups.nodes;
+
+        if (groups.nodes.length == 0) {
+            exprs    = new Expression[1];
+            exprs[0] = groups;
+        }
+
+        this.nodes    = exprs;
+        this.dataType = Type.SQL_INTEGER;
+    }
+
     /**
      * Creates an OpCodes.ASTERISK expression
      */
@@ -263,11 +297,9 @@ public class ExpressionColumn extends Expression {
         }
     }
 
-    void setAttributesAsColumn(ColumnSchema column, boolean isWritable) {
-
-        this.column     = column;
-        dataType        = column.getDataType();
-        this.isWritable = isWritable;
+    void setAttributesAsColumn(ColumnSchema column) {
+        this.column = column;
+        dataType    = column.getDataType();
     }
 
     SimpleName getSimpleName() {
@@ -393,6 +425,7 @@ public class ExpressionColumn extends Expression {
             case OpTypes.DIAGNOSTICS_VARIABLE :
                 break;
 
+            case OpTypes.GROUPING :
             case OpTypes.COALESCE :
                 for (int i = 0; i < nodes.length; i++) {
                     nodes[i].resolveColumnReferences(session, rangeGroup,
@@ -411,6 +444,16 @@ public class ExpressionColumn extends Expression {
                     return unresolvedSet;
                 }
 
+                if (isUpdateColumn) {
+                    if (unresolvedSet == null) {
+                        unresolvedSet = new ArrayListIdentity();
+                    }
+
+                    unresolvedSet.add(this);
+
+                    return unresolvedSet;
+                }
+
                 for (int i = 0; i < rangeCount; i++) {
                     RangeVariable rangeVar = rangeVarArray[i];
 
@@ -524,12 +567,16 @@ public class ExpressionColumn extends Expression {
                 }
 
                 if (resolveColumnReference(rangeVar, true)) {
-                    if (opType == OpTypes.COLUMN) {
-                        rangeGroup.setCorrelated();
+                    switch (opType) {
 
-                        for (int idxx = rangeGroups.length - 1; idxx > idx;
-                                idxx--) {
-                            rangeGroups[idxx].setCorrelated();
+                        case OpTypes.COLUMN :
+                        case OpTypes.COALESCE : {
+                            rangeGroup.setCorrelated();
+
+                            for (int idxx = rangeGroups.length - 1; idxx > idx;
+                                    idxx--) {
+                                rangeGroups[idxx].setCorrelated();
+                            }
                         }
                     }
 
@@ -541,8 +588,7 @@ public class ExpressionColumn extends Expression {
         return false;
     }
 
-    private boolean resolveColumnReference(RangeVariable rangeVar,
-                                           boolean outer) {
+    boolean resolveColumnReference(RangeVariable rangeVar, boolean outer) {
 
         if (tableName == null) {
             Expression e = rangeVar.getColumnExpression(columnName);
@@ -662,6 +708,14 @@ public class ExpressionColumn extends Expression {
 
         switch (opType) {
 
+            case OpTypes.GROUPING :
+                if (session.sessionContext.groupSet == null) {
+                    return 0;
+                }
+
+                return session.sessionContext.groupSet.isGrouped(
+                    session.sessionContext.currentGroup, this);
+
             case OpTypes.DEFAULT :
                 return null;
 
@@ -797,6 +851,13 @@ public class ExpressionColumn extends Expression {
                     return sb.toString();
                 }
             }
+            case OpTypes.SIMPLE_COLUMN : {
+                if (alias != null) {
+                    return alias.getStatementName();
+                } else {
+                    return Tokens.T_COLUMN_NAME;
+                }
+            }
             case OpTypes.MULTICOLUMN : {
                 if (nodes.length == 0) {
                     return "*";
@@ -818,6 +879,27 @@ public class ExpressionColumn extends Expression {
 
                 return sb.toString();
             }
+            case OpTypes.GROUPING : {
+                StringBuffer sb = new StringBuffer();
+
+                sb.append("GROUPING(");
+
+                for (int i = 0; i < nodes.length; i++) {
+                    Expression e = nodes[i];
+
+                    if (i > 0) {
+                        sb.append(',');
+                    }
+
+                    String s = e.getSQL();
+
+                    sb.append(s);
+                }
+
+                sb.append(")");
+
+                return sb.toString();
+            }
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500,
                                          "ExpressionColumn");
@@ -1144,7 +1226,7 @@ public class ExpressionColumn extends Expression {
     }
 
     /**
-     * SIMPLE_COLUMN expressions are now (2.3.5) always ExpressionColumn
+     * SIMPLE_COLUMN expressions are now (2.4.0) always ExpressionColumn
      */
     boolean equals(Expression other) {
 
@@ -1171,6 +1253,8 @@ public class ExpressionColumn extends Expression {
                 return nodes == other.nodes;
 
             case OpTypes.DYNAMIC_PARAM :
+                return parameterIndex == other.parameterIndex;
+
             case OpTypes.VARIABLE :
             case OpTypes.PARAMETER :
             case OpTypes.COLUMN :
