@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@ import org.hsqldb.types.Types;
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ExpressionOp extends Expression {
@@ -559,10 +559,6 @@ public class ExpressionOp extends Expression {
             dataType = nodes[RIGHT].dataType;
 
             if (!nodes[RIGHT].dataType.equals(nodes[LEFT].dataType)) {
-                if (dataType.isCharacterType()) {
-                    dataType = Type.SQL_VARCHAR_DEFAULT;
-                }
-
                 nodes[LEFT] = new ExpressionOp(nodes[LEFT], dataType);
             }
         } else {
