@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,6 +37,7 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
+import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.IntKeyIntValueHashMap;
 import org.hsqldb.map.ValuePool;
@@ -48,16 +49,13 @@ import org.hsqldb.types.Types;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ParserBase {
 
-    protected Scanner scanner;
-    protected Token   token;
-
-    //
-    private final Token dummyToken = new Token();
+    protected Scanner  scanner;
+    protected Token    token;
 
     //
     protected int           partPosition;
@@ -77,8 +75,10 @@ public class ParserBase {
      * @param scanner the token source from which to parse commands
      */
     ParserBase(Scanner scanner) {
-        this.scanner = scanner;
-        this.token   = scanner.token;
+
+        this.scanner           = scanner;
+        this.token             = scanner.token;
+        this.recordedStatement = new HsqlArrayList();
     }
 
     public Scanner getScanner() {
@@ -113,7 +113,8 @@ public class ParserBase {
         isSchemaDefinition        = false;
         isViewDefinition          = false;
         isRecording               = false;
-        recordedStatement         = null;
+
+        recordedStatement.clear();
     }
 
     int getPosition() {
@@ -153,10 +154,6 @@ public class ParserBase {
         return scanner.getPart(position, scanner.getTokenPosition());
     }
 
-    String getLastPartAndCurrent(int position) {
-        return scanner.getPart(position, scanner.getPosition());
-    }
-
     String getStatement(int startPosition, short[] startTokens) {
 
         while (true) {
@@ -213,13 +210,15 @@ public class ParserBase {
     }
 
     //
-    void startRecording() {
+    Recorder startRecording() {
 
-        recordedStatement = new HsqlArrayList();
+        if (!isRecording) {
+            recordedStatement.add(token.duplicate());
 
-        recordedStatement.add(token.duplicate());
+            isRecording = true;
+        }
 
-        isRecording = true;
+        return new Recorder();
     }
 
     Token getRecordedToken() {
@@ -231,19 +230,38 @@ public class ParserBase {
         }
     }
 
-    Token[] getRecordedStatement() {
+    void replaceToken(String tokenString) {
 
-        isRecording = false;
+        scanner.replaceToken(tokenString);
 
-        recordedStatement.remove(recordedStatement.size() - 1);
+        if (isRecording) {
+            Token dup = token.duplicate();
 
-        Token[] tokens = new Token[recordedStatement.size()];
+            dup.position = scanner.getTokenPosition();
 
-        recordedStatement.toArray(tokens);
+            recordedStatement.add(dup);
+        }
+    }
 
-        recordedStatement = null;
+    void replaceToken(Token token1, Token token2) {
 
-        return tokens;
+        String tokenString = token1.tokenString;
+
+        if (token2 != null) {
+            tokenString += " " + token2.tokenString;
+        }
+
+        scanner.replaceToken(tokenString);
+
+        if (isRecording) {
+            token1.position = scanner.getTokenPosition();
+
+            recordedStatement.set(recordedStatement.size() - 1, token1);
+
+            if (token2 != null) {
+                recordedStatement.add(token2);
+            }
+        }
     }
 
     void read() {
@@ -680,7 +698,6 @@ public class ParserBase {
 
         startToken       = endToken = token.tokenType;
         startTokenString = token.tokenString;
-
         startIndex = ArrayUtil.find(Tokens.SQL_INTERVAL_FIELD_CODES,
                                     startToken);
 
@@ -908,4 +925,86 @@ public class ParserBase {
     public Number convertToNumber(String s, NumberType type) {
         return scanner.convertToNumber(s, type);
     }
+
+    /**
+     * read list of comma separated prop = value pairs as tokens
+     * optionalEquals, requireEquals for use of '='
+     */
+    HashMappedList readPropertyValuePairs(boolean optionalEquals,
+                                          boolean requireEquals) {
+
+        HashMappedList list = null;
+        String         prop;
+        Token          value;
+        int            pos;
+
+        do {
+            pos = getPosition();
+
+            if (!token.isUndelimitedIdentifier) {
+                break;
+            }
+
+            prop = token.tokenString;
+
+            read();
+
+            if (token.tokenType == Tokens.X_ENDPARSE) {
+                break;
+            }
+
+            boolean equals = readIfThis(Tokens.EQUALS_OP);
+
+            if (optionalEquals) {}
+            else if (requireEquals) {
+                if (!equals) {
+                    break;
+                }
+            } else {
+                if (equals) {
+                    break;
+                }
+            }
+
+            if (token.tokenType == Tokens.X_VALUE
+                    || token.tokenType == Tokens.X_IDENTIFIER) {}
+            else {
+                break;
+            }
+
+            value = token.duplicate();
+
+            if (list == null) {
+                list = new HashMappedList();
+            }
+
+            list.put(prop, value);
+            read();
+
+            if (!readIfThis(Tokens.COMMA)) {
+                pos = getPosition();
+
+                break;
+            }
+        } while (true);
+
+        rewind(pos);
+
+        return list;
+    }
+
+    class Recorder {
+
+        int position = recordedStatement.size() - 1;
+
+        String getSQL() {
+
+            int     size   = recordedStatement.size() - position - 1;
+            Token[] tokens = new Token[size];
+
+            recordedStatement.toArraySlice(tokens, position, position + size);
+
+            return Token.getSQL(tokens);
+        }
+    }
 }
