@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@ package org.hsqldb;
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.index.IndexStats;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.OrderedHashSet;
@@ -53,7 +54,7 @@ import org.hsqldb.types.Types;
  * Parser for session and management statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ParserCommand extends ParserDDL {
@@ -379,6 +380,7 @@ public class ParserCommand extends ParserDDL {
         String        name      = null;
         int           scope     = 0;
         int           type      = 0;
+        Boolean       colNames  = Boolean.FALSE;
         HsqlName      tableName = null;
         TimestampData timestamp = null;
 
@@ -426,6 +428,13 @@ public class ParserCommand extends ParserDDL {
                         type = Tokens.ALL;
                 }
 
+                if (readIfThis(Tokens.WITH)) {
+                    readThis(Tokens.COLUMN);
+                    readThis(Tokens.NAMES);
+
+                    colNames = Boolean.TRUE;
+                }
+
                 readThis(Tokens.TO);
 
                 scope = Tokens.DATABASE;
@@ -439,6 +448,14 @@ public class ParserCommand extends ParserDDL {
                 tableName = table.getName();
 
                 readThis(Tokens.DATA);
+
+                if (readIfThis(Tokens.WITH)) {
+                    readThis(Tokens.COLUMN);
+                    readThis(Tokens.NAMES);
+
+                    colNames = Boolean.TRUE;
+                }
+
                 readThis(Tokens.TO);
 
                 scope = Tokens.TABLE;
@@ -465,8 +482,8 @@ public class ParserCommand extends ParserDDL {
         HsqlName[] names =
             database.schemaManager.getCatalogAndBaseTableNames();
         Object[] args = new Object[] {
-            name, Integer.valueOf(scope), Integer.valueOf(type), tableName,
-            timestamp
+            name, Integer.valueOf(scope), Integer.valueOf(type), colNames,
+            tableName, timestamp
         };
 
         return new StatementCommand(StatementTypes.DATABASE_SCRIPT, args,
@@ -1173,12 +1190,7 @@ public class ParserCommand extends ParserDDL {
                 }
 
                 if (padSpace == null) {
-                    if (session.isProcessingScript()
-                            && database.getProperties().isVersion18()) {
-                        padSpace = Boolean.FALSE;
-                    } else {
-                        padSpace = Boolean.TRUE;
-                    }
+                    padSpace = Boolean.TRUE;
                 }
 
                 Object[] args = new Object[] {
@@ -2413,32 +2425,58 @@ public class ParserCommand extends ParserDDL {
 
     private Statement compilePerform() {
 
-        Integer type   = Integer.valueOf(1);
-        Integer number = Integer.valueOf(-1);
-
         read();
 
         switch (token.tokenType) {
 
+            /**
+             * PERFORM CHECK TABLE <name> INDEX [AND FIX]
+             * PERFORM CHECK ALL TABLE INDEX [AND FIX]
+             */
             case Tokens.CHECK : {
                 read();
 
+                boolean  isAll     = false;
                 HsqlName tableName = null;
+                Integer  type      = Integer.valueOf(IndexStats.checkRows);
+                Integer  number    = Integer.valueOf(-1);
 
-                readThis(Tokens.TABLE);
-                readThis(Tokens.SPACE);
+                switch (token.tokenType) {
+
+                    case Tokens.ALL : {
+                        read();
+
+                        isAll = true;
+                    }
+
+                    // fall through
+                    case Tokens.TABLE : {
+                        readThis(Tokens.TABLE);
+
+                        if (isAll) {
+                            readThis(Tokens.INDEX);
+                        } else {
+                            tableName = readTableName().getName();
+
+                            readThis(Tokens.INDEX);
+                        }
+                    }
+                }
 
                 if (readIfThis(Tokens.AND)) {
                     readThis("FIX");
 
-                    type = Integer.valueOf(2);
+                    type = Integer.valueOf(IndexStats.fixAll);
                 }
 
                 Object[] args = new Object[] {
                     tableName, type, number
                 };
                 HsqlName[] names =
-                    database.schemaManager.getCatalogAndBaseTableNames();
+                    isAll
+                    ? database.schemaManager.getCatalogAndBaseTableNames()
+                    : database.schemaManager.getCatalogAndBaseTableNames(
+                        tableName);
 
                 return new StatementCommand(StatementTypes.CHECK_INDEX, args,
                                             null, names);
@@ -2636,6 +2674,12 @@ public class ParserCommand extends ParserDDL {
                 type = SchemaObject.DOMAIN;
                 break;
 
+            case Tokens.SEQUENCE :
+                read();
+
+                type = SchemaObject.SEQUENCE;
+                break;
+
             default :
                 throw unexpectedToken();
         }
