@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,7 +48,7 @@ import org.hsqldb.types.Type;
  * Parser for DML statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ParserDML extends ParserDQL {
@@ -62,22 +62,23 @@ public class ParserDML extends ParserDQL {
      */
     StatementDMQL compileInsertStatement(RangeGroup[] rangeGroups) {
 
-        boolean[]     insertColumnCheckList;
-        boolean[]     updateColumnCheckList = null;
-        int[]         insertColumnMap;
-        int[]         updateColumnMap = ValuePool.emptyIntArray;
-        int           colCount;
-        Table         table;
-        RangeVariable range;
-        boolean       overridingUser               = false;
-        boolean       overridingSystem             = false;
-        boolean       assignsToIdentityOrGenerated = false;
-        Token         tableToken;
-        boolean       hasColumnList = false;
-        int           isSpecial     = StatementInsert.isNone;
-        Expression    insertExpressions;
-        Expression[]  updateExpressions = Expression.emptyArray;
-        Expression[]  targets           = null;
+        boolean[]       insertColumnCheckList;
+        boolean[]       updateColumnCheckList = null;
+        int[]           insertColumnMap;
+        int[]           updateColumnMap = ValuePool.emptyIntArray;
+        int             colCount;
+        Table           table;
+        RangeVariable   targetRange;
+        RangeVariable[] rangeVariables;
+        boolean         overridingUser               = false;
+        boolean         overridingSystem             = false;
+        boolean         assignsToIdentityOrGenerated = false;
+        Token           tableToken;
+        boolean         hasColumnList = false;
+        int             isSpecial     = StatementInsert.isNone;
+        Expression      insertExpressions;
+        Expression[]    updateExpressions = Expression.emptyArray;
+        Expression[]    targets           = null;
 
         if (database.sqlSyntaxMys) {
             if (readIfThis(Tokens.REPLACE)) {
@@ -98,12 +99,13 @@ public class ParserDML extends ParserDQL {
             readThis(Tokens.INTO);
         }
 
-        tableToken = getRecordedToken();
-        range      = readRangeVariableForDataChange(StatementTypes.INSERT);
+        tableToken  = getRecordedToken();
+        targetRange = readRangeVariableForDataChange(StatementTypes.INSERT);
 
-        range.resolveRangeTableTypes(session, RangeVariable.emptyArray);
+        targetRange.resolveRangeTableTypes(session, RangeVariable.emptyArray);
 
-        table                 = range.getTable();
+        rangeVariables        = new RangeVariable[]{ targetRange };
+        table                 = targetRange.getTable();
         insertColumnCheckList = null;
         insertColumnMap       = table.getColumnMap();
         colCount              = table.getColumnCount();
@@ -126,6 +128,7 @@ public class ParserDML extends ParserDQL {
                 insertColumnCheckList = table.getNewColumnCheckList();
 
                 StatementDMQL cs = new StatementInsert(session, table,
+                                                       rangeVariables,
                                                        insertColumnMap,
                                                        insertExpressions,
                                                        insertColumnCheckList,
@@ -164,7 +167,8 @@ public class ParserDML extends ParserDQL {
                     OrderedHashSet columnNames = new OrderedHashSet();
                     boolean        withPrefix  = database.sqlSyntaxOra;
 
-                    readSimpleColumnNames(columnNames, range, withPrefix);
+                    readSimpleColumnNames(columnNames, targetRange,
+                                          withPrefix);
                     readThis(Tokens.CLOSEBRACKET);
 
                     colCount        = columnNames.size();
@@ -245,12 +249,16 @@ public class ParserDML extends ParserDQL {
                             assignsToIdentityOrGenerated = true;
 
                             if (e.getType() != OpTypes.DEFAULT) {
-                                if (baseTable.identitySequence.isAlways()) {
-                                    if (overridingUser) {
-                                        rowArgs[i] = new ExpressionColumn(
-                                            OpTypes.DEFAULT);
-                                    } else {
-                                        throw Error.error(ErrorCode.X_42543);
+                                if (overridingUser) {
+                                    rowArgs[i] =
+                                        new ExpressionColumn(OpTypes.DEFAULT);
+                                } else if (overridingSystem) {
+
+                                    // user value allowed
+                                } else {
+                                    if (baseTable.identitySequence
+                                            .isAlways()) {
+                                        throw Error.error(ErrorCode.X_42542);
                                     }
                                 }
                             }
@@ -266,7 +274,7 @@ public class ParserDML extends ParserDQL {
                                     rowArgs[i] =
                                         new ExpressionColumn(OpTypes.DEFAULT);
                                 } else {
-                                    throw Error.error(ErrorCode.X_42541);
+                                    throw Error.error(ErrorCode.X_42543);
                                 }
                             }
                         } else {
@@ -275,13 +283,14 @@ public class ParserDML extends ParserDQL {
                         }
 
                         if (e.isUnresolvedParam()) {
-                            e.setAttributesAsColumn(column, true);
+                            e.setAttributesAsColumn(column);
                         }
                     }
                 }
 
-                if (!assignsToIdentityOrGenerated && overridingUser) {
-                    throw unexpectedTokenRequire(Tokens.T_OVERRIDING);
+                if (!assignsToIdentityOrGenerated
+                        && (overridingUser || overridingSystem)) {
+                    throw unexpectedToken(Tokens.T_OVERRIDING);
                 }
 
                 if (!hasColumnList) {
@@ -295,18 +304,18 @@ public class ParserDML extends ParserDQL {
                     readThis(Tokens.KEY);
                     readThis(Tokens.UPDATE);
 
-                    OrderedHashSet  targetSet    = new OrderedHashSet();
-                    LongDeque       colIndexList = new LongDeque();
-                    HsqlArrayList   exprList     = new HsqlArrayList();
-                    RangeVariable[] rangeVariables;
-                    RangeGroup      rangeGroup;
+                    OrderedHashSet targetSet    = new OrderedHashSet();
+                    LongDeque      colIndexList = new LongDeque();
+                    HsqlArrayList  exprList     = new HsqlArrayList();
+                    RangeGroup rangeGroup =
+                        new RangeGroupSimple(rangeVariables, false);
+                    RangeVariable valueRange =
+                        new RangeVariable(targetRange.getTable(), 2);
 
-                    rangeVariables = new RangeVariable[]{ range };
-                    rangeGroup = new RangeGroupSimple(rangeVariables, false);
-                    isSpecial      = StatementInsert.isUpdate;
+                    isSpecial = StatementInsert.isUpdate;
 
-                    readSetClauseList(rangeVariables, targetSet, colIndexList,
-                                      exprList);
+                    readOnDuplicateClauseList(rangeVariables, targetSet,
+                                              colIndexList, exprList);
 
                     updateColumnMap = new int[colIndexList.size()];
 
@@ -328,10 +337,12 @@ public class ParserDML extends ParserDQL {
                     exprList.toArray(updateExpressions);
                     resolveUpdateExpressions(table, rangeGroup,
                                              updateColumnMap,
-                                             updateExpressions, rangeGroups);
+                                             updateExpressions, rangeGroups,
+                                             valueRange);
                 }
 
                 StatementDMQL cs = new StatementInsert(session, table,
+                                                       rangeVariables,
                                                        insertColumnMap,
                                                        insertExpressions,
                                                        insertColumnCheckList,
@@ -381,7 +392,7 @@ public class ParserDML extends ParserDQL {
                 overrideIndex = enforcedDefaultIndex;
             }
         } else if (overridingUser || overridingSystem) {
-            throw unexpectedTokenRequire(Tokens.T_OVERRIDING);
+            throw unexpectedToken(Tokens.T_OVERRIDING);
         }
 
         Type[] types = new Type[insertColumnMap.length];
@@ -409,18 +420,18 @@ public class ParserDML extends ParserDQL {
             readThis(Tokens.KEY);
             readThis(Tokens.UPDATE);
 
-            OrderedHashSet  targetSet    = new OrderedHashSet();
-            LongDeque       colIndexList = new LongDeque();
-            HsqlArrayList   exprList     = new HsqlArrayList();
-            RangeVariable[] rangeVariables;
-            RangeGroup      rangeGroup;
+            OrderedHashSet targetSet    = new OrderedHashSet();
+            LongDeque      colIndexList = new LongDeque();
+            HsqlArrayList  exprList     = new HsqlArrayList();
+            RangeGroup rangeGroup = new RangeGroupSimple(rangeVariables,
+                false);
+            RangeVariable valueRange =
+                new RangeVariable(targetRange.getTable(), 2);
 
-            rangeVariables = new RangeVariable[]{ range };
-            rangeGroup     = new RangeGroupSimple(rangeVariables, false);
-            isSpecial      = StatementInsert.isUpdate;
+            isSpecial = StatementInsert.isUpdate;
 
-            readSetClauseList(rangeVariables, targetSet, colIndexList,
-                              exprList);
+            readOnDuplicateClauseList(rangeVariables, targetSet, colIndexList,
+                                      exprList);
 
             updateColumnMap = new int[colIndexList.size()];
 
@@ -439,10 +450,11 @@ public class ParserDML extends ParserDQL {
 
             exprList.toArray(updateExpressions);
             resolveUpdateExpressions(table, rangeGroup, updateColumnMap,
-                                     updateExpressions, rangeGroups);
+                                     updateExpressions, rangeGroups,
+                                     valueRange);
         }
 
-        StatementDMQL cs = new StatementInsert(session, table,
+        StatementDMQL cs = new StatementInsert(session, table, rangeVariables,
                                                insertColumnMap,
                                                insertColumnCheckList,
                                                queryExpression,
@@ -464,7 +476,7 @@ public class ParserDML extends ParserDQL {
             for (int j = 0; j < list.length; j++) {
                 if (list[j].isUnresolvedParam()) {
                     list[j].setAttributesAsColumn(
-                        table.getColumn(columnMap[j]), true);
+                        table.getColumn(columnMap[j]));
                 }
             }
         }
@@ -538,7 +550,7 @@ public class ParserDML extends ParserDQL {
                 } else {
                     readThis(Tokens.CURRENT_TIMESTAMP);
 
-                    timestamp = session.getTransactionSystemTimestamp();
+                    timestamp = session.getTransactionUTC();
                 }
 
                 break;
@@ -701,7 +713,8 @@ public class ParserDML extends ParserDQL {
                                                     : table.getBaseTable();
 
         readThis(Tokens.SET);
-        readSetClauseList(rangeVariables, targetSet, colIndexList, exprList);
+        readSetClauseList(rangeGroups, rangeVariables, targetSet,
+                          colIndexList, exprList);
 
         columnMap = new int[colIndexList.size()];
 
@@ -736,7 +749,7 @@ public class ParserDML extends ParserDQL {
         }
 
         resolveUpdateExpressions(table, rangeGroup, columnMap,
-                                 updateExpressions, rangeGroups);
+                                 updateExpressions, rangeGroups, null);
 
         if (table != baseTable) {
             QuerySpecification baseSelect =
@@ -825,7 +838,8 @@ public class ParserDML extends ParserDQL {
     void resolveUpdateExpressions(Table targetTable, RangeGroup rangeGroup,
                                   int[] columnMap,
                                   Expression[] colExpressions,
-                                  RangeGroup[] rangeGroups) {
+                                  RangeGroup[] rangeGroups,
+                                  RangeVariable valuesRange) {
 
         HsqlList unresolved           = null;
         int      enforcedDefaultIndex = -1;
@@ -858,7 +872,7 @@ public class ParserDML extends ParserDQL {
 
                     if (e.isUnresolvedParam()) {
                         e.setAttributesAsColumn(
-                            targetTable.getColumn(columnMap[i]), true);
+                            targetTable.getColumn(columnMap[i]));
                     } else if (e.getType() == OpTypes.DEFAULT) {
 
                         //
@@ -898,7 +912,7 @@ public class ParserDML extends ParserDQL {
 
                 if (e.isUnresolvedParam()) {
                     e.setAttributesAsColumn(
-                        targetTable.getColumn(columnMap[i]), true);
+                        targetTable.getColumn(columnMap[i]));
                 } else if (e.getType() == OpTypes.DEFAULT) {
 
                     //
@@ -906,6 +920,19 @@ public class ParserDML extends ParserDQL {
                     unresolved = expr.resolveColumnReferences(session,
                             rangeGroup, rangeGroups, null);
 
+                    if (valuesRange != null && unresolved != null) {
+                        for (int j = unresolved.size() - 1; j >= 0; j--) {
+                            ExpressionColumn col =
+                                (ExpressionColumn) unresolved.get(j);
+
+                            col.resolveColumnReference(valuesRange, false);
+
+                            if (col.rangeVariable != null) {
+                                unresolved.remove(j);
+                            }
+                        }
+                    }
+
                     ExpressionColumn.checkColumnsResolved(unresolved);
                     e.resolveTypes(session, null);
                 }
@@ -915,7 +942,8 @@ public class ParserDML extends ParserDQL {
         }
     }
 
-    void readSetClauseList(RangeVariable[] rangeVars, OrderedHashSet targets,
+    void readSetClauseList(RangeGroup[] rangeGroups,
+                           RangeVariable[] rangeVars, OrderedHashSet targets,
                            LongDeque colIndexList, HsqlArrayList expressions) {
 
         while (true) {
@@ -951,6 +979,7 @@ public class ParserDML extends ParserDQL {
 
             if (token.tokenType == Tokens.SELECT) {
                 rewind(position);
+                compileContext.setOuterRanges(rangeGroups);
 
                 TableDerived td = XreadSubqueryTableBody(OpTypes.ROW_SUBQUERY);
                 QueryExpression qe = td.getQueryExpression();
@@ -978,13 +1007,7 @@ public class ParserDML extends ParserDQL {
                 rewind(position);
             }
 
-            boolean values = false;
-
-            if (database.sqlSyntaxMys) {
-                values = readIfThis(Tokens.VALUES);
-            }
-
-            if (degree > 1 || values) {
+            if (degree > 1) {
                 readThis(Tokens.OPENBRACKET);
 
                 Expression e = readRow();
@@ -1015,6 +1038,37 @@ public class ParserDML extends ParserDQL {
         }
     }
 
+    void readOnDuplicateClauseList(RangeVariable[] rangeVars,
+                                   OrderedHashSet targets,
+                                   LongDeque colIndexList,
+                                   HsqlArrayList expressions) {
+
+        while (true) {
+            Expression target = XreadTargetSpecification(rangeVars,
+                colIndexList);
+
+            if (!targets.add(target)) {
+                ColumnSchema col = target.getColumn();
+
+                throw Error.error(ErrorCode.X_42579, col.getName().name);
+            }
+
+            readThis(Tokens.EQUALS_OP);
+
+            Expression e = XreadValueExpressionOnDuplicate();
+
+            expressions.add(e);
+
+            if (token.tokenType == Tokens.COMMA) {
+                read();
+
+                continue;
+            }
+
+            break;
+        }
+    }
+
     void readGetClauseList(RangeVariable[] rangeVars, OrderedHashSet targets,
                            LongDeque colIndexList, HsqlArrayList expressions) {
 
@@ -1205,7 +1259,7 @@ public class ParserDML extends ParserDQL {
             }
 
             resolveUpdateExpressions(table, fullRangeGroup, updateColumnMap,
-                                     updateExpressions, rangeGroups);
+                                     updateExpressions, rangeGroups, null);
         }
 
         HsqlList unresolved = null;
@@ -1304,8 +1358,9 @@ public class ParserDML extends ParserDQL {
                 conditions[1] = condition;
 
                 readThis(Tokens.SET);
-                readSetClauseList(targetRangeVars, updateTargetSet,
-                                  updateColIndexList, updateExpressions);
+                readSetClauseList(rangeGroups, targetRangeVars,
+                                  updateTargetSet, updateColIndexList,
+                                  updateExpressions);
             } else {
                 if (conditions[2] != null) {
                     throw Error.error(ErrorCode.X_42547);
@@ -1471,8 +1526,7 @@ public class ParserDML extends ParserDQL {
             Expression e = arguments[i];
 
             if (e.isUnresolvedParam()) {
-                e.setAttributesAsColumn(routine.getParameter(i),
-                                        routine.getParameter(i).isWriteable());
+                e.setAttributesAsColumn(routine.getParameter(i));
             } else {
                 int paramMode = routine.getParameter(i).getParameterMode();
 
