@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.types.Type;
  * Parser for SQL stored procedures and functions - PSM
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class ParserRoutine extends ParserTable {
@@ -204,7 +204,8 @@ public class ParserRoutine extends ParserTable {
         HsqlArrayList  exprList     = new HsqlArrayList();
         LongDeque      colIndexList = new LongDeque();
 
-        readSetClauseList(rangeVars, targetSet, colIndexList, exprList);
+        readSetClauseList(rangeGroups, rangeVars, targetSet, colIndexList,
+                          exprList);
 
         if (exprList.size() > 1) {
             throw Error.parseError(ErrorCode.X_42602, null,
@@ -269,7 +270,8 @@ public class ParserRoutine extends ParserTable {
             rangeGroups[0].getRangeVariables()[TriggerDef.NEW_ROW] };
         LongDeque colIndexList = new LongDeque();
 
-        readSetClauseList(targetRangeVars, targetSet, colIndexList, exprList);
+        readSetClauseList(rangeGroups, targetRangeVars, targetSet,
+                          colIndexList, exprList);
 
         columnMap = new int[colIndexList.size()];
 
@@ -287,7 +289,7 @@ public class ParserRoutine extends ParserTable {
 
         exprList.toArray(updateExpressions);
         resolveUpdateExpressions(table, RangeGroup.emptyGroup, columnMap,
-                                 updateExpressions, rangeGroups);
+                                 updateExpressions, rangeGroups, null);
 
         StatementDMQL cs = new StatementSet(session, targets, table,
                                             rangeGroups[0].getRangeVariables(),
@@ -399,7 +401,8 @@ public class ParserRoutine extends ParserTable {
                 read();
 
                 TableDerived table =
-                    new TableDerived(database, SqlInvariants.MODULE_HSQLNAME,
+                    new TableDerived(database,
+                                     SqlInvariants.SYSTEM_SCHEMA_HSQLNAME,
                                      TableBase.FUNCTION_TABLE);
 
                 readTableDefinition(routine, table);
@@ -762,7 +765,8 @@ public class ParserRoutine extends ParserTable {
 
     void readRoutineSQLBody(Routine routine) {
 
-        startRecording();
+        Recorder recorder = startRecording();
+
         session.sessionContext.pushRoutineTables();
 
         try {
@@ -773,8 +777,7 @@ public class ParserRoutine extends ParserTable {
                 throw unexpectedToken();
             }
 
-            Token[] tokenisedStatement = getRecordedStatement();
-            String  sql                = Token.getSQL(tokenisedStatement);
+            String sql = recorder.getSQL();
 
             statement.setSQL(sql);
             routine.setProcedure(statement);
@@ -890,7 +893,7 @@ public class ParserRoutine extends ParserTable {
 
             HsqlName name = readNewSchemaObjectName(SchemaObject.TABLE, false);
 
-            name.schema = SqlInvariants.MODULE_HSQLNAME;
+            name.schema = SqlInvariants.SYSTEM_SCHEMA_HSQLNAME;
 
             Table table = new Table(database, name, TableBase.TEMP_TABLE);
 
@@ -2518,8 +2521,7 @@ public class ParserRoutine extends ParserTable {
         session.sessionContext.pushRoutineTables();
 
         try {
-            startRecording();
-
+            Recorder recorder = startRecording();
             StatementCompound parent =
                 new StatementCompound(StatementTypes.BEGIN_END, null, null);
 
@@ -2532,8 +2534,7 @@ public class ParserRoutine extends ParserTable {
                 throw unexpectedToken();
             }
 
-            Token[] tokenisedStatement = getRecordedStatement();
-            String  sql                = Token.getSQL(tokenisedStatement);
+            String sql = recorder.getSQL();
 
             statement.setSQL(sql);
             routine.setProcedure(statement);
