@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,6 +46,7 @@ import org.hsqldb.lib.Set;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.navigator.RowSetNavigatorData;
 import org.hsqldb.navigator.RowSetNavigatorDataTable;
+import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.types.Type;
@@ -55,7 +56,7 @@ import org.hsqldb.types.Types;
  * Implementation of an SQL query expression
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.1
  * @since 1.9.0
  */
 
@@ -630,27 +631,27 @@ public class QueryExpression implements RangeGroup {
         switch (unionType) {
 
             case UNION :
-                navigator.union(session, rightNavigator);
+                navigator.union(rightNavigator);
                 break;
 
             case UNION_ALL :
-                navigator.unionAll(session, rightNavigator);
+                navigator.unionAll(rightNavigator);
                 break;
 
             case INTERSECT :
-                navigator.intersect(session, rightNavigator);
+                navigator.intersect(rightNavigator);
                 break;
 
             case INTERSECT_ALL :
-                navigator.intersectAll(session, rightNavigator);
+                navigator.intersectAll(rightNavigator);
                 break;
 
             case EXCEPT :
-                navigator.except(session, rightNavigator);
+                navigator.except(rightNavigator);
                 break;
 
             case EXCEPT_ALL :
-                navigator.exceptAll(session, rightNavigator);
+                navigator.exceptAll(rightNavigator);
                 break;
 
             default :
@@ -658,7 +659,7 @@ public class QueryExpression implements RangeGroup {
         }
 
         if (sortAndSlice.hasOrder()) {
-            navigator.sortOrderUnion(session, sortAndSlice);
+            navigator.sortOrderUnion(sortAndSlice);
         }
 
         if (sortAndSlice.hasLimit()) {
@@ -674,35 +675,30 @@ public class QueryExpression implements RangeGroup {
 
     Result getResultRecursive(Session session) {
 
-        Result tempResult;
+        RowSetNavigatorData resultNav = new RowSetNavigatorData(session, this);
 
         recursiveTable.materialise(session);
-
-        RowSetNavigatorData recNav = recursiveTable.getNavigator(session);
-        Result              result = Result.newResult(recNav);
-
-        result.metaData = resultMetaData;
+        resultNav.unionAll(recursiveTable.getNavigator(session));
 
         for (int round = 0; ; round++) {
-            tempResult = rightQueryExpression.getResult(session, 0);
-
+            Result currentResult = rightQueryExpression.getResult(session, 0);
             RowSetNavigatorData tempNavigator =
-                (RowSetNavigatorData) tempResult.getNavigator();
+                (RowSetNavigatorData) currentResult.getNavigator();
 
             if (tempNavigator.isEmpty()) {
                 break;
             }
 
-            int startSize = recNav.getSize();
+            int startSize = resultNav.getSize();
 
             switch (unionType) {
 
                 case UNION :
-                    recNav.union(session, tempNavigator);
+                    resultNav.union(tempNavigator);
                     break;
 
                 case UNION_ALL :
-                    recNav.unionAll(session, tempNavigator);
+                    resultNav.unionAll(tempNavigator);
                     break;
 
                 default :
@@ -710,15 +706,25 @@ public class QueryExpression implements RangeGroup {
                                              "QueryExpression");
             }
 
-            if (startSize == recNav.getSize()) {
+            if (startSize == resultNav.getSize()) {
                 break;
             }
 
             if (round > 256) {
                 throw Error.error(ErrorCode.X_22522);
             }
+
+            PersistentStore store = recursiveTable.getRowStore(session);
+
+            store.removeAll();
+            currentResult.getNavigator().reset();
+            recursiveTable.insertSys(session, store, currentResult);
         }
 
+        Result result = Result.newResult(resultNav);
+
+        result.metaData = resultMetaData;
+
         return result;
     }
 
@@ -1027,6 +1033,7 @@ public class QueryExpression implements RangeGroup {
                 ColumnSchema column =
                     (ColumnSchema) columns.get(unionColumnMap[i]);
                 String name = (String) columns.getKey(unionColumnMap[i]);
+
                 list.add(name, column);
             }
 
@@ -1114,7 +1121,8 @@ public class QueryExpression implements RangeGroup {
                        other.rightQueryExpression));
     }
 
-    public void replaceColumnReferences(Session session, RangeVariable range, Expression[] list) {
+    public void replaceColumnReferences(Session session, RangeVariable range,
+                                        Expression[] list) {
         leftQueryExpression.replaceColumnReferences(session, range, list);
         rightQueryExpression.replaceColumnReferences(session, range, list);
     }
