@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,6 +45,7 @@ import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.IntValueHashMap;
+import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.OrderedIntHashSet;
 import org.hsqldb.lib.Set;
@@ -64,7 +65,7 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class QuerySpecification extends QueryExpression {
@@ -74,6 +75,8 @@ public class QuerySpecification extends QueryExpression {
     public boolean        isDistinctSelect;
     public boolean        isAggregated;
     public boolean        isGrouped;
+    public boolean        isGroupingSets;
+    boolean               isDistinctGroups;
     public boolean        isOrderSensitive;
     public boolean        isSimpleDistinct;
     RangeVariable[]       rangeVariables;
@@ -82,10 +85,10 @@ public class QuerySpecification extends QueryExpression {
     int                   endInnerRange   = -1;
     Expression            queryCondition;
     Expression            checkQueryCondition;
-    private Expression    havingCondition;
     Expression            rowExpression;
     Expression[]          exprColumns;
     HsqlArrayList         exprColumnList;
+    GroupSet              groupSet;
     private int           groupByColumnCount;    // columns in 'group by'
     private int           havingColumnCount;     // columns in 'having' (0 or 1)
     public int            indexLimitVisible;
@@ -194,7 +197,7 @@ public class QuerySpecification extends QueryExpression {
         return rangeVariableList.size();
     }
 
-    // range variable sub queries are resolves fully
+    // range variable sub queries are resolved fully
     private void resolveRangeVariables(Session session,
                                        RangeGroup[] rangeGroups) {
 
@@ -259,24 +262,119 @@ public class QuerySpecification extends QueryExpression {
         isDistinctSelect = true;
     }
 
-    void addGroupByColumnExpression(Expression e) {
+    void setDistinctGroups() {
+        isDistinctGroups = true;
+    }
 
-        if (e.getType() == OpTypes.ROW) {
-            throw Error.error(ErrorCode.X_42564);
+    void addGroupingSets(Expression[] groupingExpressions) {
+        groupSet = new GroupSet(groupingExpressions, isDistinctGroups);
+    }
+
+    /**
+     * fully resolve all group by columns
+     * add each group by expression to exprColumnList
+     */
+    void resolveColumnReferencesInGroupingSets(Session session,
+            RangeGroup[] rangeGroups) {
+
+        if (groupSet == null) {
+            return;
         }
 
-        exprColumnList.add(e);
+        Expression[] groupExpressions = groupSet.groupExpressions;
 
-        isGrouped = true;
+        for (int i = 0; i < groupExpressions.length; i++) {
+            Expression e = groupExpressions[i];
 
-        groupByColumnCount++;
+            tempSet.clear();
+            e.resolveColumnReferences(session, this, rangeVariables.length,
+                                      rangeGroups, tempSet, false);
+
+            if (!tempSet.isEmpty()) {
+                if (session.database.sqlEnforceRefs) {
+                    ExpressionColumn.checkColumnsResolved(tempSet);
+
+                    continue;
+                }
+
+                // when a column reference does not resolve to range variables,
+                // resolve to aliases in select list
+                // this is non-standard and probably should be allowed only
+                // for basic group by lists
+                Expression[] exprArray = new Expression[exprColumnList.size()];
+
+                exprColumnList.toArray(exprArray);
+
+                Expression resolved = e.replaceAliasInOrderBy(session,
+                    exprArray, indexLimitVisible);
+
+                if (resolved != e) {
+                    groupExpressions[i] = resolved;
+                }
+
+                tempSet.clear();
+                resolved.resolveColumnReferences(session, this,
+                                                 rangeVariables.length,
+                                                 RangeGroup.emptyArray,
+                                                 tempSet, false);
+                ExpressionColumn.checkColumnsResolved(tempSet);
+            }
+        }
+
+        tempSet.clear();
+        addGroupingExpressions(groupExpressions, tempSet);
+
+        Iterator it = tempSet.iterator();
+
+        while (it.hasNext()) {
+            Expression e = (Expression) it.next();
+
+            if (e.getType() == OpTypes.ROW) {
+                throw Error.error(ErrorCode.X_42564);
+            }
+
+            e.resultTableColumnIndex = indexLimitVisible + groupByColumnCount;    //
+
+            exprColumnList.add(e.resultTableColumnIndex, e);
+
+            groupByColumnCount++;
+        }
+
+        groupSet.process();
+    }
+
+    /**
+     * collect all expressions in an extended group by expression
+     * set the flags for grouping set and group by
+     */
+    void addGroupingExpressions(Expression[] nodes, OrderedHashSet set) {
+
+        for (int i = 0; i < nodes.length; i++) {
+            Expression e = nodes[i];
+
+            if (e.groupingType != OpTypes.NONE) {
+                isGroupingSets = true;
+            }
+
+            if (e.opType == OpTypes.ROW || e.opType == OpTypes.VALUELIST) {
+                for (int j = 0; j < e.nodes.length; j++) {
+                    addGroupingExpressions(e.nodes, set);
+                }
+            } else {
+                if (e.opType == OpTypes.NONE) {
+                    continue;
+                }
+
+                nodes[i]  = (Expression) set.getOrAdd(e);
+                isGrouped = true;
+            }
+        }
     }
 
     void addHavingExpression(Expression e) {
 
         exprColumnList.add(e);
 
-        havingCondition   = e;
         havingColumnCount = 1;
     }
 
@@ -294,6 +392,10 @@ public class QuerySpecification extends QueryExpression {
 
         resolveRangeVariables(session, rangeGroups);
         resolveColumnReferencesForAsterisk();
+
+        // must be after asterisk expansion
+        resolveColumnReferencesInGroupingSets(session, rangeGroups);
+        setColumnIndexes();
         finaliseColumns();
         resolveColumnReferences(session, rangeGroups);
         setReferenceableColumns();
@@ -377,6 +479,7 @@ public class QuerySpecification extends QueryExpression {
         }
 
         resolveGroups();
+        resolveGroupingSets();
 
         for (int i = 0; i < unionColumnTypes.length; i++) {
             Type type = unionColumnTypes[i];
@@ -506,11 +609,6 @@ public class QuerySpecification extends QueryExpression {
             }
         }
 
-        for (int i = indexLimitVisible; i < indexStartHaving; i++) {
-            exprColumns[i] = resolveColumnReferencesInGroupBy(session,
-                    exprColumns[i]);
-        }
-
         for (int i = indexStartHaving; i < indexStartOrderBy; i++) {
             resolveColumnReferencesAndAllocate(session, exprColumns[i],
                                                rangeVariables.length,
@@ -680,16 +778,7 @@ public class QuerySpecification extends QueryExpression {
 
                 if (resolved) {
                     if (e.isSelfAggregate()) {
-                        if (aggregateSet == null) {
-                            aggregateSet = new ArrayListIdentity();
-                        }
-
-                        aggregateSet.add(e);
-
-                        isAggregated = true;
-
-                        expression.setAggregate();
-                        e.setCorrelatedReferences(this);
+                        addAggregateToSet(expression, e);
                     }
 
                     if (resolvedSubqueryExpressions == null) {
@@ -708,35 +797,18 @@ public class QuerySpecification extends QueryExpression {
         }
     }
 
-    private Expression resolveColumnReferencesInGroupBy(Session session,
-            Expression expression) {
+    private void addAggregateToSet(Expression parent, Expression e) {
 
-        if (expression == null) {
-            return null;
+        if (aggregateSet == null) {
+            aggregateSet = new ArrayListIdentity();
         }
 
-        HsqlList list = expression.resolveColumnReferences(session, this,
-            rangeVariables.length, RangeGroup.emptyArray, null, false);
+        aggregateSet.add(e);
 
-        if (list != null) {
+        isAggregated = true;
 
-            // if not resolved, resolve as simple alias
-            if (expression.getType() == OpTypes.COLUMN) {
-                Expression resolved = expression.replaceAliasInOrderBy(session,
-                    exprColumns, indexLimitVisible);
-
-                if (resolved != expression) {
-                    return resolved;
-                }
-            }
-
-            // resolve and allocate to throw exception
-            resolveColumnReferencesAndAllocate(session, expression,
-                                               rangeVariables.length,
-                                               RangeGroup.emptyArray, false);
-        }
-
-        return expression;
+        parent.setAggregate();
+        e.setCorrelatedReferences(this);
     }
 
     private HashSet getAllNamedJoinColumns() {
@@ -852,7 +924,13 @@ public class QuerySpecification extends QueryExpression {
         e.nodes = nodes;
     }
 
-    private void finaliseColumns() {
+    private void setColumnIndexes() {
+
+        for (int i = 0; i < indexLimitVisible; i++) {
+            Expression e = (Expression) exprColumnList.get(i);
+
+            e.queryTableColumnIndex = i;
+        }
 
         indexLimitRowId   = indexLimitVisible;
         indexStartHaving  = indexLimitRowId + groupByColumnCount;
@@ -860,16 +938,16 @@ public class QuerySpecification extends QueryExpression {
         indexStartAggregates = indexStartOrderBy
                                + sortAndSlice.getOrderLength();
         indexLimitData = indexLimitExpressions = indexStartAggregates;
-        exprColumns    = new Expression[indexLimitExpressions];
+    }
+
+    private void finaliseColumns() {
+
+        exprColumns = new Expression[indexLimitExpressions];
 
         exprColumnList.toArray(exprColumns);
 
         exprColumnList = null;
 
-        for (int i = 0; i < indexLimitVisible; i++) {
-            exprColumns[i].queryTableColumnIndex = i;
-        }
-
         if (sortAndSlice.hasOrder()) {
             for (int i = 0; i < sortAndSlice.getOrderLength(); i++) {
                 exprColumns[indexStartOrderBy + i] =
@@ -880,12 +958,12 @@ public class QuerySpecification extends QueryExpression {
         rowExpression = new Expression(OpTypes.ROW, exprColumns);
     }
 
-    private void replaceColumnIndexInOrderBy(Expression orderBy) {
+    private int replaceColumnIndexInOrderBy(Expression orderBy) {
 
         Expression e = orderBy.getLeftNode();
 
         if (e.getType() != OpTypes.VALUE) {
-            return;
+            return -1;
         }
 
         Type type = e.getDataType();
@@ -896,7 +974,7 @@ public class QuerySpecification extends QueryExpression {
             if (0 < i && i <= indexLimitVisible) {
                 orderBy.setLeftNode(exprColumns[i - 1]);
 
-                return;
+                return i;
             }
         }
 
@@ -914,10 +992,6 @@ public class QuerySpecification extends QueryExpression {
             set = queryCondition.collectRangeVariables(rangeVars, set);
         }
 
-        if (havingCondition != null) {
-            set = havingCondition.collectRangeVariables(rangeVars, set);
-        }
-
         return set;
     }
 
@@ -931,10 +1005,6 @@ public class QuerySpecification extends QueryExpression {
             set = queryCondition.collectRangeVariables(set);
         }
 
-        if (havingCondition != null) {
-            set = havingCondition.collectRangeVariables(set);
-        }
-
         return set;
     }
 
@@ -982,10 +1052,9 @@ public class QuerySpecification extends QueryExpression {
             }
         }
 
-        if (havingCondition != null) {
-            havingCondition.resolveTypes(session, null);
-
-            if (havingCondition.getDataType() != Type.SQL_BOOLEAN) {
+        if (havingColumnCount != 0) {
+            if (exprColumns[indexStartHaving].getDataType()
+                    != Type.SQL_BOOLEAN) {
                 throw Error.error(ErrorCode.X_42568);
             }
         }
@@ -1288,7 +1357,7 @@ public class QuerySpecification extends QueryExpression {
 
         tempSet.clear();
 
-        if (havingCondition != null) {
+        if (havingColumnCount != 0) {
             if (unresolvedExpressions != null) {
                 tempSet.addAll(unresolvedExpressions);
             }
@@ -1302,9 +1371,8 @@ public class QuerySpecification extends QueryExpression {
                 tempSet.addAll(extraSet);
             }
 
-            if (!havingCondition.isComposedOf(
-                    tempSet, outerRanges,
-                    OpTypes.subqueryAggregateExpressionSet)) {
+            if (!exprColumns[indexStartHaving].isComposedOf(tempSet,
+                    outerRanges, OpTypes.subqueryAggregateExpressionSet)) {
                 throw Error.error(ErrorCode.X_42573);
             }
 
@@ -1348,10 +1416,6 @@ public class QuerySpecification extends QueryExpression {
             }
         }
 
-        if (!isAggregated) {
-            return;
-        }
-
         OrderedHashSet expressions = new OrderedHashSet();
 
         for (int i = indexStartAggregates; i < indexLimitExpressions; i++) {
@@ -1374,6 +1438,10 @@ public class QuerySpecification extends QueryExpression {
             expressions.add(e);
         }
 
+        if (!isAggregated) {
+            return;
+        }
+
         // order by with aggregate
         int orderCount = sortAndSlice.getOrderLength();
 
@@ -1412,6 +1480,40 @@ public class QuerySpecification extends QueryExpression {
         }
     }
 
+    /**
+     * replace all expressions in select list and having condition with
+     * pointers to columns for group by and aggregate expressions
+     */
+    public void resolveGroupingSets() {
+
+        if (!isGrouped) {
+            return;
+        }
+
+        tempSet.clear();
+
+        for (int i = indexLimitVisible; i < indexStartHaving; i++) {
+            tempSet.add(exprColumns[i]);
+        }
+
+        for (int i = indexStartAggregates; i < indexLimitExpressions; i++) {
+            tempSet.add(exprColumns[i]);
+        }
+
+        if (isGroupingSets) {
+            for (int i = 0; i < indexLimitVisible; i++) {
+                exprColumns[i] = exprColumns[i].replaceExpressions(tempSet,
+                        resultRangePosition);
+            }
+        }
+
+        for (int i = indexStartHaving;
+                i < indexStartHaving + havingColumnCount; i++) {
+            exprColumns[i] = exprColumns[i].replaceExpressions(tempSet,
+                    resultRangePosition);
+        }
+    }
+
     boolean resolveForGroupBy(HsqlList unresolvedSet) {
 
         for (int i = indexLimitVisible;
@@ -1469,11 +1571,11 @@ public class QuerySpecification extends QueryExpression {
         RowSetNavigatorData navigator = (RowSetNavigatorData) r.getNavigator();
 
         if (isDistinctSelect) {
-            navigator.removeDuplicates(session);
+            navigator.removeDuplicates();
         }
 
         if (sortAndSlice.hasOrder()) {
-            navigator.sortOrder(session);
+            navigator.sortOrder();
         }
 
         if (limits != SortAndSlice.defaultLimits
@@ -1590,13 +1692,21 @@ public class QuerySpecification extends QueryExpression {
 
             session.sessionData.startRowProcessing();
 
-            Object[] data = new Object[indexLimitData];
+            Object[] data  = new Object[indexLimitData];
+            int      start = 0;
 
-            for (int i = 0; i < indexStartAggregates; i++) {
+            if (isGroupingSets) {
+                start = indexLimitVisible;
+            }
+
+            for (int i = start; i < indexStartAggregates; i++) {
                 if (isAggregated && aggregateCheck[i]) {
                     continue;
                 } else {
-                    data[i] = exprColumns[i].getValue(session);
+                    if (havingColumnCount > 0 && i == indexStartHaving) {}
+                    else {
+                        data[i] = exprColumns[i].getValue(session);
+                    }
                 }
             }
 
@@ -1649,7 +1759,7 @@ public class QuerySpecification extends QueryExpression {
             int rowCount = navigator.getSize();
 
             if (rowCount == session.resultMaxMemoryRows && !isAggregated
-                    && !isSingleMemoryTable) {
+                    && !isSingleMemoryTable && !isGroupingSets) {
                 navigator = new RowSetNavigatorDataTable(session, this,
                         navigator);
 
@@ -1673,10 +1783,109 @@ public class QuerySpecification extends QueryExpression {
             rangeIterators[i].reset();
         }
 
-        if (!resultGrouped && !isAggregated) {
+        if (!isGroupingSets && !isAggregated && havingColumnCount == 0) {
             return result;
         }
 
+        session.sessionContext.setRangeIterator(navigator);
+
+        if (isGroupingSets) {
+            session.sessionContext.setGroupSet(groupSet);
+
+            Iterator   groupsIterator = groupSet.getIterator();
+            Object[][] baseResult     = navigator.getDataTable();
+            int        baseResultSize = navigator.getSize();
+
+            navigator.clear();
+
+            if (groupSet.nullSets != 0) {
+                Object[] data = new Object[indexLimitData];
+
+                for (int i = 0; i < indexStartAggregates; i++) {
+                    data[i] = exprColumns[i].getValue(session);
+                }
+
+                navigator.add(data);
+                navigator.next();
+
+                if (isAggregated) {
+                    for (int i = 0; i < baseResultSize; i++) {
+                        Object[] row = baseResult[i];
+
+                        for (int j = indexStartAggregates;
+                                j < indexLimitExpressions; j++) {
+                            data[j] =
+                                exprColumns[j].updateAggregatingValue(session,
+                                    (SetFunction) data[j],
+                                    (SetFunction) row[j]);
+                        }
+                    }
+
+                    for (int i = indexStartAggregates;
+                            i < indexLimitExpressions; i++) {
+                        data[i] = exprColumns[i].getAggregatedValue(session,
+                                (SetFunction) data[i]);
+                    }
+
+                    for (int i = 0; i < indexStartAggregates; i++) {
+                        if (aggregateCheck[i]) {
+                            data[i] = exprColumns[i].getValue(session);
+                        }
+                    }
+                }
+
+                for (int i = 1; i < groupSet.nullSets; i++) {
+                    navigator.add(data);
+                    navigator.next();
+                }
+            }
+
+            while (groupsIterator.hasNext()) {
+                navigator.resetRowMap();
+
+                HsqlList set = (HsqlList) groupsIterator.next();
+
+                session.sessionContext.setGroup(set);
+
+                for (int i = 0; i < baseResultSize; i++) {
+                    Object[] row  = baseResult[i];
+                    Object[] data = new Object[indexLimitData];
+
+                    for (int j = indexLimitVisible; j < indexStartHaving;
+                            j++) {
+                        if (set.contains(j)) {
+                            data[j] = row[j];
+                        }
+                    }
+
+                    Object[] groupData =
+                        navigator.getGroupDataAndPosition(data);
+
+                    if (groupData == null) {
+                        navigator.add(data);    // must populate before positioning
+                        navigator.absolute(navigator.getSize() - 1);
+                    } else {
+                        data = groupData;
+                    }
+
+                    for (int j = indexStartAggregates;
+                            j < indexLimitExpressions; j++) {
+                        data[j] =
+                            exprColumns[j].updateAggregatingValue(session,
+                                (SetFunction) data[j], (SetFunction) row[j]);
+                    }
+
+                    for (int j = 0; j < indexLimitVisible; j++) {
+                        data[j] = exprColumns[j].getValue(session);
+                    }
+
+                    navigator.update(groupData, data);
+                }
+            }
+        }
+
+        navigator.reset();
+
         if (isAggregated) {
             if (!resultGrouped && navigator.getSize() == 0) {
                 Object[] data = new Object[exprColumns.length];
@@ -1690,8 +1899,11 @@ public class QuerySpecification extends QueryExpression {
                 navigator.add(data);
             }
 
-            navigator.reset();
-            session.sessionContext.setRangeIterator(navigator);
+            if (isGroupingSets) {
+                for (int i = 0; i < groupSet.nullSets; i++) {
+                    navigator.next();
+                }
+            }
 
             while (navigator.next()) {
                 Object[] data = navigator.getCurrent();
@@ -1708,18 +1920,17 @@ public class QuerySpecification extends QueryExpression {
                     }
                 }
             }
-
-            session.sessionContext.unsetRangeIterator(navigator);
         }
 
         navigator.reset();
 
-        if (havingCondition != null) {
+        if (havingColumnCount != 0) {
             while (navigator.next()) {
                 Object[] data = navigator.getCurrent();
+                boolean test =
+                    exprColumns[indexStartHaving].testCondition(session);
 
-                if (!Boolean.TRUE.equals(
-                        data[indexLimitVisible + groupByColumnCount])) {
+                if (!test) {
                     navigator.removeCurrent();
                 }
             }
@@ -1727,6 +1938,8 @@ public class QuerySpecification extends QueryExpression {
             navigator.reset();
         }
 
+        session.sessionContext.unsetRangeIterator(navigator);
+
         return result;
     }
 
@@ -1984,9 +2197,9 @@ public class QuerySpecification extends QueryExpression {
             }
         }
 
-        if (havingCondition != null) {
+        if (havingColumnCount != 0) {
             sb.append(' ').append(Tokens.T_HAVING).append(' ');
-            sb.append(havingCondition.getSQL());
+            sb.append(exprColumns[indexStartHaving].getSQL());
         }
 
         if (sortAndSlice.hasOrder()) {
@@ -2088,8 +2301,8 @@ public class QuerySpecification extends QueryExpression {
             sb.append(b).append("]\n");
         }
 
-        if (havingCondition != null) {
-            temp = havingCondition.describe(session, blanks);
+        if (havingColumnCount != 0) {
+            temp = exprColumns[indexStartHaving].describe(session, blanks);
 
             sb.append(b).append("havingCondition=[").append(temp).append(
                 "]\n");
@@ -2398,10 +2611,6 @@ public class QuerySpecification extends QueryExpression {
             set = queryCondition.collectAllSubqueries(set);
         }
 
-        if (havingCondition != null) {
-            set = havingCondition.collectAllSubqueries(set);
-        }
-
         for (int i = 0; i < rangeVariables.length; i++) {
             OrderedHashSet temp = rangeVariables[i].getSubqueries();
 
@@ -2457,11 +2666,6 @@ public class QuerySpecification extends QueryExpression {
                     stopAtTypeSet);
         }
 
-        if (havingCondition != null) {
-            set = havingCondition.collectAllExpressions(set, typeSet,
-                    stopAtTypeSet);
-        }
-
         for (int i = 0; i < rangeVariables.length; i++) {
             rangeVariables[i].collectAllExpressions(set, typeSet,
                     stopAtTypeSet);
@@ -2480,10 +2684,6 @@ public class QuerySpecification extends QueryExpression {
             queryCondition.collectObjectNames(set);
         }
 
-        if (havingCondition != null) {
-            havingCondition.collectObjectNames(set);
-        }
-
         for (int i = 0, len = rangeVariables.length; i < len; i++) {
             HsqlName name = rangeVariables[i].getTable().getName();
 
@@ -2504,11 +2704,6 @@ public class QuerySpecification extends QueryExpression {
                     range, list);
         }
 
-        if (havingCondition != null) {
-            havingCondition = havingCondition.replaceColumnReferences(session,
-                    range, list);
-        }
-
         for (int i = 0, len = rangeVariables.length; i < len; i++) {
             rangeVariables[i].replaceColumnReferences(session, range, list);
         }
@@ -2525,10 +2720,6 @@ public class QuerySpecification extends QueryExpression {
             queryCondition.replaceRangeVariables(ranges, newRanges);
         }
 
-        if (havingCondition != null) {
-            havingCondition.replaceRangeVariables(ranges, newRanges);
-        }
-
         for (int i = 0, len = rangeVariables.length; i < len; i++) {
             rangeVariables[i].getSubqueries();
         }
@@ -2547,11 +2738,6 @@ public class QuerySpecification extends QueryExpression {
                     resultRangePosition);
         }
 
-        if (havingCondition != null) {
-            havingCondition = havingCondition.replaceExpressions(expressions,
-                    resultRangePosition);
-        }
-
         for (int i = 0, len = rangeVariables.length; i < len; i++) {
             rangeVariables[i].replaceExpressions(expressions,
                                                  resultRangePosition);
