@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@ import org.hsqldb.types.Type;
  * Metadata for range variables, including conditions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class RangeVariable {
@@ -69,9 +69,6 @@ public class RangeVariable {
     public static final int VARIALBE_RANGE    = 4;
     public static final int PLACEHOLDER_RANGE = 5;
 
-    //
-    private static final RowIterator emptyIterator = new RangeIteratorEmpty();
-
     //
     Table                  rangeTable;
     final SimpleName       tableAlias;
@@ -98,7 +95,7 @@ public class RangeVariable {
     ExpressionPeriodOp periodCondition;
 
     // role based condition
-    ExpressionLogical filterCondition;
+    Expression filterCondition;
 
     //
     boolean isLateral;
@@ -210,8 +207,8 @@ public class RangeVariable {
 
         columnsInGroupBy              = rangeTable.getNewColumnCheckList();
         usedColumns                   = rangeTable.getNewColumnCheckList();
-        joinConditions[0].rangeIndex  = rangeTable.getPrimaryIndex();
-        whereConditions[0].rangeIndex = rangeTable.getPrimaryIndex();
+        joinConditions[0].rangeIndex  = rangeTable.getDefaultIndex();
+        whereConditions[0].rangeIndex = rangeTable.getDefaultIndex();
     }
 
     public void setJoinType(boolean isLeft, boolean isRight) {
@@ -229,16 +226,15 @@ public class RangeVariable {
         periodCondition = condition;
     }
 
-    public void setFilterExpression(Session session, ExpressionLogical expr) {
+    public void setFilterExpression(Session session, Expression expr) {
 
         if (expr != null) {
-            expr = (ExpressionLogical) expr.duplicate();
-
             RangeGroup ranges =
                 new RangeGroupSimple(new RangeVariable[]{ this }, false);
 
             expr.resolveColumnReferences(session, ranges,
                                          RangeGroup.emptyArray, null);
+            expr.resolveTypes(session, null);
 
             filterCondition = expr;
         }
@@ -1157,7 +1153,7 @@ public class RangeVariable {
 
         Session         session;
         int             rangePosition;
-        RowIterator     it = emptyIterator;
+        RowIterator     it = RangeIterator.emptyRowIterator;
         PersistentStore store;
         boolean         isBeforeFirst;
         RangeVariable   rangeVar;
@@ -1173,7 +1169,7 @@ public class RangeVariable {
             if (isBeforeFirst) {
                 isBeforeFirst = false;
             } else {
-                if (it == emptyIterator) {
+                if (it == RangeIterator.emptyRowIterator) {
                     return false;
                 }
             }
@@ -1219,7 +1215,7 @@ public class RangeVariable {
 
             it.release();
 
-            it            = emptyIterator;
+            it            = RangeIterator.emptyRowIterator;
             isBeforeFirst = true;
         }
 
@@ -1244,7 +1240,7 @@ public class RangeVariable {
         RangeVariableConditions[] whereConditions;
         RangeVariableConditions[] joinConditions;
         ExpressionPeriodOp        periodCondition;
-        ExpressionLogical         filterCondition;
+        Expression                filterCondition;
         int                       condIndex = 0;
 
         //
@@ -1309,7 +1305,7 @@ public class RangeVariable {
 
                 it.release();
 
-                it            = emptyIterator;
+                it            = RangeIterator.emptyRowIterator;
                 isBeforeFirst = true;
 
                 condIndex++;
@@ -1326,7 +1322,7 @@ public class RangeVariable {
 
             it.release();
 
-            it            = emptyIterator;
+            it            = RangeIterator.emptyRowIterator;
             isBeforeFirst = true;
             condIndex     = 0;
         }
@@ -1344,7 +1340,7 @@ public class RangeVariable {
             }
 
             if (conditions[condIndex].isFalse) {
-                it = conditions[condIndex].rangeIndex.emptyIterator();
+                it = RowIterator.emptyRowIterator;
 
                 return;
             }
@@ -1402,7 +1398,7 @@ public class RangeVariable {
                         .getDataType();
 
                 if (i == 0 && value == null) {
-                    it = conditions[condIndex].rangeIndex.emptyIterator();
+                    it = RowIterator.emptyRowIterator;
 
                     return;
                 }
@@ -1434,8 +1430,7 @@ public class RangeVariable {
                                 break;
 
                             default :
-                                it = conditions[condIndex].rangeIndex
-                                    .emptyIterator();
+                                it = RowIterator.emptyRowIterator;
 
                                 return;
                         }
@@ -1457,8 +1452,7 @@ public class RangeVariable {
 
                             // fall through
                             default :
-                                it = conditions[condIndex].rangeIndex
-                                    .emptyIterator();
+                                it = RowIterator.emptyRowIterator;
 
                                 return;
                         }
@@ -1496,7 +1490,7 @@ public class RangeVariable {
                 else {
                     it.release();
 
-                    it = emptyIterator;
+                    it = RangeIterator.emptyRowIterator;
 
                     break;
                 }
@@ -1564,7 +1558,7 @@ public class RangeVariable {
 
             it.release();
 
-            it = emptyIterator;
+            it = RangeIterator.emptyRowIterator;
 
             if (hasLeftOuterRow && condIndex == conditions.length - 1) {
                 result =
@@ -1612,7 +1606,7 @@ public class RangeVariable {
         public boolean next() {
 
             if (isOnRightOuterRows) {
-                if (it == emptyIterator) {
+                if (it == RangeIterator.emptyRowIterator) {
                     return false;
                 }
 
@@ -1629,7 +1623,7 @@ public class RangeVariable {
             while (true) {
                 if (it.next()) {}
                 else {
-                    it = emptyIterator;
+                    it = RangeIterator.emptyRowIterator;
 
                     break;
                 }
@@ -1758,45 +1752,6 @@ public class RangeVariable {
         }
     }
 
-    static final class RangeIteratorEmpty implements RowIterator {
-
-        public Object getField(int col) {
-            return null;
-        }
-
-        public boolean next() {
-            return false;
-        }
-
-        public Row getCurrentRow() {
-            return null;
-        }
-
-        public Object[] getCurrent() {
-            return null;
-        }
-
-        public boolean hasNext() {
-            return false;
-        }
-
-        public long getPos() {
-            return -1;
-        }
-
-        public int getStorageSize() {
-            return 0;
-        }
-
-        public void release() {}
-
-        public void removeCurrent() {}
-
-        public long getRowId() {
-            return 0L;
-        }
-    }
-
     static final class RangeIteratorCheck implements RangeIterator {
 
         final int rangePosition;
