@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ import java.sql.ResultSet;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.HsqlNameManager.SimpleName;
+import org.hsqldb.ParserBase.Recorder;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.jdbc.JDBCResultSet;
@@ -58,7 +59,7 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.4.1
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class Routine implements SchemaObject, RangeGroup, Cloneable {
@@ -190,12 +191,11 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
 
         try {
             p.read();
-            p.startRecording();
 
+            Recorder recorder = p.startRecording();
             Statement newStatement = p.compileSQLProcedureStatementOrNull(this,
                 null);
-            Token[] tokenisedStatement = p.getRecordedStatement();
-            String  sql                = Token.getSQL(tokenisedStatement);
+            String sql = recorder.getSQL();
 
             newStatement.setSQL(sql);
             setProcedure(newStatement);
@@ -694,7 +694,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
             HsqlName[] names = statement.getTableNamesForWrite();
 
             for (int i = 0; i < names.length; i++) {
-                if (names[i].schema != SqlInvariants.MODULE_HSQLNAME) {
+                if (names[i].schema != SqlInvariants.SYSTEM_SCHEMA_HSQLNAME) {
                     throw Error.error(ErrorCode.X_42608,
                                       Tokens.T_MODIFIES + ' ' + Tokens.T_SQL
                                       + ' ' + Tokens.T_DATA);
@@ -706,7 +706,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
             HsqlName[] names = statement.getTableNamesForRead();
 
             for (int i = 0; i < names.length; i++) {
-                if (names[i].schema != SqlInvariants.MODULE_HSQLNAME) {
+                if (names[i].schema != SqlInvariants.SYSTEM_SCHEMA_HSQLNAME) {
                     throw Error.error(ErrorCode.X_42608,
                                       Tokens.T_READS + ' ' + Tokens.T_SQL
                                       + ' ' + Tokens.T_DATA);
