@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,7 @@ import org.hsqldb.persist.PersistentStore;
  * Represents the chain of insert / delete / rollback / commit actions on a row.
  *
  * @author Fred Toussi (fredt@users dot sourceforge dot net)
- * @version 2.3.3
+ * @version 2.5.1
  * @since 2.0.0
  */
 public class RowAction extends RowActionBase {
@@ -110,19 +110,6 @@ public class RowAction extends RowActionBase {
         this.changeColumnMap = colMap;
     }
 
-    private RowAction(RowAction other) {
-
-        this.session         = other.session;
-        this.type            = other.type;
-        this.actionTimestamp = other.actionTimestamp;
-        this.table           = other.table;
-        this.store           = other.store;
-        this.isMemory        = other.isMemory;
-        this.memoryRow       = other.memoryRow;
-        this.rowId           = other.rowId;
-        this.changeColumnMap = other.changeColumnMap;
-    }
-
     synchronized public int getType() {
         return type;
     }
@@ -130,7 +117,7 @@ public class RowAction extends RowActionBase {
     synchronized RowAction addDeleteAction(Session session, int[] colMap) {
 
         if (type == ACTION_NONE) {
-            setAsAction(session, ACTION_DELETE);
+            setNoOpAsAction(session, ACTION_DELETE);
 
             changeColumnMap = colMap;
         } else {
@@ -214,7 +201,7 @@ public class RowAction extends RowActionBase {
     synchronized boolean addRefAction(Session session, int[] colMap) {
 
         if (type == ACTION_NONE) {
-            setAsAction(session, ACTION_REF);
+            setNoOpAsAction(session, ACTION_REF);
 
             changeColumnMap = colMap;
 
@@ -268,31 +255,18 @@ public class RowAction extends RowActionBase {
         return true;
     }
 
-    public boolean checkDeleteActions() {
-        return false;
-    }
-
-    public synchronized RowAction duplicate(Row newRow) {
-
-        RowAction action = new RowAction(session, table, type, newRow,
-                                         changeColumnMap);
-
-        return action;
-    }
-
-    synchronized void setAsAction(Session session, byte type) {
+    synchronized void setNoOpAsAction(Session session, byte type) {
 
         this.session    = session;
         this.type       = type;
         actionTimestamp = session.actionTimestamp;
-        changeColumnMap = null;
     }
 
     synchronized void setAsAction(RowActionBase action) {
         super.setAsAction(action);
     }
 
-    public void setAsNoOp() {
+    synchronized void setAsNoOp() {
 
 //        memoryRow       = null;
         session         = null;
@@ -383,7 +357,7 @@ public class RowAction extends RowActionBase {
         return deleteComplete;
     }
 
-    public void setDeleteComplete() {
+    public synchronized void setDeleteComplete() {
         deleteComplete = true;
     }
 
@@ -594,7 +568,7 @@ public class RowAction extends RowActionBase {
     /**
      * merge rolled back actions
      */
-    synchronized int mergeRollback(Session session, long timestamp, Row row) {
+    synchronized int mergeRollback(Session session, long timestamp) {
 
         RowActionBase action         = this;
         RowActionBase head           = null;
