@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -312,6 +312,14 @@ public class Scanner {
         return token.dataType;
     }
 
+    public void replaceToken(String newToken) {
+
+        sqlString = sqlString.substring(0, tokenPosition) + newToken
+                    + sqlString.substring(currentPosition);
+        limit           = sqlString.length();
+        currentPosition = tokenPosition + newToken.length();
+    }
+
     public int getLineNumber() {
         return lineNumber;
     }
@@ -320,10 +328,6 @@ public class Scanner {
         return tokenPosition;
     }
 
-    int getPosition() {
-        return tokenPosition;
-    }
-
     void position(int position) {
         currentPosition = tokenPosition = position;
     }
@@ -372,9 +376,9 @@ public class Scanner {
 
         if (c >= '0' && c <= '9') {
             c -= '0';
-        } else  if (c >= 'a' && c <= 'f') {
+        } else if (c >= 'a' && c <= 'f') {
             c -= ('a' - 10);
-        } else  if (c >= 'A' && c <= 'F') {
+        } else if (c >= 'A' && c <= 'F') {
             c -= ('A' - 10);
         } else {
             c = -1;
