@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@ import org.hsqldb.lib.OrderedHashSet;
  * Collection of SQL schema objects of a specific type in a schema
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class SchemaObjectSet {
@@ -71,10 +71,11 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.REFERENCE :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE :
                 map = new HashMappedList();
                 break;
 
-            case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
                 map = new HashMap();
@@ -102,12 +103,13 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.REFERENCE :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE :
                 SchemaObject object = ((SchemaObject) map.get(name));
 
                 return object == null ? null
                                       : object.getName();
 
-            case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX : {
                 return (HsqlName) map.get(name);
@@ -134,6 +136,8 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.REFERENCE :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE :
                 return (SchemaObject) map.get(name);
 
             default :
@@ -185,7 +189,6 @@ public class SchemaObjectSet {
 
         switch (name.type) {
 
-            case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
                 value = name;
@@ -245,7 +248,9 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.ROUTINE :
-            case SchemaObject.REFERENCE : {
+            case SchemaObject.REFERENCE :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE : {
                 int i = ((HashMappedList) map).getIndex(name.name);
 
                 if (i == -1) {
@@ -278,7 +283,6 @@ public class SchemaObjectSet {
 
                 break;
             }
-            case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX : {
                 map.remove(name.name);
@@ -310,7 +314,8 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.REFERENCE :
-            case SchemaObject.COLUMN :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
                 code = ErrorCode.X_42504;
@@ -342,7 +347,8 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
             case SchemaObject.REFERENCE :
-            case SchemaObject.COLUMN :
+            case SchemaObject.EXCEPTION :
+            case SchemaObject.MODULE :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
                 code = ErrorCode.X_42501;
@@ -395,12 +401,15 @@ public class SchemaObjectSet {
             case SchemaObject.TRIGGER :
                 return Tokens.T_TRIGGER;
 
+            case SchemaObject.EXCEPTION :
+                return Tokens.T_EXCEPTION;
+
+            case SchemaObject.MODULE :
+                return Tokens.T_MODULE;
+
             case SchemaObject.REFERENCE :
                 return Tokens.T_SYNONYM;
 
-            case SchemaObject.COLUMN :
-                return Tokens.T_COLUMN;
-
             case SchemaObject.PARAMETER :
                 return Tokens.T_PARAMETER;
 
@@ -441,12 +450,7 @@ public class SchemaObjectSet {
                 for (int i = 0; i < routineSchema.routines.length; i++) {
                     Routine routine = routineSchema.routines[i];
 
-                    if (routine.dataImpact == Routine.NO_SQL
-                            || routine.dataImpact == Routine.CONTAINS_SQL
-                            || routine.language == Routine.LANGUAGE_JAVA) {}
-                    else {
-                        set.add(routine);
-                    }
+                    set.add(routine);
                 }
             }
 
@@ -462,8 +466,16 @@ public class SchemaObjectSet {
 
         while (it.hasNext()) {
             SchemaObject   object     = (SchemaObject) it.next();
-            OrderedHashSet references = object.getReferences();
             boolean        isResolved = true;
+            OrderedHashSet references;
+
+            if (object.getType() == SchemaObject.TABLE) {
+                ((Table) object).setForwardConstraints(resolved);
+
+                references = ((Table) object).getReferencesForScript();
+            } else {
+                references = object.getReferences();
+            }
 
             for (int j = 0; j < references.size(); j++) {
                 HsqlName name = (HsqlName) references.get(j);
@@ -528,6 +540,7 @@ public class SchemaObjectSet {
                     case SchemaObject.FUNCTION :
                     case SchemaObject.PROCEDURE :
                     case SchemaObject.SPECIFIC_ROUTINE :
+                    case SchemaObject.SEQUENCE :
                         if (!resolved.contains(name)) {
                             isResolved = false;
                         }
@@ -558,26 +571,80 @@ public class SchemaObjectSet {
                 newResolved.add(object);
             }
 
-            if (object.getType() == SchemaObject.TABLE) {
-                list.addAll(((Table) object).getSQL(resolved, unresolved));
-            } else {
-                switch (object.getType()) {
+            switch (object.getType()) {
 
-                    case SchemaObject.FUNCTION :
-                    case SchemaObject.PROCEDURE :
-                        Routine routine = ((Routine) object);
+                case SchemaObject.TABLE : {
+                    list.addAll(((Table) object).getSQL(resolved, unresolved));
 
-                        if (routine.isRecursive) {
-                            list.add(((Routine) object).getSQLDeclaration());
-                            list.add(((Routine) object).getSQLAlter());
-                        } else {
-                            list.add(object.getSQL());
+                    String comment =
+                        object.getName().getCommentSQL(Tokens.T_TABLE);
+
+                    if (comment != null) {
+                        list.add(comment);
+                    }
+
+                    for (int j = 0; j < ((Table) object).getColumnCount();
+                            j++) {
+                        ColumnSchema column = ((Table) object).getColumn(j);
+
+                        comment =
+                            column.getName().getCommentSQL(Tokens.T_COLUMN);
+
+                        if (comment != null) {
+                            list.add(comment);
                         }
-                        break;
+                    }
 
-                    default :
-                        list.add(object.getSQL());
+                    break;
                 }
+                case SchemaObject.FUNCTION :
+                case SchemaObject.PROCEDURE : {
+                    Routine routine = ((Routine) object);
+
+                    if (routine.isRecursive) {
+                        list.add(((Routine) object).getSQLDeclaration());
+                        list.add(((Routine) object).getSQLAlter());
+                    } else {
+                        list.add(object.getSQL());
+                    }
+
+                    String comment =
+                        object.getName().getCommentSQL(Tokens.T_ROUTINE);
+
+                    if (comment != null) {
+                        list.add(comment);
+                    }
+
+                    break;
+                }
+                case SchemaObject.TRIGGER : {
+                    TriggerDef trigger = ((TriggerDef) object);
+
+                    list.add(object.getSQL());
+
+                    String comment =
+                        object.getName().getCommentSQL(Tokens.T_TRIGGER);
+
+                    if (comment != null) {
+                        list.add(comment);
+                    }
+
+                    break;
+                }
+                case SchemaObject.SEQUENCE : {
+                    list.add(object.getSQL());
+
+                    String comment =
+                        object.getName().getCommentSQL(Tokens.T_SEQUENCE);
+
+                    if (comment != null) {
+                        list.add(comment);
+                    }
+
+                    break;
+                }
+                default :
+                    list.add(object.getSQL());
             }
         }
     }
