@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,6 @@
 
 package org.hsqldb;
 
-import java.io.InputStream;
 import java.text.SimpleDateFormat;
 import java.util.Calendar;
 import java.util.GregorianCalendar;
@@ -76,7 +75,7 @@ import org.hsqldb.types.TypedComparator;
  * Implementation of SQL sessions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.0
  */
 public class Session implements SessionInterface {
@@ -98,6 +97,7 @@ public class Session implements SessionInterface {
     int                     actionIndex;
     long                    actionStartTimestamp;
     long                    actionTimestamp;
+    long                    statementStartTimestamp;
     long                    transactionTimestamp;
     long                    transactionEndTimestamp;
     final boolean           txConflictRollback;
@@ -447,7 +447,7 @@ public class Session implements SessionInterface {
      * @throws  HsqlException
      */
     public void addDeleteAction(Table table, PersistentStore store, Row row,
-                                int[] colMap) {
+                                int[] changedColumns) {
 
 //        tempActionHistory.add("add delete action " + actionTimestamp);
         if (abortTransaction) {
@@ -458,14 +458,15 @@ public class Session implements SessionInterface {
             throw Error.error(ErrorCode.X_40502);
         }
 
-        getTransactionSystemTimestamp();
-        database.txManager.addDeleteAction(this, table, store, row, colMap);
+        getTransactionUTC();
+        database.txManager.addDeleteAction(this, table, store, row,
+                                           changedColumns);
     }
 
     void addInsertAction(Table table, PersistentStore store, Row row,
                          int[] changedColumns) {
 
-        getTransactionSystemTimestamp();
+        getTransactionUTC();
 
 //        tempActionHistory.add("add insert to transaction " + actionTimestamp);
         database.txManager.addInsertAction(this, table, store, row,
@@ -523,7 +524,6 @@ public class Session implements SessionInterface {
         abortAction = false;
 
         timeoutManager.endTimeout();
-
         sessionData.persistentStoreCollection.clearStatementTables();
 
         if (result.mode == ResultConstants.ERROR) {
@@ -637,9 +637,9 @@ public class Session implements SessionInterface {
 
     private void endTransaction(boolean commit, boolean chain) {
 
-        abortAction        = false;
-        abortTransaction   = false;
-        systemTimestampSet = false;
+        abortAction       = false;
+        abortTransaction  = false;
+        transactionUTCSet = false;
 
         sessionContext.resetStack();
         sessionContext.savepoints.clear();
@@ -647,7 +647,6 @@ public class Session implements SessionInterface {
         rowActionList.clear();
         sessionData.persistentStoreCollection.clearTransactionTables();
         sessionData.closeAllTransactionNavigators();
-        sessionData.clearLobOps();
 
         if (!chain) {
             sessionContext.isReadOnly = isReadOnlyDefault ? Boolean.TRUE
@@ -684,7 +683,6 @@ public class Session implements SessionInterface {
         rollbackNoCheck(false);
         sessionData.closeAllNavigators();
         sessionData.persistentStoreCollection.clearAllTables();
-        sessionData.clearLobOps();
         statementManager.reset();
 
         sessionContext.lastIdentity = ValuePool.INTEGER_0;
@@ -1320,6 +1318,8 @@ public class Session implements SessionInterface {
         }
 
         sessionContext.currentStatement = cs;
+        statementStartTimestamp =
+            database.txManager.getGlobalChangeTimestamp();
 
         boolean isTX = cs.isTransactionStatement();
 
@@ -1346,7 +1346,6 @@ public class Session implements SessionInterface {
             return r;
         }
 
-
         timeoutManager.startTimeout(timeout);
 
         repeatLoop:
@@ -1650,17 +1649,15 @@ public class Session implements SessionInterface {
     }
 
 // session DATETIME functions
-    long                  currentDateSCN;
     long                  currentTimestampSCN;
     long                  currentMillis;
     private TimestampData currentDate;
     private TimestampData currentTimestamp;
     private TimestampData localTimestamp;
-    private TimestampData transactionSystemTimestamp =
-        getSystemTimestamp(false);
-    boolean          systemTimestampSet = false;
-    private TimeData currentTime;
-    private TimeData localTime;
+    private TimestampData transactionUTC;
+    boolean               transactionUTCSet;
+    private TimeData      currentTime;
+    private TimeData      localTime;
 
     /**
      * Returns the current date, unchanged for the duration of the current
@@ -1753,32 +1750,35 @@ public class Session implements SessionInterface {
         }
     }
 
-    synchronized TimestampData getSystemTimestamp(boolean withZone) {
+    static TimestampData getSystemTimestamp(boolean withZone, boolean utc) {
 
         long millis  = System.currentTimeMillis();
         long seconds = millis / 1000;
         int  nanos   = (int) (millis % 1000) * 1000000;
         int  offset  = 0;
 
-        if (!withZone) {
+        if (!utc) {
             TimeZone zone = TimeZone.getDefault();
 
-            offset  = zone.getOffset(millis) / 1000;
-            seconds += offset;
-            offset  = 0;
+            offset = zone.getOffset(millis) / 1000;
+
+            if (!withZone) {
+                seconds += offset;
+                offset  = 0;
+            }
         }
 
         return new TimestampData(seconds, nanos, offset);
     }
 
-    TimestampData getTransactionSystemTimestamp() {
+    TimestampData getTransactionUTC() {
 
-        if (!systemTimestampSet) {
-            transactionSystemTimestamp = getSystemTimestamp(false);
-            systemTimestampSet         = true;
+        if (!transactionUTCSet) {
+            transactionUTC    = getSystemTimestamp(true, true);
+            transactionUTCSet = true;
         }
 
-        return transactionSystemTimestamp;
+        return transactionUTC;
     }
 
     private void resetCurrentTimestamp() {
@@ -1939,8 +1939,6 @@ public class Session implements SessionInterface {
             throw Error.error(ErrorCode.X_0F502);
         }
 
-        sessionData.registerNewLob(lobID);
-
         return new BlobDataID(lobID);
     }
 
@@ -1952,8 +1950,6 @@ public class Session implements SessionInterface {
             throw Error.error(ErrorCode.X_0F502);
         }
 
-        sessionData.registerNewLob(lobID);
-
         return new ClobDataID(lobID);
     }
 
@@ -1961,8 +1957,8 @@ public class Session implements SessionInterface {
         sessionData.registerLobForResult(result);
     }
 
-    public void allocateResultLob(ResultLob result, InputStream inputStream) {
-        sessionData.allocateLobForResult(result, inputStream);
+    public Result allocateResultLob(ResultLob result) {
+        return sessionData.allocateLobForResult(result);
     }
 
     Result performLOBOperation(ResultLob cmd) {
@@ -2026,7 +2022,8 @@ public class Session implements SessionInterface {
         if (result.getType() == ResultConstants.SQLCANCEL) {
             if (result.getSessionRandomID() == randomId) {
                 database.txManager.resetSession(
-                    null, this, TransactionManager.resetSessionStatement);
+                    null, this, Long.MAX_VALUE,
+                    TransactionManager.resetSessionStatement);
             }
         }
 
@@ -2179,6 +2176,9 @@ public class Session implements SessionInterface {
 
         if (calendarGMT == null) {
             calendarGMT = new GregorianCalendar(TimeZone.getTimeZone("GMT"));
+
+            calendarGMT.setFirstDayOfWeek(Calendar.MONDAY);
+            calendarGMT.setMinimalDaysInFirstWeek(4);
         }
 
         return calendarGMT;
@@ -2345,6 +2345,7 @@ public class Session implements SessionInterface {
     class TimeoutManager {
 
         AtomicInteger currentTimeout = new AtomicInteger();
+        volatile long checkTimestamp;
 
         void startTimeout(int timeout) {
 
@@ -2354,6 +2355,8 @@ public class Session implements SessionInterface {
                 return;
             }
 
+            checkTimestamp = Session.this.statementStartTimestamp;
+
             database.timeoutRunner.addSession(Session.this);
         }
 
@@ -2375,7 +2378,7 @@ public class Session implements SessionInterface {
             if (result <= 0) {
                 currentTimeout.set(0);
                 database.txManager.resetSession(
-                    null, Session.this,
+                    null, Session.this, checkTimestamp,
                     TransactionManager.resetSessionStatement);
 
                 return true;
