@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.HsqlArrayList;
+import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.LongDeque;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.navigator.RangeIterator;
@@ -49,7 +50,7 @@ import org.hsqldb.result.Result;
  * Session execution context and temporary data structures
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.4.0
  * @since 1.9.0
  */
 public class SessionContext {
@@ -86,6 +87,10 @@ public class SessionContext {
     // range variable data
     RangeIterator[] rangeIterators;
 
+    // grouping sets data
+    GroupSet groupSet;
+    HsqlList currentGroup;
+
     // session tables
     HashMappedList sessionTables;
     HashMappedList popSessionTables;
@@ -323,6 +328,14 @@ public class SessionContext {
         rangeIterators[position] = null;
     }
 
+    public void setGroupSet(GroupSet set) {
+        groupSet = set;
+    }
+
+    public void setGroup(HsqlList group) {
+        currentGroup = group;
+    }
+
     /**
      * For cascade operations
      */
