@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,12 +39,14 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArraySort;
 import org.hsqldb.lib.HashSet;
+import org.hsqldb.lib.Iterator;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.types.DTIType;
 import org.hsqldb.types.IntervalMonthData;
 import org.hsqldb.types.IntervalSecondData;
 import org.hsqldb.types.IntervalType;
 import org.hsqldb.types.NumberType;
+import org.hsqldb.types.TimeData;
 import org.hsqldb.types.TimestampData;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.TypedComparator;
@@ -57,7 +59,7 @@ import org.hsqldb.types.Types;
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.2
  *
  */
@@ -83,7 +85,9 @@ public class SetFunctionValueAggregate implements SetFunction {
     private boolean    hasNull;
     private boolean    every = true;
     private boolean    some  = false;
-    private long       currentLong;
+    private long       hiLong;
+    private long       loLong;
+    private long       fraction;
     private double     currentDouble;
     private BigDecimal currentBigDecimal;
     private Object     currentValue;
@@ -163,7 +167,7 @@ public class SetFunctionValueAggregate implements SetFunction {
                     case Types.TINYINT :
                     case Types.SQL_SMALLINT :
                     case Types.SQL_INTEGER :
-                        currentLong += ((Number) item).intValue();
+                        loLong += ((Number) item).intValue();
 
                         return;
 
@@ -171,13 +175,12 @@ public class SetFunctionValueAggregate implements SetFunction {
                         if (item instanceof IntervalSecondData) {
                             addLong(((IntervalSecondData) item).getSeconds());
 
-                            currentLong +=
-                                ((IntervalSecondData) item).getNanos();
+                            fraction += ((IntervalSecondData) item).getNanos();
 
-                            if (currentLong > 1000000000) {
-                                addLong(currentLong / 1000000000);
+                            if (fraction > 1000000000) {
+                                addLong(fraction / 1000000000);
 
-                                currentLong %= 1000000000;
+                                fraction %= 1000000000;
                             }
                         }
 
@@ -195,18 +198,34 @@ public class SetFunctionValueAggregate implements SetFunction {
                     case Types.SQL_TIMESTAMP_WITH_TIME_ZONE : {
                         addLong(((TimestampData) item).getSeconds());
 
-                        currentLong += ((TimestampData) item).getNanos();
+                        fraction += ((TimestampData) item).getNanos();
 
-                        if (currentLong > 1000000000) {
-                            addLong(currentLong / 1000000000);
+                        if (fraction > 1000000000) {
+                            addLong(fraction / 1000000000);
 
-                            currentLong %= 1000000000;
+                            fraction %= 1000000000;
                         }
 
                         currentDouble = ((TimestampData) item).getZone();
 
                         return;
                     }
+                    case Types.SQL_TIME :
+                    case Types.SQL_TIME_WITH_TIME_ZONE : {
+                        addLong(((TimeData) item).getSeconds());
+
+                        fraction += ((TimeData) item).getNanos();
+
+                        if (fraction > 1000000000) {
+                            addLong(fraction / 1000000000);
+
+                            fraction %= 1000000000;
+                        }
+
+                        currentDouble = ((TimeData) item).getZone();
+
+                        return;
+                    }
                     case Types.SQL_BIGINT :
                         addLong(((Number) item).longValue());
 
@@ -296,6 +315,105 @@ public class SetFunctionValueAggregate implements SetFunction {
         }
     }
 
+    public void addGroup(SetFunction group) {
+
+        SetFunctionValueAggregate item = (SetFunctionValueAggregate) group;
+
+        if (isDistinct) {
+            HashSet  otherSet = item.distinctValues;
+            Iterator it       = otherSet.iterator();
+
+            while (it.hasNext()) {
+                Object value = it.next();
+
+                add(value);
+            }
+
+            return;
+        }
+
+        switch (setType) {
+
+            case OpTypes.COUNT :
+                count += item.count;
+
+                return;
+
+            case OpTypes.STDDEV_POP :
+            case OpTypes.STDDEV_SAMP :
+            case OpTypes.VAR_POP :
+            case OpTypes.VAR_SAMP :
+                count += item.count;
+
+                addDataGroup(item);
+
+                return;
+
+            case OpTypes.AVG :
+            case OpTypes.SUM :
+                count += item.count;
+
+                switch (typeCode) {
+
+                    case Types.TINYINT :
+                    case Types.SQL_SMALLINT :
+                    case Types.SQL_INTEGER :
+                        loLong += item.loLong;
+
+                        return;
+
+                    case Types.SQL_INTERVAL_SECOND :
+                    case Types.SQL_INTERVAL_MONTH :
+                    case Types.SQL_DATE :
+                    case Types.SQL_TIMESTAMP :
+                    case Types.SQL_TIMESTAMP_WITH_TIME_ZONE : {
+                        addLongGroup(item);
+
+                        fraction += item.fraction;
+
+                        if (fraction > 1000000000) {
+                            addLong(fraction / 1000000000);
+
+                            fraction %= 1000000000;
+                        }
+
+                        currentDouble = item.currentDouble;
+
+                        return;
+                    }
+                    case Types.SQL_BIGINT :
+                        addLongGroup(item);
+
+                        return;
+
+                    case Types.SQL_REAL :
+                    case Types.SQL_FLOAT :
+                    case Types.SQL_DOUBLE :
+                        currentDouble += item.currentDouble;
+
+                        return;
+
+                    case Types.SQL_NUMERIC :
+                    case Types.SQL_DECIMAL :
+                        if (currentBigDecimal == null) {
+                            currentBigDecimal = item.currentBigDecimal;
+                        } else {
+                            currentBigDecimal =
+                                currentBigDecimal.add(item.currentBigDecimal);
+                        }
+
+                        return;
+
+                    default :
+                        throw Error.error(ErrorCode.X_42563);
+                }
+            default :
+                add(group.getValue());
+
+                return;
+        }
+    }
+
     public Object getValue() {
 
         if (hasNull) {
@@ -334,11 +452,11 @@ public class SetFunctionValueAggregate implements SetFunction {
                     case Types.SQL_INTEGER :
                         if (returnType.scale != 0) {
                             return returnType.divide(session,
-                                                     Long.valueOf(currentLong),
+                                                     Long.valueOf(loLong),
                                                      Long.valueOf(count));
                         }
 
-                        return Long.valueOf(currentLong / count);
+                        return Long.valueOf(loLong / count);
 
                     case Types.SQL_BIGINT : {
                         long value = getLongSum().divide(
@@ -372,8 +490,11 @@ public class SetFunctionValueAggregate implements SetFunction {
 
                         if (type.isIntervalDaySecondType()) {
                             long nanos =
-                                (bi[1].longValue() * DTIType
-                                    .limitNanoseconds + currentLong) / count;
+                                (fraction + bi[1].longValue() * DTIType
+                                    .limitNanoseconds) / count;
+
+                            nanos = DTIType.normaliseFraction((int) nanos,
+                                                              type.scale);
 
                             return new IntervalSecondData(bi[0].longValue(),
                                                           nanos,
@@ -387,16 +508,50 @@ public class SetFunctionValueAggregate implements SetFunction {
                     case Types.SQL_DATE :
                     case Types.SQL_TIMESTAMP :
                     case Types.SQL_TIMESTAMP_WITH_TIME_ZONE : {
-                        BigInteger bi =
-                            getLongSum().divide(BigInteger.valueOf(count));
+                        BigInteger[] bi = getLongSum().divideAndRemainder(
+                            BigInteger.valueOf(count));
 
-                        if (NumberType.compareToLongLimits(bi) != 0) {
+                        if (NumberType.compareToLongLimits(bi[0]) != 0) {
                             throw Error.error(ErrorCode.X_22015);
                         }
 
-                        return new TimestampData(bi.longValue(),
-                                                 (int) currentLong,
-                                                 (int) currentDouble);
+                        long seconds = bi[0].longValue();
+                        long nanos =
+                            (fraction + bi[1].longValue() * DTIType
+                                .limitNanoseconds) / count;
+
+                        nanos = DTIType.normaliseFraction((int) nanos,
+                                                          type.scale);
+
+                        if (setType == Types.SQL_DATE) {
+                            seconds = HsqlDateTime.getNormalisedDate(seconds);
+                            nanos   = 0;
+                        }
+
+                        return new TimestampData(seconds, (int) nanos, 0);
+                    }
+                    case Types.SQL_TIME :
+                    case Types.SQL_TIME_WITH_TIME_ZONE : {
+                        BigInteger[] bi = getLongSum().divideAndRemainder(
+                            BigInteger.valueOf(count));
+
+                        if (NumberType.compareToLongLimits(bi[0]) != 0) {
+                            throw Error.error(ErrorCode.X_22015);
+                        }
+
+                        long seconds = bi[0].longValue();
+                        long nanos =
+                            (fraction + bi[1].longValue() * DTIType
+                                .limitNanoseconds) / count;
+
+                        nanos = DTIType.normaliseFraction((int) nanos, 0);
+
+                        if (setType == Types.SQL_DATE) {
+                            seconds = HsqlDateTime.getNormalisedDate(seconds);
+                            nanos   = 0;
+                        }
+
+                        return new TimeData((int) seconds, (int) nanos, 0);
                     }
                     default :
                         throw Error.runtimeError(ErrorCode.U_S0500,
@@ -409,7 +564,7 @@ public class SetFunctionValueAggregate implements SetFunction {
                     case Types.TINYINT :
                     case Types.SQL_SMALLINT :
                     case Types.SQL_INTEGER :
-                        return Long.valueOf(currentLong);
+                        return Long.valueOf(loLong);
 
                     case Types.SQL_BIGINT :
                         return new BigDecimal(getLongSum());
@@ -433,7 +588,7 @@ public class SetFunctionValueAggregate implements SetFunction {
 
                         if (type.isIntervalDaySecondType()) {
                             return new IntervalSecondData(bi.longValue(),
-                                                          currentLong,
+                                                          fraction,
                                                           (IntervalType) type,
                                                           true);
                         } else {
@@ -482,31 +637,36 @@ public class SetFunctionValueAggregate implements SetFunction {
      */
     static final BigInteger multiplier =
         BigInteger.valueOf(0x0000000100000000L);
-    long hi;
-    long lo;
 
     private void addLong(long value) {
 
         if (value == 0) {}
         else if (value > 0) {
-            hi += value >> 32;
-            lo += value & 0x00000000ffffffffL;
+            hiLong += value >> 32;
+            loLong += value & 0x00000000ffffffffL;
         } else {
             if (value == Long.MIN_VALUE) {
-                hi -= 0x000000080000000L;
+                hiLong -= 0x000000080000000L;
             } else {
                 long temp = ~value + 1;
 
-                hi -= temp >> 32;
-                lo -= temp & 0x00000000ffffffffL;
+                hiLong -= temp >> 32;
+                loLong -= temp & 0x00000000ffffffffL;
             }
         }
     }
 
+    private void addLongGroup(SetFunctionValueAggregate item) {
+
+        addLong(item.loLong);
+
+        hiLong += item.hiLong;
+    }
+
     private BigInteger getLongSum() {
 
-        BigInteger biglo  = BigInteger.valueOf(lo);
-        BigInteger bighi  = BigInteger.valueOf(hi);
+        BigInteger biglo  = BigInteger.valueOf(loLong);
+        BigInteger bighi  = BigInteger.valueOf(hiLong);
         BigInteger result = (bighi.multiply(multiplier)).add(biglo);
 
 /*
@@ -555,6 +715,30 @@ public class SetFunctionValueAggregate implements SetFunction {
         sk  += xi;
     }
 
+    private void addDataGroup(SetFunctionValueAggregate value) {
+
+        double cm;
+
+        if (value == null || value.count == 0) {
+            return;
+        }
+
+        if (!initialized) {
+            n           = value.n;
+            sk          = value.sk;
+            vk          = value.vk;
+            initialized = true;
+
+            return;
+        }
+
+        cm = (sk + value.sk) / (n + value.n);
+        vk = vk + value.vk + n * (sk / n - cm) * (sk / n - cm)
+             + value.n * (value.sk / value.n - cm) * (value.sk / value.n - cm);
+        sk += value.sk;
+        n  += value.n;
+    }
+
     private Double getVariance() {
 
         if (!initialized) {
