@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@ import org.hsqldb.lib.HsqlArrayList;
 /**
  * Value for array aggregate functions.
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 2.4.1
  *
  */
@@ -50,6 +50,11 @@ public class SetFunctionValueArray implements SetFunction {
         list.add(item);
     }
 
+    public void addGroup(SetFunction item){
+        HsqlArrayList otherList = ((SetFunctionValueArray) item).list;
+        list.addAll(otherList);
+    }
+
     public Object getValue() {
         return list.toArray();
     }
