@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,7 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.index.Index;
+import org.hsqldb.index.IndexAVLCheck;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.OrderedHashSet;
@@ -49,13 +50,15 @@ import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.rights.User;
 import org.hsqldb.scriptio.ScriptWriterText;
+import org.hsqldb.scriptio.ScriptWriterTextColumnNames;
 import org.hsqldb.types.TimestampData;
+import org.hsqldb.persist.RowStoreAVLDisk;
 
 /**
  * Implementation of Statement for SQL commands.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class StatementCommand extends Statement {
@@ -298,15 +301,15 @@ public class StatementCommand extends Statement {
                         name, set);
                 }
 
-                Result result =
-                    Result.newSingleColumnResult(Tokens.T_REFERENCES);
+                Result result = Result.newDoubleColumnResult("OBJCT_TYPE",
+                    "OBJECT_NAME");
 
                 for (int i = 0; i < set.size(); i++) {
                     HsqlName current = (HsqlName) set.get(i);
-                    String objectName =
-                        SchemaObjectSet.getName(current.type) + ' '
-                        + current.getSchemaQualifiedStatementName();
-                    Object[] data = new Object[]{ objectName };
+                    Object[] data    = new Object[] {
+                        SchemaObjectSet.getName(current.type),
+                        current.getSchemaQualifiedStatementName()
+                    };
 
                     result.navigator.add(data);
                 }
@@ -359,17 +362,7 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.SET_DATABASE_FILES_BACKUP_INCREMENT : {
-                try {
-                    boolean mode = ((Boolean) arguments[0]).booleanValue();
-
-                    session.checkAdmin();
-                    session.checkDDLWrite();
-                    session.database.logger.setIncrementBackup(mode);
-
-                    return Result.updateZeroResult;
-                } catch (HsqlException e) {
-                    return Result.newErrorResult(e, sql);
-                }
+                return Result.updateZeroResult;
             }
             case StatementTypes.SET_DATABASE_FILES_CACHE_ROWS : {
                 try {
@@ -425,6 +418,14 @@ public class StatementCommand extends Statement {
 
                     if (session.isProcessingScript()) {
                         session.database.logger.setFilesTimestamp(value1);
+                        session.database.txManager.setGlobalChangeTimestamp(
+                            value1 + 1);
+                    } else if (session.isProcessingLog()) {
+                        if (value2 > 0) {
+                            session.database.logger.setFilesTimestamp(value1);
+                            session.database.txManager
+                                .setGlobalChangeTimestamp(value1 + 1);
+                        }
                     }
 
                     return Result.updateZeroResult;
@@ -504,11 +505,11 @@ public class StatementCommand extends Statement {
                         boolean value =
                             ((Boolean) arguments[0]).booleanValue();
 
-                        session.database.logger.setDataFileSpaces(value);
+                        session.database.logger.setDataFileSpace(value);
                     } else {
                         int value = ((Integer) arguments[0]).intValue();
 
-                        session.database.logger.setDataFileSpaces(value);
+                        session.database.logger.setDataFileSpace(value);
                     }
 
                     return Result.updateZeroResult;
@@ -955,8 +956,9 @@ public class StatementCommand extends Statement {
                 String           name      = (String) arguments[0];
                 Integer          scope     = (Integer) arguments[1];
                 Integer          type      = (Integer) arguments[2];
-                HsqlName         tableName = (HsqlName) arguments[3];
-                TimestampData    timestamp = (TimestampData) arguments[4];
+                Boolean          colNames  = (Boolean) arguments[3];
+                HsqlName         tableName = (HsqlName) arguments[4];
+                TimestampData    timestamp = (TimestampData) arguments[5];
 
                 try {
                     session.checkAdmin();
@@ -964,8 +966,13 @@ public class StatementCommand extends Statement {
                     if (name == null) {
                         return session.database.getScript(false);
                     } else {
-                        dsw = new ScriptWriterText(session.database, name,
-                                                   true, true, true);
+                        if (colNames.booleanValue()) {
+                            dsw = new ScriptWriterTextColumnNames(
+                                session.database, name);
+                        } else {
+                            dsw = new ScriptWriterText(session.database, name,
+                                                       true, true, true);
+                        }
 
                         switch (type.intValue()) {
 
@@ -1029,9 +1036,19 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.CHECK_INDEX : {
                 try {
-                    int type = ((Integer) arguments[1]).intValue();
-                    Result result = Result.newDoubleColumnResult("TABLE_NAME",
-                        "INFO");
+                    HsqlName tableName = (HsqlName) arguments[0];
+                    int      type      = ((Integer) arguments[1]).intValue();
+                    Table    table;
+                    Result   result;
+
+                    if (tableName == null) {
+                        result = IndexAVLCheck.checkAllTables(session, type);
+                    } else {
+                        table = session.database.schemaManager.getUserTable(
+                            tableName.name, tableName.schema.name);
+                        result = IndexAVLCheck.checkTable(session, table,
+                                                          type);
+                    }
 
                     return result;
                 } catch (HsqlException e) {
@@ -1052,7 +1069,7 @@ public class StatementCommand extends Statement {
                         return Result.updateZeroResult;
                     }
 
-                    if (session.database.logger.getDataFileSpaces() == 0) {
+                    if (session.database.logger.getDataFileSpace() == 0) {
                         throw Error.error(ErrorCode.ACCESS_IS_DENIED);
                     }
 
@@ -1071,19 +1088,18 @@ public class StatementCommand extends Statement {
                     DataSpaceManager dataSpace = cache.spaceManager;
                     int tableSpaceID = dataSpace.getNewTableSpaceID();
 
-                    table.setSpaceID(tableSpaceID);
-
                     // if cache exists, a memory table can get a space id
                     // it can then be converted to cached
+                    table.setSpaceID(tableSpaceID);
+
                     if (!table.isCached()) {
                         return Result.updateZeroResult;
                     }
 
-                    TableSpaceManager tableSpace =
-                        dataSpace.getTableSpace(tableSpaceID);
-                    PersistentStore store = table.getRowStore(session);
+                    RowStoreAVLDisk store =
+                        (RowStoreAVLDisk) table.getRowStore(session);
 
-                    store.setSpaceManager(tableSpace);
+                    store.setSpaceManager(session);
 
                     return Result.updateZeroResult;
                 } catch (HsqlException e) {
@@ -1111,23 +1127,20 @@ public class StatementCommand extends Statement {
                         return Result.updateZeroResult;
                     }
 
-                    table.setSpaceID(spaceid);
-
-                    if (table.store == null) {
-                        return Result.updateZeroResult;
-                    }
-
                     DataFileCache cache = session.database.logger.getCache();
 
                     if (cache == null) {
                         return Result.updateZeroResult;
                     }
 
+                    table.setSpaceID(spaceid);
+
                     DataSpaceManager dataSpace = cache.spaceManager;
                     TableSpaceManager tableSpace =
                         dataSpace.getTableSpace(table.getSpaceID());
+                    PersistentStore store = table.getRowStore(session);
 
-                    table.store.setSpaceManager(tableSpace);
+                    store.setSpaceManager(tableSpace);
 
                     return Result.updateZeroResult;
                 } catch (HsqlException e) {
@@ -1376,37 +1389,37 @@ public class StatementCommand extends Statement {
 
                         case Tokens.ALL :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionResetAll);
                             break;
 
                         case Tokens.TABLE :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionTables);
                             break;
 
                         case Tokens.RESULT :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionResults);
                             break;
 
                         case Tokens.CLOSE :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionClose);
                             break;
 
                         case Tokens.RELEASE :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionRollback);
                             break;
 
                         case Tokens.END :
                             session.database.txManager.resetSession(session,
-                                    targetSession,
+                                    targetSession, Long.MAX_VALUE,
                                     TransactionManager.resetSessionStatement);
                             break;
                     }
