@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.types.Type;
  * Implementation of Statement for PSM compound statements.
 
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class StatementCompound extends Statement implements RangeGroup {
@@ -451,7 +451,9 @@ public class StatementCompound extends Statement implements RangeGroup {
                             break;
 
                         case StatementHandler.UNDO :
-                            session.rollbackToSavepoint();
+                            if (session.sessionContext.savepoints.size() > 0) {
+                                session.rollbackToSavepoint();
+                            }
 
                             result = Result.newPSMResult(StatementTypes.LEAVE,
                                                          labelString, null);
@@ -692,14 +694,15 @@ public class StatementCompound extends Statement implements RangeGroup {
     private Result executeProtected(Session session, Statement statement) {
 
         int actionIndex = session.rowActionList.size();
-
-        session.actionTimestamp =
+        long actionTimestamp =
             session.database.txManager.getNextGlobalChangeTimestamp();
 
+        session.actionTimestamp = actionTimestamp;
+
         Result result = statement.execute(session);
 
         if (result.isError()) {
-            session.rollbackAction(actionIndex, session.actionTimestamp);
+            session.rollbackAction(actionIndex, actionTimestamp);
         }
 
         return result;
