@@ -192,16 +192,18 @@ public class StatementResultUpdate extends StatementDML {
 
         QueryExpression qe = statement.queryExpression;
 
-        this.result             = result;
-        this.actionType         = action;
-        this.baseTable          = qe.getBaseTable();
-        this.types              = types;
-        this.baseColumnMap      = qe.getBaseTableColumnMap();
-        this.writeTableNames[0] = baseTable.getName();
+        this.result               = result;
+        this.actionType           = action;
+        this.baseTable            = qe.getBaseTable();
+        this.types                = types;
+        this.baseColumnMap        = qe.getBaseTableColumnMap();
+        this.writeTableNames[0]   = baseTable.getName();
+        this.rangeVariables       = qe.getRangeVariables();
+        this.targetRangeVariables = qe.getRangeVariables();
 
         // used for statement logging - needs improvements to list only the updated values
-        this.sql                = statement.getSQL();
-        this.parameterMetaData  = qe.getMetaData();
+        this.sql               = statement.getSQL();
+        this.parameterMetaData = qe.getMetaData();
     }
 
     void checkAccessRights(Session session) {
