@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,6 @@
 package org.hsqldb;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
-import org.hsqldb.RangeGroup.RangeGroupSimple;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.index.Index;
@@ -52,7 +51,7 @@ import org.hsqldb.types.Type;
  * Implementation of Statement for DDL statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class StatementSchema extends Statement {
@@ -454,6 +453,36 @@ public class StatementSchema extends Statement {
                 try {
                     int  subType = ((Integer) arguments[0]).intValue();
                     Type domain  = (Type) arguments[1];
+                    Expression domainDefault =
+                        domain.userTypeModifier.getDefaultClause();
+                    OrderedHashSet refSet =
+                        session.database.schemaManager.getReferencesTo(
+                            domain.getName());
+                    OrderedHashSet tableSet = new OrderedHashSet();
+
+                    for (int i = 0; i < refSet.size(); i++) {
+                        HsqlName objectName = (HsqlName) refSet.get(i);
+                        HsqlName tableName  = objectName.parent;
+
+                        if (tableName.type == SchemaObject.TABLE) {
+                            SchemaObject table =
+                                session.database.schemaManager.getSchemaObject(
+                                    tableName);
+
+                            tableSet.add(table);
+                        }
+                    }
+
+                    if (subType == StatementTypes.ADD_CONSTRAINT) {
+                        Constraint c = (Constraint) arguments[2];
+
+                        for (int i = 0; i < tableSet.size(); i++) {
+                            Table      table = (Table) tableSet.get(i);
+                            TableWorks tw    = new TableWorks(session, table);
+
+                            tw.checkAddDomainConstraint(domain, c);
+                        }
+                    }
 
                     switch (subType) {
 
@@ -489,6 +518,26 @@ public class StatementSchema extends Statement {
                         }
                     }
 
+                    for (int i = 0; i < tableSet.size(); i++) {
+                        Table table = (Table) tableSet.get(i);
+
+                        if (subType == StatementTypes.DROP_DEFAULT) {
+                            for (int j = 0; j < table.getColumnCount(); j++) {
+                                ColumnSchema column = table.getColumn(j);
+
+                                if (column.dataType == domain) {
+                                    if (column.getDefaultExpression()
+                                            == null) {
+                                        column.setDefaultExpression(
+                                            domainDefault);
+                                    }
+                                }
+                            }
+                        }
+
+                        table.resetDefaultFlags();
+                    }
+
                     break;
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
@@ -1023,25 +1072,20 @@ public class StatementSchema extends Statement {
 
                             right.setColumns(t);
 
-                            if (t.getTableType() == TableBase.TEMP_TABLE
-                                    && !right.isFull()) {
+                            if (t.isTemp && !right.isFull()) {
                                 return Result.newErrorResult(
                                     Error.error(ErrorCode.X_42595), sql);
                             }
 
-                            Expression filterExpr =
-                                right.getFilterExpression();
+                            Expression[] filters = right.getFiltersArray();
 
-                            if (filterExpr != null) {
-                                filterExpr = filterExpr.duplicate();
+                            // only check the expression resolves
+                            for (int i = 0; i < filters.length; i++) {
+                                if (filters[i] != null) {
+                                    Expression expr = filters[i].duplicate();
 
-                                RangeGroup ranges =
-                                    new RangeGroupSimple(t.getDefaultRanges(),
-                                                         false);
-
-                                filterExpr.resolveColumnReferences(
-                                    session, ranges, RangeGroup.emptyArray,
-                                    null);
+                                    expr.resolveGrantFilter(session, t);
+                                }
                             }
                         }
                     }
@@ -1196,12 +1240,6 @@ public class StatementSchema extends Statement {
                         Schema schema = schemaManager.findSchema(name.name);
 
                         this.sql = schema.getSQL();
-
-                        if (session.isProcessingScript()
-                                && session.database.getProperties()
-                                    .isVersion18()) {
-                            session.setCurrentSchemaHsqlName(schema.getName());
-                        }
                     }
 
                     break;
@@ -1317,7 +1355,7 @@ public class StatementSchema extends Statement {
                     table.compile(session, null);
                     schemaManager.addSchemaObject(table);
 
-                    if (tempIndexes != null && !tempIndexes.isEmpty()) {
+                    if (!tempIndexes.isEmpty()) {
                         TableWorks tableWorks = new TableWorks(session, table);
 
                         for (int i = 0; i < tempIndexes.size(); i++) {
@@ -1356,6 +1394,7 @@ public class StatementSchema extends Statement {
                 } catch (HsqlException e) {
                     schemaManager.removeExportedKeys(table);
                     schemaManager.removeDependentObjects(table.getName());
+                    schemaManager.removeSchemaObject(table.getName());
 
                     return Result.newErrorResult(e, sql);
                 }
@@ -1555,32 +1594,21 @@ public class StatementSchema extends Statement {
 
                         break;
                     }
-                    case SchemaObject.ROUTINE : {
-                        RoutineSchema routine =
-                            (RoutineSchema) schemaManager.getSchemaObject(
-                                name.name, name.schema.name,
-                                SchemaObject.ROUTINE);
-
-                        if (!session.getGrantee().isFullyAccessibleByRole(
-                                routine.getName())) {
-                            throw Error.error(ErrorCode.X_42501);
-                        }
-
-                        routine.getName().comment = comment;
-
-                        break;
-                    }
+                    case SchemaObject.TRIGGER :
+                    case SchemaObject.SEQUENCE :
+                    case SchemaObject.ROUTINE :
                     case SchemaObject.TABLE : {
-                        Table table =
-                            (Table) schemaManager.getSchemaObject(name.name,
-                                name.schema.name, SchemaObject.TABLE);
+                        SchemaObject object =
+                            schemaManager.getSchemaObject(name.name,
+                                                          name.schema.name,
+                                                          name.type);
 
                         if (!session.getGrantee().isFullyAccessibleByRole(
-                                table.getName())) {
+                                object.getName())) {
                             throw Error.error(ErrorCode.X_42501);
                         }
 
-                        table.getName().comment = comment;
+                        object.getName().comment = comment;
 
                         break;
                     }
@@ -1627,15 +1655,14 @@ public class StatementSchema extends Statement {
                               objectName.getSchemaQualifiedStatementName());
         }
 
-        Constraint[] constraints = domain.userTypeModifier.getConstraints();
-
-        set = new OrderedHashSet();
+        Constraint[]   constraints = domain.userTypeModifier.getConstraints();
+        OrderedHashSet constraintNames = new OrderedHashSet();
 
         for (int i = 0; i < constraints.length; i++) {
-            set.add(constraints[i].getName());
+            constraintNames.add(constraints[i].getName());
         }
 
-        session.database.schemaManager.removeSchemaObjects(set);
+        session.database.schemaManager.removeSchemaObjects(constraintNames);
         session.database.schemaManager.removeSchemaObject(domain.getName(),
                 cascade);
 
