@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -64,7 +64,7 @@ import org.hsqldb.types.Type;
  * Holds the data structures and methods for creation of a named database table.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.6.1
  */
 public class Table extends TableBase implements SchemaObject {
@@ -139,18 +139,17 @@ public class Table extends TableBase implements SchemaObject {
                 break;
 
             case CACHED_TABLE :
+                persistenceScope = SCOPE_FULL;
+                isSchemaBased    = true;
+
                 if (database.logger.isFileDatabase()) {
-                    persistenceScope = SCOPE_FULL;
-                    isSchemaBased    = true;
-                    isCached         = true;
-                    isLogged         = !database.isFilesReadOnly();
-
-                    break;
+                    isCached = true;
+                    isLogged = !database.isFilesReadOnly();
+                } else {
+                    type = MEMORY_TABLE;
                 }
+                break;
 
-                type = MEMORY_TABLE;
-
-            // fall through
             case MEMORY_TABLE :
                 persistenceScope = SCOPE_FULL;
                 isSchemaBased    = true;
@@ -263,13 +262,6 @@ public class Table extends TableBase implements SchemaObject {
         return tableName;
     }
 
-    /**
-     * Returns the catalog name or null, depending on a database property.
-     */
-    public HsqlName getCatalogName() {
-        return database.getCatalogName();
-    }
-
     /**
      * Returns the schema name.
      */
@@ -277,6 +269,13 @@ public class Table extends TableBase implements SchemaObject {
         return tableName.schema;
     }
 
+    /**
+     * Returns the catalog name
+     */
+    public HsqlName getCatalogName() {
+        return database.getCatalogName();
+    }
+
     public Grantee getOwner() {
         return tableName.schema.owner;
     }
@@ -292,13 +291,20 @@ public class Table extends TableBase implements SchemaObject {
         return set;
     }
 
-    public RangeVariable[] getDefaultRanges() {
+    public OrderedHashSet getReferencesForScript() {
 
-        if (defaultRanges == null) {
-            defaultRanges = new RangeVariable[]{ new RangeVariable(this, 0) };
+        OrderedHashSet set = getReferences();
+
+        for (int i = 0; i < colTypes.length; i++) {
+            ColumnSchema   column = getColumn(i);
+            OrderedHashSet refs   = column.getReferences();
+
+            if (refs != null && !refs.isEmpty()) {
+                set.addAll(refs);
+            }
         }
 
-        return defaultRanges;
+        return set;
     }
 
     public OrderedHashSet getReferencesForDependents() {
@@ -518,6 +524,15 @@ public class Table extends TableBase implements SchemaObject {
         return changeTimestamp;
     }
 
+    public RangeVariable[] getDefaultRanges() {
+
+        if (defaultRanges == null) {
+            defaultRanges = new RangeVariable[]{ new RangeVariable(this, 0) };
+        }
+
+        return defaultRanges;
+    }
+
     public final void setName(HsqlName name) {
         tableName = name;
     }
@@ -726,6 +741,23 @@ public class Table extends TableBase implements SchemaObject {
         return sb.toString();
     }
 
+    public void setForwardConstraints(OrderedHashSet resolved) {
+
+        for (int i = 0; i < constraintList.length; i++) {
+            Constraint c = constraintList[i];
+
+            if (c.getConstraintType()
+                    == SchemaObject.ConstraintTypes.FOREIGN_KEY) {
+                Table mainTable = c.getMain();
+
+                if (mainTable != this
+                        && !resolved.contains(mainTable.getName())) {
+                    c.isForward = true;
+                }
+            }
+        }
+    }
+
     public boolean isConnected() {
         return true;
     }
@@ -1312,7 +1344,7 @@ public class Table extends TableBase implements SchemaObject {
         tn.applicationPeriod = applicationPeriod;
         tn.isSystemVersioned = isSystemVersioned;
 
-        if (tableType == TEMP_TABLE) {
+        if (isTemp) {
             tn.persistenceScope = persistenceScope;
         }
 
@@ -1643,9 +1675,9 @@ public class Table extends TableBase implements SchemaObject {
     }
 
     /**
-     * sets the flag for the presence of any default expression
+     * recreates various flags based on column properties
      */
-    void resetDefaultsFlag() {
+    void resetDefaultFlags() {
 
         hasDefaultValues   = false;
         hasGeneratedValues = false;
@@ -1655,6 +1687,12 @@ public class Table extends TableBase implements SchemaObject {
         hasLobColumn       = false;
 
         for (int i = 0; i < columnCount; i++) {
+            ColumnSchema column = getColumn(i);
+
+            colNotNull[i]       = column.isPrimaryKey() || !column.isNullable();
+            colDefaults[i]     = column.getDefaultExpression();
+            colGenerated[i]    = column.isGenerated();
+            colUpdated[i]      = column.isAutoUpdate();
             hasDefaultValues   |= colDefaults[i] != null;
             hasGeneratedValues |= colGenerated[i];
             hasUpdatedValues   |= colUpdated[i];
@@ -1824,12 +1862,12 @@ public class Table extends TableBase implements SchemaObject {
             setSingleColumnTypeVars(i);
         }
 
-        resetDefaultsFlag();
+        resetDefaultFlags();
     }
 
     void setColumnTypeVars(int i) {
         setSingleColumnTypeVars(i);
-        resetDefaultsFlag();
+        resetDefaultFlags();
     }
 
     private void setSingleColumnTypeVars(int i) {
@@ -1868,7 +1906,7 @@ public class Table extends TableBase implements SchemaObject {
     /**
      * Returns direct mapping array.
      */
-    int[] getColumnMap() {
+    public int[] getColumnMap() {
         return defaultColumnMap;
     }
 
@@ -1982,6 +2020,17 @@ public class Table extends TableBase implements SchemaObject {
         return set;
     }
 
+    public String[] getColumnLabels() {
+
+        String[] labels = new String[columnCount];
+
+        for (int i = 0; i < columnCount; i++) {
+            labels[i] = ((ColumnSchema) columnList.get(i)).getName().name;
+        }
+
+        return labels;
+    }
+
     /**
      * Returns array for a new row with SQL DEFAULT value for each column n
      * where exists[n] is false. This provides default values only where
@@ -2822,6 +2871,23 @@ public class Table extends TableBase implements SchemaObject {
         }
     }
 
+    /**
+     * for default values (not expressions)
+     */
+    public void generateDefaultForNull(Object[] data) {
+
+        if (hasNotNullColumns) {
+            for (int i = 0; i < colDefaults.length; i++) {
+                if (data[i] == null) {
+                    if (colNotNull[i] && colDefaults[i] != null
+                            && colDefaults[i].getType() == OpTypes.VALUE) {
+                        data[i] = colDefaults[i].getValue(null);
+                    }
+                }
+            }
+        }
+    }
+
     /**
      *  Mid level method for inserting single rows. Performs constraint checks and
      *  fires row level triggers.
@@ -2850,7 +2916,7 @@ public class Table extends TableBase implements SchemaObject {
     Row insertSystemVersionHistoryRow(Session session, PersistentStore store,
                                       Object[] data) {
 
-        TimestampData txTimestamp = session.getTransactionSystemTimestamp();
+        TimestampData txTimestamp = session.getTransactionUTC();
 
         if (txTimestamp.equals(data[systemPeriodStartColumn])) {
             return null;
@@ -2862,7 +2928,8 @@ public class Table extends TableBase implements SchemaObject {
 
         Row row = (Row) store.getNewCachedObject(session, newData, true);
 
-        session.database.txManager.addInsertAction(session, store, row);
+        session.database.txManager.addInsertAction(session, this, store, row,
+                null);
 
         return row;
     }
@@ -2880,6 +2947,11 @@ public class Table extends TableBase implements SchemaObject {
             Object[] newData =
                 (Object[]) ArrayUtil.resizeArrayIfDifferent(data, columnCount);
 
+            for (int i = 0; i < newData.length; i++) {
+                newData[i] = colTypes[i].convertToTypeLimits(session,
+                        newData[i]);
+            }
+
             insertData(session, store, newData, true);
         }
     }
@@ -2900,12 +2972,7 @@ public class Table extends TableBase implements SchemaObject {
                 DateTimeType.epochLimitTimestamp);
         }
 
-        if (enforceUnique) {
-            database.txManager.addInsertAction(session, this, store, row,
-                                               null);
-        } else {
-            database.txManager.addInsertAction(session, store, row);
-        }
+        database.txManager.addInsertAction(session, this, store, row, null);
     }
 
     /**
@@ -2955,40 +3022,6 @@ public class Table extends TableBase implements SchemaObject {
 
         systemUpdateIdentityValue(data);
 
-        if (session.database.getProperties().isVersion18()) {
-            for (int i = 0; i < columnCount; i++) {
-                if (data[i] != null) {
-                    int length;
-
-                    if (colTypes[i].isCharacterType()
-                            || colTypes[i].isBinaryType()) {
-                        if (data[i] instanceof String) {
-                            length = ((String) data[i]).length();
-                        } else if (data[i] instanceof BinaryData) {
-                            length =
-                                (int) ((BinaryData) data[i]).length(session);
-                        } else {
-                            throw Error.runtimeError(ErrorCode.X_07000,
-                                                     "Table");
-                        }
-
-                        if (length > colTypes[i].precision) {
-                            length = ((length / 10) + 1) * 10;
-                            colTypes[i] =
-                                Type.getType(colTypes[i].typeCode,
-                                             colTypes[i].getCharacterSet(),
-                                             colTypes[i].getCollation(),
-                                             length, 0);
-
-                            ColumnSchema column = getColumn(i);
-
-                            column.setType(colTypes[i]);
-                        }
-                    }
-                }
-            }
-        }
-
         if (isSystemVersioned) {
             enforceUnique = data[systemPeriodEndColumn].equals(
                 DateTimeType.epochLimitTimestamp);
@@ -3066,7 +3099,7 @@ public class Table extends TableBase implements SchemaObject {
             switch (column.getSystemPeriodType()) {
 
                 case SchemaObject.PeriodSystemColumnType.PERIOD_ROW_START :
-                    value = session.getTransactionSystemTimestamp();
+                    value = session.getTransactionUTC();
                     break;
 
                 case SchemaObject.PeriodSystemColumnType.PERIOD_ROW_END :
@@ -3099,9 +3132,8 @@ public class Table extends TableBase implements SchemaObject {
         }
 
         if (systemPeriod != null) {
-            data[systemPeriodStartColumn] =
-                session.getTransactionSystemTimestamp();
-            data[systemPeriodEndColumn] = DateTimeType.epochLimitTimestamp;
+            data[systemPeriodStartColumn] = session.getTransactionUTC();
+            data[systemPeriodEndColumn]   = DateTimeType.epochLimitTimestamp;
         }
     }
 
