@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.types.Types;
  * existing table which may result in a new Table object
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.0
  */
 public class TableWorks {
@@ -65,6 +65,10 @@ public class TableWorks {
         this.database = table.database;
         this.table    = table;
         this.session  = session;
+
+        if (table.isView()) {
+            throw Error.error(ErrorCode.X_42524);
+        }
     }
 
     public Table getTable() {
@@ -162,8 +166,8 @@ public class TableWorks {
         }
 
         // check after UNIQUE check
-        c.core.mainTable.checkReferentialColumnsMatch(c.core.mainCols, fkTable,
-                c.core.refCols);
+        c.core.mainTable.checkReferentialColumnsMatch(c.core.mainCols,
+                fkTable, c.core.refCols);
         ArrayUtil.reorderMaps(unique.getMainColumns(), c.getMainColumns(),
                               c.getRefColumns());
 
@@ -587,7 +591,7 @@ public class TableWorks {
 
         indexes[position] = newIndex;
 
-        store.reindex(session, newIndex);
+        store.reindex(session, newIndex, null);
         database.schemaManager.recompileDependentObjects(table);
     }
 
@@ -1149,7 +1153,7 @@ public class TableWorks {
 
         int colIndex = table.getColumnIndex(oldCol.getName().name);
 
-        // 0 if only metadata change is required ; 1 if only check is required ; -1 if data conversion is required
+        // 0 if only metadata change is required ; 1 only range check is required ; -1 data conversion is required
         int checkData = newType.canMoveFrom(oldType);
 
         if (newCol.isIdentity() && table.hasIdentityColumn()
@@ -1164,6 +1168,11 @@ public class TableWorks {
                     checkData = 1;
                 }
             }
+
+            if (newType.isDomainType()
+                    && newType.userTypeModifier.getConstraints().length > 0) {
+                checkData = 1;
+            }
         }
 
         if (checkData == 1) {
@@ -1181,7 +1190,7 @@ public class TableWorks {
             oldCol.setDefaultExpression(newCol.getDefaultExpression());
             oldCol.setIdentity(newCol.getIdentitySequence());
             table.setColumnTypeVars(colIndex);
-            table.resetDefaultsFlag();
+            table.resetDefaultFlags();
 
             return;
         }
@@ -1202,18 +1211,53 @@ public class TableWorks {
     void checkConvertColDataType(ColumnSchema oldCol, ColumnSchema newCol) {
 
         int         colIndex = table.getColumnIndex(oldCol.getName().name);
+        Type        oldType  = oldCol.getDataType();
+        Type        newType  = newCol.getDataType();
         RowIterator it       = table.rowIterator(session);
 
         while (it.next()) {
             Row    row = it.getCurrentRow();
-            Object o   = row.getData()[colIndex];
+            Object o   = row.getField(colIndex);
 
             if (!newCol.isNullable() && o == null) {
                 throw Error.error(ErrorCode.X_23502);
             }
 
-            newCol.getDataType().convertToType(session, o,
-                                               oldCol.getDataType());
+            newType.convertToType(session, o, oldType);
+
+            if (newType.isDomainType()) {
+                Constraint[] checks =
+                    newType.userTypeModifier.getConstraints();
+
+                for (int i = 0; i < checks.length; i++) {
+                    checks[i].checkCheckConstraint(session, table, oldCol, o);
+                    checkAddDomainConstraint(newType, checks[i]);
+                }
+            }
+        }
+    }
+
+    /**
+     *
+     * @param domain domain
+     * @param check  added constraint
+     */
+    void checkAddDomainConstraint(Type domain, Constraint check) {
+
+        Type[]      dataTypes = table.getColumnTypes();
+        RowIterator it        = table.rowIterator(session);
+
+        while (it.next()) {
+            Row row = it.getCurrentRow();
+
+            for (int i = 0; i < table.getColumnCount(); i++) {
+                if (dataTypes[i] == domain) {
+                    ColumnSchema column = table.getColumn(i);
+
+                    check.checkCheckConstraint(session, table, column,
+                                               row.getField(i));
+                }
+            }
         }
     }
 
@@ -1524,9 +1568,7 @@ public class TableWorks {
 
     void moveData(Table oldTable, Table newTable, int[] colIndex, int adjust) {
 
-        int tableType = oldTable.getTableType();
-
-        if (tableType == Table.TEMP_TABLE) {
+        if (oldTable.isTemp) {
             Session[] sessions = database.sessionManager.getAllSessions();
 
             for (int i = 0; i < sessions.length; i++) {
