@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,7 @@ import org.hsqldb.persist.PersistentStore;
  * Manages rows involved in transactions
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 2.0.0
  */
 public class TransactionManagerMVCC extends TransactionManagerCommon
@@ -309,7 +309,7 @@ implements TransactionManager {
             try {
                 action.rollback(session, timestamp);
 
-                int type = action.mergeRollback(session, timestamp, row);
+                int type = action.mergeRollback(session, timestamp);
 
                 if (action.type == RowActionBase.ACTION_DELETE_FINAL) {
                     if (action.deleteComplete) {
@@ -330,10 +330,21 @@ implements TransactionManager {
 
     public RowAction addDeleteAction(Session session, Table table,
                                      PersistentStore store, Row row,
-                                     int[] colMap) {
+                                     int[] changedColumns) {
+
+        RowAction action = store.addDeleteActionToRow(session, row,
+            changedColumns, true);
+
+        if (table.isTemp) {
+            store.delete(session, row);
+
+            row.rowAction = null;
+
+            if (table.persistenceScope == Table.SCOPE_ROUTINE) {
+                return action;
+            }
+        }
 
-        RowAction action = addDeleteActionToRow(session, table, store, row,
-            colMap);
         Session actionSession = null;
         boolean redoAction    = true;
 
@@ -431,10 +442,12 @@ implements TransactionManager {
         }
 
         if (!redoAction) {
-            if (table.persistenceScope == Table.SCOPE_ROUTINE) {
+            if (table.isTemp) {
                 row.rowAction = null;
 
-                return;
+                if (table.persistenceScope == Table.SCOPE_ROUTINE) {
+                    return;
+                }
             }
 
             session.rowActionList.add(action);
@@ -490,20 +503,6 @@ implements TransactionManager {
         }
     }
 
-    public void addInsertAction(Session session, PersistentStore store,
-                                Row row) {
-
-        RowAction action = row.rowAction;
-
-        if (action == null) {
-            throw Error.runtimeError(ErrorCode.GENERAL_ERROR,
-                                     "null insert action ");
-        }
-
-        store.indexRow(session, row);
-        session.rowActionList.add(action);
-    }
-
 // functional unit - accessibility of rows
     public boolean canRead(Session session, PersistentStore store, Row row,
                            int mode, int[] colMap) {
@@ -512,7 +511,7 @@ implements TransactionManager {
 
         if (action == null) {
             return true;
-        } else if (action.table.tableType == TableBase.TEMP_TABLE) {
+        } else if (action.table.isTemp) {
             return true;
         }
 
@@ -713,22 +712,6 @@ implements TransactionManager {
         }
     }
 
-    RowAction addDeleteActionToRow(Session session, Table table,
-                                   PersistentStore store, Row row,
-                                   int[] colMap) {
-
-        RowAction action = store.addDeleteActionToRow(session, row, colMap,
-            true);
-
-        if (table.tableType == TableBase.TEMP_TABLE) {
-            store.delete(session, row);
-
-            row.rowAction = null;
-        }
-
-        return action;
-    }
-
     /**
      * remove session from queue when a transaction ends
      * and expire any committed transactions
@@ -858,7 +841,7 @@ implements TransactionManager {
     }
 
     public void resetSession(Session session, Session targetSession,
-                             int mode) {
-        super.resetSession(session, targetSession, mode);
+                             long statementTimestamp, int mode) {
+        super.resetSession(session, targetSession, statementTimestamp, mode);
     }
 }
