@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@ import org.hsqldb.types.Type;
 /**
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public interface Index extends SchemaObject {
@@ -63,8 +63,6 @@ public interface Index extends SchemaObject {
 
     IndexUse[] asArray();
 
-    RowIterator emptyIterator();
-
     int getPosition();
 
     void setPosition(int position);
@@ -93,21 +91,29 @@ public interface Index extends SchemaObject {
 
     /**
      * Returns the array containing column indexes for index
+     *
+     * @return int[]
      */
     int[] getColumns();
 
     /**
      * Returns the array containing column indexes for index
+     *
+     * @return Type[]
      */
     Type[] getColumnTypes();
 
     /**
      * Returns the count of visible columns used
+     *
+     * @return boolean[]
      */
     boolean[] getColumnDesc();
 
     /**
      * Returns the array containing 0, 1, .. column indexes
+     *
+     * @return int[]
      */
     int[] getDefaultColumnMap();
 
@@ -150,17 +156,12 @@ public interface Index extends SchemaObject {
 
     double[] searchCost(Session session, PersistentStore store);
 
-    long getNodeCount(Session session, PersistentStore store);
-
     boolean isEmpty(PersistentStore store);
 
-    int checkIndex(Session session, PersistentStore store);
-
     /**
      * Insert a node into the index
      */
-    void insert(Session session, PersistentStore store, Row row
-                );
+    void insert(Session session, PersistentStore store, Row row);
 
     void delete(Session session, PersistentStore store, Row row);
 
@@ -168,12 +169,17 @@ public interface Index extends SchemaObject {
                          Object[] rowdata, int[] rowColMap);
 
     /**
-     * Return the first node equal to the indexdata object. The rowdata has
-     * the same column mapping as this index.
+     * Return the first node equal to the indexdata object. The rowdata has the
+     * same column mapping as this index.
      *
      * @param session session object
      * @param store store object
+     * @param rowdata Object[]
      * @param matchCount count of columns to match
+     * @param distinctCount int
+     * @param compareType int
+     * @param reversed boolean
+     * @param map boolean[]
      * @return iterator
      */
     RowIterator findFirstRow(Session session, PersistentStore store,
@@ -194,12 +200,13 @@ public interface Index extends SchemaObject {
                              Object[] rowdata);
 
     /**
-     * Return the first node equal to the rowdata object.
-     * The rowdata has the column mapping provided in rowColMap.
+     * Return the first node equal to the rowdata object. The rowdata has the
+     * column mapping provided in rowColMap.
      *
      * @param session session object
      * @param store store object
      * @param rowdata array containing table row data
+     * @param rowColMap int[]
      * @return iterator
      */
     RowIterator findFirstRow(Session session, PersistentStore store,
@@ -209,6 +216,8 @@ public interface Index extends SchemaObject {
      * Finds the first node where the data is not null.
      *
      * @return iterator
+     * @param session Session
+     * @param store PersistentStore
      */
     RowIterator findFirstRowNotNull(Session session, PersistentStore store);
 
@@ -227,20 +236,24 @@ public interface Index extends SchemaObject {
      * Returns the row for the last node of the index
      *
      * @return last row
+     * @param session Session
+     * @param store PersistentStore
+     * @param distinctCount int
+     * @param map boolean[]
      */
     RowIterator lastRow(Session session, PersistentStore store,
-                        int distinctCount, boolean[] map);
+                        int distinctCount, boolean[] usedColumnCheck);
 
     /**
      * Compares two table rows based on the columns of this index. The rowColMap
      * parameter specifies which columns of the other table are to be compared
-     * with the colIndex columns of this index. The rowColMap can cover all
-     * or only some columns of this index.
+     * with the colIndex columns of this index. The rowColMap can cover all or
+     * only some columns of this index.
      *
+     * @param session Session
      * @param a row from another table
-     * @param rowColMap column indexes in the other table
      * @param b a full row in this table
-     *
+     * @param rowColMap column indexes in the other table
      * @return comparison result, -1,0,+1
      */
     int compareRowNonUnique(Session session, Object[] a, Object[] b,
