@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2019, The HSQL Development Group
+ * Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -86,6 +86,7 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.OrderedHashSet;
+import org.hsqldb.navigator.RangeIterator;
 import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.rights.Grantee;
@@ -111,14 +112,11 @@ import org.hsqldb.types.Type;
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since Hypersonic SQL
  */
 public class IndexAVL implements Index {
 
-    private static final IndexRowIterator emptyIterator =
-        new IndexRowIterator(null, null, null, null, 0, false, false);
-
     // fields
     private final long       persistenceId;
     protected final HsqlName name;
@@ -207,14 +205,14 @@ public class IndexAVL implements Index {
         return name;
     }
 
-    public HsqlName getCatalogName() {
-        return name.schema.schema;
-    }
-
     public HsqlName getSchemaName() {
         return name.schema;
     }
 
+    public HsqlName getCatalogName() {
+        return name.schema.schema;
+    }
+
     public Grantee getOwner() {
         return name.schema.owner;
     }
@@ -257,10 +255,6 @@ public class IndexAVL implements Index {
         return asArray;
     }
 
-    public RowIterator emptyIterator() {
-        return emptyIterator;
-    }
-
     public int getPosition() {
         return position;
     }
@@ -380,7 +374,15 @@ public class IndexAVL implements Index {
      * Returns the node count.
      */
     public long size(Session session, PersistentStore store) {
-        return getNodeCount(session, store);
+
+        long        count = 0;
+        RowIterator it    = firstRow(session, store, null, 0, null);
+
+        while (it.next()) {
+            count++;
+        }
+
+        return count;
     }
 
     public long sizeUnique(PersistentStore store) {
@@ -506,18 +508,6 @@ public class IndexAVL implements Index {
         return depth - probeDepth;
     }
 
-    public long getNodeCount(Session session, PersistentStore store) {
-
-        long        count = 0;
-        RowIterator it    = firstRow(session, store, null, 0, null);
-
-        while (it.next()) {
-            count++;
-        }
-
-        return count;
-    }
-
     public boolean isEmpty(PersistentStore store) {
 
         store.readLock();
@@ -589,93 +579,6 @@ public class IndexAVL implements Index {
         return x;
     }
 
-    public int checkIndex(Session session, PersistentStore store) {
-
-        int errors = 0;
-
-        store.readLock();
-
-        try {
-            NodeAVL p = getAccessor(store);
-            NodeAVL f = null;
-
-            while (p != null) {
-                f = p;
-
-                checkNodes(store, p);
-
-                p = p.getLeft(store);
-            }
-
-            p = f;
-
-            while (f != null) {
-                errors += checkNodes(store, f);
-
-                NodeAVL fnext = next(store, f);
-
-                if (fnext != null) {
-                    int c = compareRowForInsertOrDelete(session,
-                                                        fnext.getRow(store),
-                                                        f.getRow(store), true,
-                                                        0);
-
-                    if (c <= 0) {
-                        if (errors < 10) {
-                            System.out.println("broken index order "
-                                               + getName().name);
-                        }
-
-                        errors++;
-                    }
-                }
-
-                f = fnext;
-            }
-        } finally {
-            store.readUnlock();
-        }
-
-        if (errors > 0) {
-            System.out.println("total errors " + getName().name);
-        }
-
-        return errors;
-    }
-
-    int checkNodes(PersistentStore store, NodeAVL p) {
-
-        NodeAVL l      = p.nLeft;
-        NodeAVL r      = p.nRight;
-        int     errors = 0;
-
-        if (l != null && l.getBalance(store) == -2) {
-            System.out.print("broken index - deleted");
-
-            errors++;
-        }
-
-        if (r != null && r.getBalance(store) == -2) {
-            System.out.print("broken index -deleted");
-
-            errors++;
-        }
-
-        if (l != null && !p.equals(l.getParent(store))) {
-            System.out.print("broken index - no parent");
-
-            errors++;
-        }
-
-        if (r != null && !p.equals(r.getParent(store))) {
-            System.out.print("broken index - no parent");
-
-            errors++;
-        }
-
-        return errors;
-    }
-
     /**
      * Compares two table rows based on the columns of this index. The rowColMap
      * parameter specifies which columns of the other table are to be compared
@@ -1138,6 +1041,7 @@ public class IndexAVL implements Index {
      * @param store store object
      * @param rowdata array containing index column data
      * @param matchCount count of columns to match
+     * @param distinctCount int
      * @param compareType int
      * @param reversed boolean
      * @param map boolean[]
@@ -1153,7 +1057,7 @@ public class IndexAVL implements Index {
                                 TransactionManager.ACTION_READ, reversed);
 
         if (node == null) {
-            return emptyIterator;
+            return RangeIterator.emptyRowIterator;
         }
 
         return new IndexRowIterator(session, store, this, node, distinctCount,
@@ -1177,7 +1081,7 @@ public class IndexAVL implements Index {
                                 TransactionManager.ACTION_READ, false);
 
         if (node == null) {
-            return emptyIterator;
+            return RangeIterator.emptyRowIterator;
         }
 
         return new IndexRowIterator(session, store, this, node, 0, false,
@@ -1202,7 +1106,7 @@ public class IndexAVL implements Index {
                                 TransactionManager.ACTION_READ, false);
 
         if (node == null) {
-            return emptyIterator;
+            return RangeIterator.emptyRowIterator;
         }
 
         return new IndexRowIterator(session, store, this, node, 0, false,
@@ -1213,6 +1117,8 @@ public class IndexAVL implements Index {
      * Finds the first node where the data is not null.
      *
      * @return iterator
+     * @param session Session
+     * @param store PersistentStore
      */
     public RowIterator findFirstRowNotNull(Session session,
                                            PersistentStore store) {
@@ -1222,7 +1128,7 @@ public class IndexAVL implements Index {
                                 TransactionManager.ACTION_READ, false);
 
         if (node == null) {
-            return emptyIterator;
+            return RangeIterator.emptyRowIterator;
         }
 
         return new IndexRowIterator(session, store, this, node, 0, false,
@@ -1261,7 +1167,7 @@ public class IndexAVL implements Index {
             }
 
             if (x == null) {
-                return emptyIterator;
+                return RangeIterator.emptyRowIterator;
             }
 
             return new IndexRowIterator(session, store, this, x,
@@ -1285,7 +1191,7 @@ public class IndexAVL implements Index {
             }
 
             if (x == null) {
-                return emptyIterator;
+                return RangeIterator.emptyRowIterator;
             }
 
             return new IndexRowIterator(null, store, this, x, 0, false, false);
@@ -1325,7 +1231,7 @@ public class IndexAVL implements Index {
             }
 
             if (x == null) {
-                return emptyIterator;
+                return RangeIterator.emptyRowIterator;
             }
 
             return new IndexRowIterator(session, store, this, x,
@@ -1896,11 +1802,11 @@ public class IndexAVL implements Index {
         return node;
     }
 
-    IndexRowIterator getIterator(Session session, PersistentStore store,
+    RowIterator getIterator(Session session, PersistentStore store,
                                  NodeAVL x, boolean single, boolean reversed) {
 
         if (x == null) {
-            return emptyIterator;
+            return RangeIterator.emptyRowIterator;
         } else {
             IndexRowIterator it = new IndexRowIterator(session, store, this,
                 x, 0, single, reversed);
