@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2019, The HSQL Development Group
+ * Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -97,7 +97,7 @@ import org.hsqldb.rowio.RowOutputInterface;
  *
  * @author Fred Toussi (fredt@users dot sourceforge dot net)
  * @author Thomas Mueller (Hypersonic SQL Group)
- * @version 2.3.5
+ * @version 2.4.0
  * @since Hypersonic SQL
  */
 public class NodeAVLDisk extends NodeAVL {
@@ -472,4 +472,17 @@ public class NodeAVLDisk extends NodeAVL {
     public boolean keepInMemory(boolean keep) {
         return false;
     }
+
+    long getLeftPos() {
+        return iLeft;
+    }
+
+    long getRightPos() {
+        return iRight;
+    }
+
+    long getParentPos() {
+        return iParent;
+    }
+
 }
