@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,8 @@
 
 package org.hsqldb.map;
 
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.Arrays;
 import java.util.NoSuchElementException;
 
 import org.hsqldb.lib.ArrayCounter;
@@ -46,7 +48,7 @@ import org.hsqldb.lib.ObjectComparator;
  * Special getOrAddXXX() methods are used for object maps in some subclasses.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.2
  */
 public class BaseHashMap {
@@ -110,11 +112,11 @@ public class BaseHashMap {
     protected long[]   longValueTable;
 
     //
-    protected int       accessMin;
-    protected int       accessCount;
-    protected int[]     accessTable;
-    protected boolean[] multiValueTable;
-    protected Object[]  objectValueTable2;
+    protected int           accessMin;
+    protected AtomicInteger accessCount;
+    protected int[]         accessTable;
+    protected boolean[]     multiValueTable;
+    protected Object[]      objectValueTable2;
 
     //
     final float                loadFactor;
@@ -198,6 +200,7 @@ public class BaseHashMap {
 
         if (hasAccessCount) {
             accessTable = new int[arraySize];
+            accessCount = new AtomicInteger();
         }
     }
 
@@ -415,7 +418,7 @@ public class BaseHashMap {
             }
 
             if (isLastAccessCount) {
-                accessTable[lookup] = ++accessCount;
+                accessTable[lookup] = accessCount.incrementAndGet();
             } else if (isAccessCount) {
                 accessTable[lookup]++;
             }
@@ -468,7 +471,7 @@ public class BaseHashMap {
 
         //
         if (isLastAccessCount) {
-            accessTable[lookup] = ++accessCount;
+            accessTable[lookup] = accessCount.incrementAndGet();
         } else if (isAccessCount) {
             accessTable[lookup] = 1;
         }
@@ -668,7 +671,7 @@ public class BaseHashMap {
 
         //
         if (isLastAccessCount) {
-            accessTable[lookup] = ++accessCount;
+            accessTable[lookup] = accessCount.incrementAndGet();
         } else if (isAccessCount) {
             accessTable[lookup] = 1;
         }
@@ -741,7 +744,7 @@ public class BaseHashMap {
             }
 
             if (isLastAccessCount) {
-                accessTable[lookup] = ++accessCount;
+                accessTable[lookup] = accessCount.incrementAndGet();
             } else if (isAccessCount) {
                 accessTable[lookup]++;
             }
@@ -783,7 +786,7 @@ public class BaseHashMap {
         }
 
         if (isLastAccessCount) {
-            accessTable[lookup] = ++accessCount;
+            accessTable[lookup] = accessCount.incrementAndGet();
         } else if (isAccessCount) {
             accessTable[lookup] = 1;
         }
@@ -876,7 +879,7 @@ public class BaseHashMap {
                 objectKeyTable[lookup] = object;
 
                 if (isLastAccessCount) {
-                    accessTable[lookup] = ++accessCount;
+                    accessTable[lookup] = accessCount.incrementAndGet();
                 } else if (isAccessCount) {
                     accessTable[lookup]++;
                 }
@@ -899,7 +902,7 @@ public class BaseHashMap {
         objectKeyTable[lookup] = object;
 
         if (isLastAccessCount) {
-            accessTable[lookup] = ++accessCount;
+            accessTable[lookup] = accessCount.incrementAndGet();
         } else if (isAccessCount) {
             accessTable[lookup] = 1;
         }
@@ -1081,60 +1084,32 @@ public class BaseHashMap {
      */
     private void clearElementArrays(final int from, final int to) {
 
-        if (isIntKey) {
-            int counter = to;
-
-            while (--counter >= from) {
-                intKeyTable[counter] = 0;
-            }
-        } else if (isLongKey) {
-            int counter = to;
-
-            while (--counter >= from) {
-                longKeyTable[counter] = 0;
-            }
-        } else if (isObjectKey || objectKeyTable != null) {
-            int counter = to;
-
-            while (--counter >= from) {
-                objectKeyTable[counter] = null;
-            }
+        if (intKeyTable != null) {
+            Arrays.fill(intKeyTable, from, to, 0);
+        } else if (longKeyTable != null) {
+            Arrays.fill(longKeyTable, from, to, 0);
+        } else if (objectKeyTable != null) {
+            Arrays.fill(objectKeyTable, from, to, null);
         }
 
-        if (isIntValue) {
-            int counter = to;
+        if (intValueTable != null) {
+            Arrays.fill(intValueTable, from, to, 0);
+        } else if (longValueTable != null) {
+            Arrays.fill(longValueTable, from, to, 0);
+        } else if (objectValueTable != null) {
+            Arrays.fill(objectValueTable, from, to, null);
+        }
 
-            while (--counter >= from) {
-                intValueTable[counter] = 0;
-            }
-        } else if (isLongValue) {
-            int counter = to;
-
-            while (--counter >= from) {
-                longValueTable[counter] = 0;
-            }
-        } else if (isObjectValue) {
-            int counter = to;
-
-            while (--counter >= from) {
-                objectValueTable[counter] = null;
-            }
+        if (objectValueTable2 != null) {
+            Arrays.fill(objectValueTable2, from, to, null);
         }
 
         if (accessTable != null) {
-            int counter = to;
-
-            while (--counter >= from) {
-                accessTable[counter] = 0;
-            }
+            Arrays.fill(accessTable, from, to, 0);
         }
 
         if (multiValueTable != null) {
-            int counter = to;
-
-            while (--counter >= from) {
-                multiValueTable[counter] = false;
-            }
+            Arrays.fill(multiValueTable, from, to, false);
         }
     }
 
@@ -1164,7 +1139,7 @@ public class BaseHashMap {
             longKeyTable[lastPointer] = 0;
         }
 
-        if (isObjectKey || objectKeyTable != null) {
+        if (objectKeyTable != null) {
             Object array = objectKeyTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
@@ -1274,8 +1249,11 @@ public class BaseHashMap {
     public void clear() {
 
         if (hashIndex.modified) {
-            accessCount  = 0;
-            accessMin    = accessCount;
+            if (accessCount != null) {
+                accessCount.set(0);
+            }
+
+            accessMin    = 0;
             hasZeroKey   = false;
             zeroKeyIndex = -1;
 
@@ -1294,7 +1272,7 @@ public class BaseHashMap {
      */
     protected int getAccessCountCeiling(int count, int margin) {
         return ArrayCounter.rank(accessTable, hashIndex.newNodePointer, count,
-                                 accessMin, accessCount, margin);
+                                 accessMin, accessCount.get(), margin);
     }
 
     /**
@@ -1305,10 +1283,6 @@ public class BaseHashMap {
         accessMin = count;
     }
 
-    protected int incrementAccessCount() {
-        return ++accessCount;
-    }
-
     /**
      * Clear approximately half elements from the map, starting with
      * those with low accessTable ranking.
@@ -1344,22 +1318,39 @@ public class BaseHashMap {
 
     protected void resetAccessCount() {
 
-        if (accessCount < ACCESS_MAX) {
+        int accessMax = accessCount.get();
+
+        if (accessMax > 0 && accessMax < ACCESS_MAX) {
             return;
         }
 
-        double factor = 0.5;
+        int limit = hashIndex.getNewNodePointer();
 
-        for (int i = 0; i < accessTable.length; i++) {
-            if (accessTable[i] < accessMin) {
-                accessTable[i] = 0;
-            } else {
-                accessTable[i] = (int) ((accessTable[i] - accessMin) * factor);
+        accessMax = 0;
+        accessMin = Integer.MAX_VALUE;
+
+        for (int i = 0; i < limit; i++) {
+            int access = accessTable[i];
+
+            if (access == 0) {
+                continue;
+            }
+
+            access         = (access >>> 2) + 1;
+            accessTable[i] = access;
+
+            if (access > accessMax) {
+                accessMax = access;
+            } else if (access < accessMin) {
+                accessMin = access;
             }
         }
 
-        accessCount = (int) ((accessCount - accessMin) * factor);
-        accessMin   = 0;
+        if (accessMin > accessMax) {
+            accessMin = accessMax;
+        }
+
+        accessCount.set(accessMax);
     }
 
     protected int capacity() {
