@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,7 +53,7 @@ import org.hsqldb.rowio.RowOutputInterface;
  * Implementation of RowSetNavigator for result sets.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class RowSetNavigatorData extends RowSetNavigator
@@ -81,8 +81,8 @@ implements Comparator<Object[]> {
     Index idIndex;
 
     //
-    TreeMap<Object[], Object[]>        rowMap;
-    LongKeyHashMap idMap;
+    TreeMap<Object[], Integer> rowMap;
+    LongKeyHashMap             idMap;
 
     RowSetNavigatorData(Session session) {
         this.session = session;
@@ -100,7 +100,7 @@ implements Comparator<Object[]> {
 
         if (select.isGrouped) {
             mainIndex = select.groupIndex;
-            rowMap    = new TreeMap<Object[], Object[]>(this);
+            rowMap    = new TreeMap<Object[], Integer>(this);
         }
 
         if (select.idIndex != null) {
@@ -129,7 +129,7 @@ implements Comparator<Object[]> {
         }
     }
 
-    public void sortFull(Session session) {
+    public void sortFull() {
 
         mainIndex = fullIndex;
 
@@ -137,7 +137,7 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void sortOrder(Session session) {
+    public void sortOrder() {
 
         if (orderIndex != null) {
             mainIndex = orderIndex;
@@ -148,7 +148,7 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void sortOrderUnion(Session session, SortAndSlice sortAndSlice) {
+    public void sortOrderUnion(SortAndSlice sortAndSlice) {
 
         if (sortAndSlice.index != null) {
             mainIndex = sortAndSlice.index;
@@ -164,10 +164,8 @@ implements Comparator<Object[]> {
 
         dataTable[size] = data;
 
-        size++;
-
         if (rowMap != null) {
-            rowMap.put(data, data);
+            rowMap.put(data, size);
         }
 
         if (idMap != null) {
@@ -175,6 +173,21 @@ implements Comparator<Object[]> {
 
             idMap.put(id.longValue(), data);
         }
+
+        size++;
+    }
+
+    public void setPosition(Object[] data) {
+
+        Integer mapPos = rowMap.get(data);
+
+        if (mapPos == null) {
+            return;
+        }
+
+        int pos = mapPos.intValue();
+
+        currentPos = pos;
     }
 
     public boolean addRow(Row row) {
@@ -228,6 +241,10 @@ implements Comparator<Object[]> {
         size++;
     }
 
+    public Object[][] getDataTable() {
+        return dataTable;
+    }
+
     public void release() {
 
         this.dataTable = emptyTable;
@@ -246,6 +263,10 @@ implements Comparator<Object[]> {
         reset();
     }
 
+    public void resetRowMap() {
+        rowMap = new TreeMap(this);
+    }
+
     public boolean absolute(int position) {
         return super.absolute(position);
     }
@@ -324,12 +345,12 @@ implements Comparator<Object[]> {
         }
     }
 
-    public void union(Session session, RowSetNavigatorData other) {
+    public void union(RowSetNavigatorData other) {
 
         Object[] currentData;
 
-        removeDuplicates(session);
-        other.removeDuplicates(session);
+        removeDuplicates();
+        other.removeDuplicates();
 
         mainIndex = fullIndex;
 
@@ -350,7 +371,7 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void unionAll(Session session, RowSetNavigatorData other) {
+    public void unionAll(RowSetNavigatorData other) {
 
         mainIndex = fullIndex;
 
@@ -365,10 +386,10 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void intersect(Session session, RowSetNavigatorData other) {
+    public void intersect(RowSetNavigatorData other) {
 
-        removeDuplicates(session);
-        other.sortFull(session);
+        removeDuplicates();
+        other.sortFull();
 
         while (next()) {
             Object[] currentData = getCurrent();
@@ -382,23 +403,23 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void intersectAll(Session session, RowSetNavigatorData other) {
+    public void intersectAll(RowSetNavigatorData other) {
 
         Object[]    compareData = null;
         RowIterator it;
         Object[]    otherData = null;
 
-        sortFull(session);
-        other.sortFull(session);
+        sortFull();
+        other.sortFull();
 
-        it = fullIndex.emptyIterator();
+        it = RowIterator.emptyRowIterator;
 
         while (next()) {
             Object[] currentData = getCurrent();
             boolean newGroup =
                 compareData == null
                 || fullIndex.compareRowNonUnique(
-                    session, currentData, compareData,
+                    (Session) session, currentData, compareData,
                     visibleColumnCount) != 0;
 
             if (newGroup) {
@@ -410,7 +431,7 @@ implements Comparator<Object[]> {
                 otherData = it.getCurrent();
 
                 if (fullIndex.compareRowNonUnique(
-                        session, currentData, otherData,
+                        (Session) session, currentData, otherData,
                         visibleColumnCount) == 0) {
                     continue;
                 }
@@ -422,10 +443,10 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void except(Session session, RowSetNavigatorData other) {
+    public void except(RowSetNavigatorData other) {
 
-        removeDuplicates(session);
-        other.sortFull(session);
+        removeDuplicates();
+        other.sortFull();
 
         while (next()) {
             Object[] currentData = getCurrent();
@@ -439,23 +460,23 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public void exceptAll(Session session, RowSetNavigatorData other) {
+    public void exceptAll(RowSetNavigatorData other) {
 
         Object[]    compareData = null;
         RowIterator it;
         Object[]    otherData = null;
 
-        sortFull(session);
-        other.sortFull(session);
+        sortFull();
+        other.sortFull();
 
-        it = fullIndex.emptyIterator();
+        it = RowIterator.emptyRowIterator;
 
         while (next()) {
             Object[] currentData = getCurrent();
             boolean newGroup =
                 compareData == null
                 || fullIndex.compareRowNonUnique(
-                    session, currentData, compareData,
+                    (Session) session, currentData, compareData,
                     fullIndex.getColumnCount()) != 0;
 
             if (newGroup) {
@@ -467,7 +488,7 @@ implements Comparator<Object[]> {
                 otherData = it.getCurrent();
 
                 if (fullIndex.compareRowNonUnique(
-                        session, currentData, otherData,
+                        (Session) session, currentData, otherData,
                         fullIndex.getColumnCount()) == 0) {
                     removeCurrent();
                 }
@@ -477,9 +498,9 @@ implements Comparator<Object[]> {
         reset();
     }
 
-    public boolean hasUniqueNotNullRows(Session session) {
+    public boolean hasUniqueNotNullRows() {
 
-        sortFull(session);
+        sortFull();
         reset();
 
         Object[] lastRowData = null;
@@ -492,8 +513,8 @@ implements Comparator<Object[]> {
             }
 
             if (lastRowData != null
-                    && fullIndex.compareRow(session, lastRowData, currentData)
-                       == 0) {
+                    && fullIndex.compareRow(
+                        (Session) session, lastRowData, currentData) == 0) {
                 return false;
             } else {
                 lastRowData = currentData;
@@ -503,9 +524,9 @@ implements Comparator<Object[]> {
         return true;
     }
 
-    public void removeDuplicates(Session session) {
+    public void removeDuplicates() {
 
-        sortFull(session);
+        sortFull();
         reset();
 
         int      lastRowPos  = -1;
@@ -521,7 +542,8 @@ implements Comparator<Object[]> {
                 continue;
             }
 
-            if (fullIndex.compareRow(session, lastRowData, currentData) != 0) {
+            if (fullIndex.compareRow(
+                    (Session) session, lastRowData, currentData) != 0) {
                 lastRowPos++;
 
                 lastRowData           = currentData;
@@ -602,7 +624,33 @@ implements Comparator<Object[]> {
             return simpleAggregateData;
         }
 
-        return rowMap.get(data);
+        Integer position = rowMap.get(data);
+
+        if (position == null) {
+            return null;
+        }
+
+        int pos = position.intValue();
+
+        return dataTable[pos];
+    }
+
+    /**
+     * Special case for isSimpleAggregate cannot use index lookup.
+     */
+    public Object[] getGroupDataAndPosition(Object[] data) {
+
+        Integer mapPos = rowMap.get(data);
+
+        if (mapPos == null) {
+            return null;
+        }
+
+        int pos = mapPos.intValue();
+
+        currentPos = pos;
+
+        return dataTable[pos];
     }
 
     boolean containsRow(Object[] data) {
@@ -653,10 +701,6 @@ implements Comparator<Object[]> {
         }
     }
 
-    void implement() {
-        throw Error.error(ErrorCode.U_S0500, "RSND");
-    }
-
     class DataIterator implements RowIterator {
 
         int pos;
@@ -677,7 +721,6 @@ implements Comparator<Object[]> {
         public boolean next() {
 
             if (pos < size - 1) {
-
                 pos++;
 
                 return true;
