@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 package org.hsqldb.persist;
 
 import java.io.IOException;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
@@ -43,13 +44,10 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.FileAccess;
 import org.hsqldb.lib.FileArchiver;
-import org.hsqldb.lib.FileUtil;
 import org.hsqldb.lib.IntIndex;
 import org.hsqldb.map.BitMap;
-import org.hsqldb.rowio.RowInputBinary180;
 import org.hsqldb.rowio.RowInputBinaryDecode;
 import org.hsqldb.rowio.RowInputInterface;
-import org.hsqldb.rowio.RowOutputBinary180;
 import org.hsqldb.rowio.RowOutputBinaryEncode;
 import org.hsqldb.rowio.RowOutputInterface;
 
@@ -60,32 +58,41 @@ import org.hsqldb.rowio.RowOutputInterface;
  * and access.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.2
  */
 public class DataFileCache {
 
+    interface Flags {
+
+        int FLAG_ISSHADOWED = 1;
+        int FLAG_ISSAVED    = 2;
+        int FLAG_ROWINFO    = 3;
+        int FLAG_200        = 4;
+        int FLAG_HX         = 5;
+        int FLAG_251        = 6;
+    }
+
+    /**
+     * file format fields
+     */
+    interface Positions {
+
+        int LONG_EMPTY_SIZE      = 4;                                          // empty space size
+        int LONG_FREE_POS        = 12;                                         // where iFreePos is saved
+        int INT_SPACE_PROPS      = 20;                                         // (space size << 16) + scale
+        int INT_SPACE_LIST_POS   = 24;                                         // space list
+        int INT_FLAGS            = 28;
+        int LONG_TIMESTAMP       = 32;                                         // db open tx timestamp
+        int MIN_INITIAL_FREE_POS = 64;                                         // not used up to this
+        int MAX_INITIAL_FREE_POS = DataSpaceManager.fixedDiskBlockSize * 2;    // not used up to this
+    }
+
     protected FileAccess fa;
 
-    // flags
-    public static final int FLAG_ISSHADOWED = 1;
-    public static final int FLAG_ISSAVED    = 2;
-    public static final int FLAG_ROWINFO    = 3;
-    public static final int FLAG_190        = 4;
-    public static final int FLAG_HX         = 5;
-
-    // file format fields
-    static final int LONG_EMPTY_SIZE      = 4;        // empty space size
-    static final int LONG_FREE_POS_POS    = 12;       // where iFreePos is saved
-    static final int INT_SPACE_PROPS_POS  = 20;       // space size << 16 + scale
-    static final int INT_SPACE_LIST_POS   = 24;       // space list
-    static final int FLAGS_POS            = 28;
-    static final int MIN_INITIAL_FREE_POS = 32;
-
     //
-    public DataSpaceManager  spaceManager;
-    static final int         initIOBufferSize = 4096;
-    private static final int diskBlockSize    = 4096;
+    public DataSpaceManager spaceManager;
+    static final int        initIOBufferSize = 4096;
 
     //
     protected String   dataFileName;
@@ -99,6 +106,7 @@ public class DataFileCache {
     protected boolean fileModified;
     protected boolean cacheModified;
     protected int     dataFileScale;
+    protected int     dataFileSpace;
 
     // post opening constant fields
     protected boolean cacheReadonly;
@@ -107,12 +115,10 @@ public class DataFileCache {
     protected int cachedRowPadding;
 
     //
-    protected long    initialFreePos;
-    protected long    lostSpaceSize;
-    protected long    spaceManagerPosition;
-    protected long    fileStartFreePosition;
-    protected boolean hasRowInfo = false;
-    protected int     storeCount;
+    protected long lostSpaceSize;
+    protected long spaceManagerPosition;
+    protected long fileStartFreePosition;
+    protected int  storeCount;
 
     // reusable input / output streams
     protected RowInputInterface rowIn;
@@ -122,7 +128,7 @@ public class DataFileCache {
     public long maxDataFileSize;
 
     //
-    boolean is180;
+    boolean is251;
 
     //
     protected RandomAccessInterface dataFile;
@@ -164,7 +170,7 @@ public class DataFileCache {
         initNewFile();
         initBuffers();
 
-        if (database.logger.getDataFileSpaces() > 0) {
+        if (dataFileSpace > 0) {
             spaceManager = new DataSpaceManagerBlocks(this);
         } else {
             spaceManager = new DataSpaceManagerSimple(this, false);
@@ -177,21 +183,16 @@ public class DataFileCache {
     protected void initParams(Database database, String baseFileName,
                               boolean defrag) {
 
-        this.dataFileName   = baseFileName + Logger.dataFileExtension;
-        this.backupFileName = baseFileName + Logger.backupFileExtension;
-        this.database       = database;
-        fa                  = database.logger.getFileAccess();
-        dataFileScale       = database.logger.getDataFileScale();
-        cachedRowPadding    = 8;
+        this.database    = database;
+        dataFileName     = baseFileName + Logger.dataFileExtension;
+        backupFileName   = baseFileName + Logger.backupFileExtension;
+        fa               = database.logger.getFileAccess();
+        dataFileScale    = database.logger.getDataFileScale();
+        dataFileSpace    = database.logger.getDataFileSpace();
+        cachedRowPadding = dataFileScale;
 
-        if (dataFileScale > 8) {
-            cachedRowPadding = dataFileScale;
-        }
-
-        initialFreePos = MIN_INITIAL_FREE_POS;
-
-        if (initialFreePos < dataFileScale) {
-            initialFreePos = dataFileScale;
+        if (dataFileScale < 8) {
+            cachedRowPadding = 8;
         }
 
         cacheReadonly = database.isFilesReadOnly();
@@ -201,10 +202,10 @@ public class DataFileCache {
                           * database.logger.getDataFileFactor();
 
         if (defrag) {
-            this.dataFileName   = dataFileName + Logger.newFileExtension;
-            this.backupFileName = backupFileName + Logger.newFileExtension;
-            this.maxCacheRows   = 1024;
-            this.maxCacheBytes  = 1024 * 4096;
+            dataFileName   = dataFileName + Logger.newFileExtension;
+            backupFileName = backupFileName + Logger.newFileExtension;
+            maxCacheRows   = 1024;
+            maxCacheBytes  = 1024 * 4096;
         }
     }
 
@@ -214,8 +215,6 @@ public class DataFileCache {
      */
     public void open(boolean readonly) {
 
-        fileFreePosition = initialFreePos;
-
         logInfoEvent("dataFileCache open start");
 
         try {
@@ -236,20 +235,18 @@ public class DataFileCache {
 
                 int flags = getFlags();
 
-                is180 = !BitMap.isSet(flags, FLAG_190);
-
-                if (BitMap.isSet(flags, FLAG_HX)) {
+                if (BitMap.isSet(flags, Flags.FLAG_HX)) {
                     throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION);
                 }
 
-                dataFile.seek(LONG_FREE_POS_POS);
+                dataFile.seek(Positions.LONG_FREE_POS);
 
                 fileFreePosition = dataFile.readLong();
 
-                dataFile.seek(INT_SPACE_LIST_POS);
+                dataFile.seek(Positions.INT_SPACE_LIST_POS);
 
                 spaceManagerPosition = (long) dataFile.readInt()
-                                       * DataSpaceManager.fixedBlockSizeUnit;
+                                       * DataSpaceManager.fixedDiskBlockSize;
 
                 initBuffers();
 
@@ -259,8 +256,9 @@ public class DataFileCache {
             }
 
             boolean preexists     = fa.isStreamElement(dataFileName);
-            boolean isIncremental = database.logger.propIncrementBackup;
+            boolean isIncremental = true;
             boolean isSaved       = false;
+            boolean doRestore     = false;
 
             if (preexists) {
                 dataFile = new RAFileSimple(database.logger, dataFileName,
@@ -269,16 +267,28 @@ public class DataFileCache {
                 long    length       = dataFile.length();
                 boolean wrongVersion = false;
 
-                if (length >= initialFreePos) {
+                if (length > Positions.LONG_TIMESTAMP) {
                     int flags = getFlags();
 
-                    isSaved       = BitMap.isSet(flags, FLAG_ISSAVED);
-                    isIncremental = BitMap.isSet(flags, FLAG_ISSHADOWED);
-                    is180         = !BitMap.isSet(flags, FLAG_190);
+                    isSaved       = BitMap.isSet(flags, Flags.FLAG_ISSAVED);
+                    isIncremental = BitMap.isSet(flags, Flags.FLAG_ISSHADOWED);
+                    is251         = BitMap.isSet(flags, Flags.FLAG_251);
 
-                    if (BitMap.isSet(flags, FLAG_HX)) {
+                    if (BitMap.isSet(flags, Flags.FLAG_HX)) {
                         wrongVersion = true;
                     }
+                } else {
+                    preexists = false;
+                }
+
+                if (isSaved && is251) {
+                    dataFile.seek(Positions.LONG_TIMESTAMP);
+
+                    long timestamp = dataFile.readLong();
+
+                    if (timestamp > database.logger.getFilesTimestamp()) {
+                        doRestore = true;
+                    }
                 }
 
                 dataFile.close();
@@ -300,43 +310,38 @@ public class DataFileCache {
                     throw Error.error(ErrorCode.DATA_FILE_IS_FULL,
                                       String.valueOf(maxDataFileSize));
                 }
+            }
 
-                if (isSaved && isIncremental) {
+            if (preexists) {
+                if (isSaved) {
                     boolean existsBackup = fa.isStreamElement(backupFileName);
 
                     if (existsBackup) {
                         logInfoEvent(
                             "data file was not modified but inc backup exists");
-                    }
-                }
-            }
 
-            if (isSaved) {
-                if (isIncremental) {
-                    deleteBackup();
-                } else {
-                    boolean existsBackup = fa.isStreamElement(backupFileName);
-
-                    if (!existsBackup) {
-                        backupDataFile(false);
-                    }
-                }
-            } else {
-                if (isIncremental) {
-                    if (preexists) {
-                        preexists = restoreBackupIncremental();
-
-                        if (!preexists) {
-                            database.logger.logSevereEvent(
-                                "DataFileCache data file modified but no backup exists",
-                                null);
-
-                            throw Error.error(
-                                ErrorCode.DATA_FILE_BACKUP_MISMATCH);
+                        if (doRestore) {
+                            restoreBackupIncremental();
                         }
                     }
+
+                    deleteBackupFile();
                 } else {
-                    preexists = restoreBackup();
+                    boolean restored;
+
+                    if (isIncremental) {
+                        restored = restoreBackupIncremental();
+                    } else {
+                        restored = restoreBackup();
+                    }
+
+                    if (!restored) {
+                        database.logger.logSevereEvent(
+                            "DataFileCache data file modified but no backup exists",
+                            null);
+
+                        throw Error.error(ErrorCode.DATA_FILE_BACKUP_MISMATCH);
+                    }
                 }
             }
 
@@ -344,25 +349,31 @@ public class DataFileCache {
                                               readonly, fileType);
 
             if (preexists) {
-                int flags = getFlags();
-
-                is180 = !BitMap.isSet(flags, FLAG_190);
-
-                dataFile.seek(LONG_EMPTY_SIZE);
+                dataFile.seek(Positions.LONG_EMPTY_SIZE);
 
                 lostSpaceSize = dataFile.readLong();
 
-                dataFile.seek(LONG_FREE_POS_POS);
+                dataFile.seek(Positions.LONG_FREE_POS);
 
                 fileFreePosition      = dataFile.readLong();
                 fileStartFreePosition = fileFreePosition;
 
-                dataFile.seek(INT_SPACE_LIST_POS);
+                dataFile.seek(Positions.INT_SPACE_PROPS);
+
+                int spaceProps = dataFile.readInt();
+
+                setSpaceProps(spaceProps);
+                dataFile.seek(Positions.INT_SPACE_LIST_POS);
 
                 spaceManagerPosition = (long) dataFile.readInt()
-                                       * DataSpaceManager.fixedBlockSizeUnit;
+                                       * DataSpaceManager.fixedDiskBlockSize;
 
-                openShadowFile();
+                int flags = getFlags();
+
+                flags = BitMap.set(flags, Flags.FLAG_ISSHADOWED);
+                flags = BitMap.set(flags, Flags.FLAG_ISSAVED);
+
+                setFlags(flags);
             } else {
                 initNewFile();
             }
@@ -372,12 +383,17 @@ public class DataFileCache {
             fileModified  = false;
             cacheModified = false;
 
-            if (database.logger.getDataFileSpaces() > 0) {
+            if (dataFileSpace > 0) {
                 spaceManager = new DataSpaceManagerBlocks(this);
             } else {
                 spaceManager = new DataSpaceManagerSimple(this, false);
             }
 
+            if (!preexists) {
+                reset();
+            }
+
+            openShadowFile();
             logInfoEvent("dataFileCache open end");
         } catch (HsqlException e) {
             throw e;
@@ -392,147 +408,66 @@ public class DataFileCache {
         }
     }
 
-    boolean setDataSpaceManager() {
+    void setSpaceProps(int spaceProps) {
 
-        writeLock.lock();
+        if (spaceProps == 0) {
+            spaceProps = dataFileScale | (dataFileSpace << 16);
 
-        int fileSpaceSize = database.logger.propFileSpaceValue;
-
-        try {
-            if (fileSpaceSize > 0 && spaceManagerPosition == 0) {
-                spaceManager.reset();
-
-                spaceManager = new DataSpaceManagerBlocks(this);
-
-                return true;
+            try {
+                dataFile.seek(Positions.INT_SPACE_PROPS);
+                dataFile.writeInt(spaceProps);
+                dataFile.synch();
+            } catch (Throwable t) {
+                throw Error.error(ErrorCode.FILE_IO_ERROR, t);
             }
 
-            if (fileSpaceSize == 0 && spaceManagerPosition != 0) {
-                spaceManager.reset();
-
-                spaceManager = new DataSpaceManagerSimple(this, false);
-
-                return true;
-            }
-
-            return false;
-        } finally {
-            writeLock.unlock();
+            return;
         }
-    }
 
-    void openStoredFileAccess(boolean readonly) {
+        dataFileScale = spaceProps & 0xffff;
+        dataFileSpace = spaceProps >>> 16;
 
-        fileFreePosition = initialFreePos;
-
-        logInfoEvent("dataFileCache open start");
-
-        try {
-            int fileType = RAFile.DATA_FILE_STORED;
-
-            if (readonly) {
-                dataFile = RAFile.newScaledRAFile(database, dataFileName,
-                                                  readonly, fileType);
-
-                dataFile.seek(FLAGS_POS);
-
-                int flags = dataFile.readInt();
-
-                is180 = !BitMap.isSet(flags, FLAG_190);
-
-                dataFile.seek(LONG_FREE_POS_POS);
-
-                fileFreePosition = dataFile.readLong();
-
-                initBuffers();
-
-                return;
-            }
-
-            boolean preexists     = fa.isStreamElement(dataFileName);
-            boolean isIncremental = database.logger.propIncrementBackup;
-            boolean restore = database.getProperties().getDBModified()
-                              == HsqlDatabaseProperties.FILES_MODIFIED;
-
-            if (preexists && restore) {
-                if (isIncremental) {
-                    preexists = restoreBackupIncremental();
-                } else {
-                    preexists = restoreBackup();
-                }
-            }
-
-            dataFile = RAFile.newScaledRAFile(database, dataFileName,
-                                              readonly, fileType);
-
-            if (preexists) {
-                dataFile.seek(LONG_EMPTY_SIZE);
-
-                lostSpaceSize = dataFile.readLong();
-
-                dataFile.seek(LONG_FREE_POS_POS);
-
-                fileFreePosition      = dataFile.readLong();
-                fileStartFreePosition = fileFreePosition;
-
-                dataFile.seek(FLAGS_POS);
-
-                int flags = dataFile.readInt();
-
-                is180 = !BitMap.isSet(flags, FLAG_190);
-
-                openShadowFile();
-            } else {
-                initNewFile();
-            }
-
-            initBuffers();
-
-            fileModified  = false;
-            cacheModified = false;
-            spaceManager  = new DataSpaceManagerSimple(this, false);
-
-            logInfoEvent("dataFileCache open end");
-        } catch (Throwable t) {
-            logSevereEvent("dataFileCache open failed", t);
-            release();
-
-            throw Error.error(t, ErrorCode.FILE_IO_ERROR,
-                              ErrorCode.M_DataFileCache_open, new Object[] {
-                t.toString(), dataFileName
-            });
-        }
+        database.logger.setDataFileScaleNoCheck(dataFileScale);
+        database.logger.setDataFileSpace(dataFileSpace);
     }
 
     void initNewFile() {
 
         try {
+            int initialFreePos = Positions.MAX_INITIAL_FREE_POS;
+
+            if (dataFileSpace == 0) {
+                initialFreePos = Positions.MIN_INITIAL_FREE_POS;
+
+                if (initialFreePos < dataFileScale) {
+                    initialFreePos = dataFileScale;
+                }
+            }
+
             fileFreePosition      = initialFreePos;
             fileStartFreePosition = initialFreePos;
 
-            dataFile.seek(LONG_FREE_POS_POS);
+            dataFile.seek(Positions.LONG_FREE_POS);
             dataFile.writeLong(fileFreePosition);
 
-            int spaceProps = dataFileScale;
+            int spaceProps = dataFileScale | (dataFileSpace << 16);
 
-            spaceProps |= (database.logger.getDataFileSpaces() << 16);
-
-            dataFile.seek(INT_SPACE_PROPS_POS);
+            dataFile.seek(Positions.INT_SPACE_PROPS);
             dataFile.writeInt(spaceProps);
+            dataFile.seek(Positions.LONG_TIMESTAMP);
+            dataFile.writeLong(database.logger.getFilesTimestamp());
 
             // set shadowed flag;
             int flags = 0;
 
-            if (database.logger.propIncrementBackup) {
-                flags = BitMap.set(flags, FLAG_ISSHADOWED);
-            }
-
-            flags = BitMap.set(flags, FLAG_ISSAVED);
-            flags = BitMap.set(flags, FLAG_190);
+            flags = BitMap.set(flags, Flags.FLAG_ISSHADOWED);
+            flags = BitMap.set(flags, Flags.FLAG_ISSAVED);
+            flags = BitMap.set(flags, Flags.FLAG_200);
+            flags = BitMap.set(flags, Flags.FLAG_251);
 
             setFlags(flags);
 
-            is180 = false;
+            is251 = true;
         } catch (Throwable t) {
             throw Error.error(ErrorCode.FILE_IO_ERROR, t);
         }
@@ -540,27 +475,9 @@ public class DataFileCache {
 
     private void openShadowFile() {
 
-        if (database.logger.propIncrementBackup
-                && fileFreePosition != initialFreePos) {
-            shadowFile = new RAShadowFile(database.logger, dataFile,
-                                          backupFileName, fileFreePosition,
-                                          1 << 14);
-        }
-    }
-
-    void setIncrementBackup(boolean value) {
-
-        writeLock.lock();
-
-        try {
-            setFlag(FLAG_ISSHADOWED, value);
-
-            fileModified = true;
-        } catch (Throwable t) {
-            logSevereEvent("DataFileCache.setIncrementalBackup", t);
-        } finally {
-            writeLock.unlock();
-        }
+        shadowFile = new RAShadowFile(database.logger, dataFile,
+                                      backupFileName, fileFreePosition,
+                                      1 << 14);
     }
 
     /**
@@ -571,8 +488,7 @@ public class DataFileCache {
         try {
             FileAccess fileAccess = database.logger.getFileAccess();
 
-            // todo - in case data file cannot be deleted, reset it
-            deleteFile(database, dataFileName);
+            deleteBackupFile();
 
             if (fileAccess.isStreamElement(backupFileName)) {
                 FileArchiver.unarchive(backupFileName, dataFileName,
@@ -604,7 +520,7 @@ public class DataFileCache {
             if (fileAccess.isStreamElement(backupFileName)) {
                 RAShadowFile.restoreFile(database, backupFileName,
                                          dataFileName);
-                deleteFile(database, backupFileName);
+                deleteBackupFile();
 
                 return true;
             }
@@ -667,13 +583,6 @@ public class DataFileCache {
             logDetailEvent("dataFileCache file close end");
 
             dataFile = null;
-
-            boolean empty = fileFreePosition == initialFreePos;
-
-            if (empty) {
-                deleteFile();
-                deleteBackup();
-            }
         } catch (HsqlException e) {
             throw e;
         } catch (Throwable t) {
@@ -741,30 +650,33 @@ public class DataFileCache {
             // set empty
             long lostSize = spaceManager.getLostBlocksSize();
 
-            dataFile.seek(LONG_EMPTY_SIZE);
+            dataFile.seek(Positions.LONG_EMPTY_SIZE);
             dataFile.writeLong(lostSize);
 
             // set end
-            dataFile.seek(LONG_FREE_POS_POS);
+            dataFile.seek(Positions.LONG_FREE_POS);
             dataFile.writeLong(fileFreePosition);
 
             // set space props
-            int spaceProps = dataFileScale;
+            int spaceProps = dataFileScale | (dataFileSpace << 16);
 
-            spaceProps |= (database.logger.getDataFileSpaces() << 16);
-
-            dataFile.seek(INT_SPACE_PROPS_POS);
+            dataFile.seek(Positions.INT_SPACE_PROPS);
             dataFile.writeInt(spaceProps);
 
             // set space list
             int pos = (int) (spaceManagerPosition
-                             / DataSpaceManager.fixedBlockSizeUnit);
+                             / DataSpaceManager.fixedDiskBlockSize);
 
-            dataFile.seek(INT_SPACE_LIST_POS);
+            dataFile.seek(Positions.INT_SPACE_LIST_POS);
             dataFile.writeInt(pos);
 
-            // set saved flag;
-            setFlag(DataFileCache.FLAG_ISSAVED, true);
+            if (is251) {
+                dataFile.seek(Positions.LONG_TIMESTAMP);
+                dataFile.writeLong(database.logger.getFilesTimestamp());
+            }
+
+            // set saved flag and sync file
+            setFlag(Flags.FLAG_ISSAVED, true);
             logDetailEvent("file sync end");
 
             fileModified          = false;
@@ -793,23 +705,14 @@ public class DataFileCache {
     protected void initBuffers() {
 
         if (rowOut == null) {
-            if (is180) {
-                rowOut = new RowOutputBinary180(initIOBufferSize,
-                                                cachedRowPadding);
-            } else {
-                rowOut = new RowOutputBinaryEncode(database.logger.getCrypto(),
-                                                   initIOBufferSize,
-                                                   cachedRowPadding);
-            }
+            rowOut = new RowOutputBinaryEncode(database.logger.getCrypto(),
+                                               initIOBufferSize,
+                                               cachedRowPadding);
         }
 
         if (rowIn == null) {
-            if (is180) {
-                rowIn = new RowInputBinary180(new byte[initIOBufferSize]);
-            } else {
-                rowIn = new RowInputBinaryDecode(database.logger.getCrypto(),
-                                                 new byte[initIOBufferSize]);
-            }
+            rowIn = new RowInputBinaryDecode(database.logger.getCrypto(),
+                                             new byte[initIOBufferSize]);
         }
     }
 
@@ -823,38 +726,6 @@ public class DataFileCache {
             DataFileDefrag dfd = new DataFileDefrag(database, this);
 
             dfd.process(session);
-            close();
-            cache.clear();
-
-            if (!database.logger.propIncrementBackup) {
-                backupNewDataFile(true);
-            }
-
-            database.schemaManager.setTempIndexRoots(dfd.getIndexRoots());
-
-            try {
-                database.logger.log.writeScript(false);
-            } finally {
-                database.schemaManager.setTempIndexRoots(null);
-            }
-
-            database.getProperties().setProperty(
-                HsqlDatabaseProperties.hsqldb_script_format,
-                database.logger.propScriptFormat);
-            database.getProperties().setDBModified(
-                HsqlDatabaseProperties.FILES_MODIFIED_NEW);
-            database.logger.log.closeLog();
-            database.logger.log.deleteLog();
-            database.logger.log.renameNewScript();
-            renameBackupFile();
-            renameDataFile();
-            database.getProperties().setDBModified(
-                HsqlDatabaseProperties.FILES_NOT_MODIFIED);
-            open(false);
-
-            if (database.logger.log.dbLogWriter != null) {
-                database.logger.log.openLog();
-            }
 
             return dfd;
         } finally {
@@ -865,7 +736,6 @@ public class DataFileCache {
     /**
      * Used when a row is deleted as a result of some DML or DDL statement.
      * Removes the row from the cache data structures.
-     * Adds the file space for the row to the list of free positions.
      */
     public void remove(CachedObject object) {
         release(object.getPos());
@@ -1059,8 +929,6 @@ public class DataFileCache {
                 object.keepInMemory(true);
             }
 
-            store.set(object);
-
             return object;
         } catch (HsqlException e) {
             logSevereEvent(dataFileName + " getFromFile failed " + pos, e);
@@ -1120,8 +988,6 @@ public class DataFileCache {
                 object.keepInMemory(true);
             }
 
-            store.set(object);
-
             return object;
         } catch (HsqlException e) {
             logSevereEvent(dataFileName + " getFromFile failed " + pos, e);
@@ -1362,158 +1228,41 @@ public class DataFileCache {
         return 0;
     }
 
-    /**
-     *  Saves the *.data file as compressed *.backup.
-     *
-     * @throws  HsqlException
-     */
-    void backupDataFile(boolean newFile) {
-        backupFile(database, dataFileName, backupFileName, newFile);
+    void deleteDataFile() {
+        Log.deleteFile(fa, dataFileName);
     }
 
-    void backupNewDataFile(boolean newFile) {
-        backupFile(database, dataFileName + Logger.newFileExtension,
-                   backupFileName, newFile);
-    }
-
-    static void backupFile(Database database, String fileName,
-                           String backupFileName, boolean newFile) {
-
-        try {
-            FileAccess fa = database.logger.getFileAccess();
-
-            if (database.logger.propIncrementBackup) {
-                if (fa.isStreamElement(backupFileName)) {
-                    deleteFile(database, backupFileName);
-
-                    if (fa.isStreamElement(backupFileName)) {
-                        throw Error.error(ErrorCode.DATA_FILE_ERROR,
-                                          "cannot delete old backup file");
-                    }
-                }
-
-                return;
-            }
-
-            if (fa.isStreamElement(fileName)) {
-                if (newFile) {
-                    backupFileName += Logger.newFileExtension;
-                } else {
-                    deleteFile(database, backupFileName);
-
-                    if (fa.isStreamElement(backupFileName)) {
-                        throw Error.error(ErrorCode.DATA_FILE_ERROR,
-                                          "cannot delete old backup file");
-                    }
-                }
-
-                FileArchiver.archive(fileName, backupFileName, fa,
-                                     FileArchiver.COMPRESSION_ZIP);
-            }
-        } catch (Throwable t) {
-            database.logger.logSevereEvent("DataFileCache.backupFile", t);
-
-            throw Error.error(ErrorCode.DATA_FILE_ERROR, t);
-        }
-    }
-
-    void renameBackupFile() {
-        renameBackupFile(database, backupFileName);
-    }
-
-    static void renameBackupFile(Database database, String backupFileName) {
-
-        FileAccess fileAccess = database.logger.getFileAccess();
-
-        if (database.logger.propIncrementBackup) {
-            deleteFile(database, backupFileName);
-
-            return;
-        }
-
-        if (fileAccess.isStreamElement(backupFileName
-                                       + Logger.newFileExtension)) {
-            deleteFile(database, backupFileName);
-            fileAccess.renameElement(backupFileName + Logger.newFileExtension,
-                                     backupFileName, true);
-        }
-    }
-
-    /**
-     *  Renames the *.data.new file.
-     *
-     * @throws  HsqlException
-     */
-    void renameDataFile() {
-        renameDataFile(database, dataFileName);
-    }
-
-    static void renameDataFile(Database database, String dataFileName) {
-
-        FileAccess fileAccess = database.logger.getFileAccess();
-
-        if (fileAccess.isStreamElement(dataFileName
-                                       + Logger.newFileExtension)) {
-            deleteFile(database, dataFileName);
-            fileAccess.renameElement(dataFileName + Logger.newFileExtension,
-                                     dataFileName, true);
-        }
-    }
-
-    void deleteFile() {
-        deleteFile(database, dataFileName);
-    }
-
-    static void deleteFile(Database database, String fileName) {
-
-        FileAccess fileAccess = database.logger.getFileAccess();
-
-        // first attempt to delete
-        fileAccess.removeElement(fileName);
-
-        if (fileAccess.isStreamElement(fileName)) {
-            fileAccess.removeElement(fileName);
-
-            if (fileAccess.isStreamElement(fileName)) {
-                String discardName = FileUtil.newDiscardFileName(fileName);
-
-                fileAccess.renameElement(fileName, discardName, false);
-            }
-        }
-    }
-
-    void deleteBackup() {
-        deleteFile(database, backupFileName);
+    private void deleteBackupFile() {
+        Log.deleteFile(fa, backupFileName);
     }
 
     /**
      * Delta must always result in block multiples
      */
-    public long enlargeFileSpace(long delta) {
+    public long enlargeFileSpace(long newLength) {
 
         writeLock.lock();
 
         try {
             long position = fileFreePosition;
 
-            if (position + delta > maxDataFileSize) {
+            if (newLength > maxDataFileSize) {
                 logSevereEvent("data file reached maximum allowed size: "
                                + dataFileName + " " + maxDataFileSize, null);
 
                 throw Error.error(ErrorCode.DATA_FILE_IS_FULL);
             }
 
-            boolean result = dataFile.ensureLength(position + delta);
+            boolean result = dataFile.ensureLength(newLength);
 
             if (!result) {
                 logSevereEvent("data file cannot be enlarged - disk space: "
-                               + dataFileName + " "
-                               + (position + delta), null);
+                               + dataFileName + " " + newLength, null);
 
                 throw Error.error(ErrorCode.DATA_FILE_IS_FULL);
             }
 
-            fileFreePosition += delta;
+            fileFreePosition = newLength;
 
             return position;
         } finally {
@@ -1553,8 +1302,8 @@ public class DataFileCache {
         return dataFileScale;
     }
 
-    public boolean hasRowInfo() {
-        return hasRowInfo;
+    public int getDataFileSpace() {
+        return dataFileSpace;
     }
 
     public boolean isFileModified() {
@@ -1575,7 +1324,7 @@ public class DataFileCache {
             if (!fileModified) {
 
                 // unset saved flag;
-                setFlag(FLAG_ISSAVED, false);
+                setFlag(Flags.FLAG_ISSAVED, false);
                 logDetailEvent("setFileModified flag set ");
 
                 fileModified = true;
@@ -1589,7 +1338,7 @@ public class DataFileCache {
 
     int getFlags() throws IOException {
 
-        dataFile.seek(FLAGS_POS);
+        dataFile.seek(Positions.INT_FLAGS);
 
         int flags = dataFile.readInt();
 
@@ -1598,21 +1347,21 @@ public class DataFileCache {
 
     void setFlags(int flags) throws IOException {
 
-        dataFile.seek(FLAGS_POS);
+        dataFile.seek(Positions.INT_FLAGS);
         dataFile.writeInt(flags);
         dataFile.synch();
     }
 
     void setFlag(int singleFlag, boolean val) throws IOException {
 
-        dataFile.seek(FLAGS_POS);
+        dataFile.seek(Positions.INT_FLAGS);
 
         int flags = dataFile.readInt();
 
         flags = val ? BitMap.set(flags, singleFlag)
                     : BitMap.unset(flags, singleFlag);
 
-        dataFile.seek(FLAGS_POS);
+        dataFile.seek(Positions.INT_FLAGS);
         dataFile.writeInt(flags);
         dataFile.synch();
     }
@@ -1625,6 +1374,10 @@ public class DataFileCache {
         return shadowFile;
     }
 
+    public AtomicInteger getAccessCount() {
+        return cache.getAccessCount();
+    }
+
     private void logSevereEvent(String message, Throwable t, long position) {
 
         if (logEvents) {
