@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,11 +31,11 @@
 
 package org.hsqldb.persist;
 
-import org.hsqldb.lib.DoubleIntIndex;
+import org.hsqldb.lib.LongLookup;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.5.1
  * @since 2.3.0
  */
 public interface DataSpaceManager {
@@ -48,7 +48,7 @@ public interface DataSpaceManager {
     int tableIdFirst     = 8;
 
     //
-    int fixedBlockSizeUnit = 4096;
+    int fixedDiskBlockSize = 4096;
 
     TableSpaceManager getDefaultTableSpace();
 
@@ -58,10 +58,12 @@ public interface DataSpaceManager {
 
     long getFileBlocks(int spaceId, int blockCount);
 
+    void initialiseTableSpace(TableSpaceManagerBlocks tableSpace);
+
     void freeTableSpace(int spaceId);
 
-    void freeTableSpace(int spaceId, DoubleIntIndex spaceList, long offset,
-                        long limit, boolean full);
+    void freeTableSpace(int spaceId, LongLookup spaceList, long offset,
+                        long limit);
 
     long getLostBlocksSize();
 
