@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,16 +35,16 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
-import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.DoubleIntIndex;
 import org.hsqldb.lib.IntIndex;
 import org.hsqldb.lib.IntKeyHashMap;
 import org.hsqldb.lib.Iterator;
+import org.hsqldb.lib.LongLookup;
 import org.hsqldb.lib.OrderedIntHashSet;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.4.1
+ * @version 2.5.1
  * @since 2.3.0
  */
 public class DataSpaceManagerBlocks implements DataSpaceManager {
@@ -61,9 +61,11 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
     final BlockObjectStore rootStore;
     final BlockObjectStore directoryStore;
     final BlockObjectStore bitMapStore;
+    final BlockObjectStore lastBlockStore;
 
     //
-    IntArrayCachedObject rootBlock;
+    IntArrayCachedObject       rootBlock;
+    DoubleIntArrayCachedObject lastBlocks;
 
     //
     final AtomicInteger spaceIdSequence = new AtomicInteger(tableIdFirst);
@@ -71,6 +73,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
     int                 released = 0;
 
     //
+    public static final int lastBlockListSize       = 1024;
     public static final int dirBlockSize            = 1024 * 2;
     public static final int fileBlockItemCountLimit = 64 * 1024;
 
@@ -88,17 +91,16 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
         int bitmapStoreSizeTemp;
 
-        cache         = dataFileCache;
-        dataFileScale = cache.getDataFileScale();
-        fileBlockSize = cache.database.logger.getDataFileSpaces() * 1024
-                        * 1024;
+        cache              = dataFileCache;
+        dataFileScale      = cache.getDataFileScale();
+        fileBlockSize      = cache.getDataFileSpace() * 1024 * 1024;
         fileBlockItemCount = fileBlockSize / dataFileScale;
         bitmapIntSize      = fileBlockItemCount / Integer.SIZE;
         bitmapStoreSizeTemp = BitMapCachedObject.fileSizeFactor
                               * bitmapIntSize;
 
-        if (bitmapStoreSizeTemp < DataSpaceManager.fixedBlockSizeUnit) {
-            bitmapStoreSizeTemp = DataSpaceManager.fixedBlockSizeUnit;
+        if (bitmapStoreSizeTemp < fixedDiskBlockSize) {
+            bitmapStoreSizeTemp = fixedDiskBlockSize;
         }
 
         bitmapStorageSize = bitmapStoreSizeTemp;
@@ -108,28 +110,28 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
         //
         directorySpaceManager = new TableSpaceManagerBlocks(this,
-                tableIdDirectory, fileBlockSize, 16, dataFileScale, 0);
+                tableIdDirectory, fileBlockSize, 16, dataFileScale);
         defaultSpaceManager = new TableSpaceManagerBlocks(this,
                 tableIdDefault, fileBlockSize,
-                cache.database.logger.propMaxFreeBlocks, dataFileScale,
-                cache.database.logger.propMinReuse);
+                cache.database.logger.propMaxFreeBlocks, dataFileScale);
 
         spaceManagerList.put(tableIdDirectory, directorySpaceManager);
         spaceManagerList.put(tableIdDefault, defaultSpaceManager);
 
         //
         rootStore      = getRootStore();
-        directoryStore = getDirectoryStore(false);
+        directoryStore = getDirectoryStore();
         bitMapStore    = getBitMapStore();
+        lastBlockStore = getLastBlockStore();
 
         if (cache.spaceManagerPosition == 0) {
             initialiseNewSpaceDirectory();
 
             cache.spaceManagerPosition = rootBlock.getPos() * dataFileScale;
         } else {
-            long pos = cache.spaceManagerPosition / dataFileScale;
+            long position = cache.spaceManagerPosition / dataFileScale;
 
-            rootBlock = (IntArrayCachedObject) rootStore.get(pos, true);
+            rootBlock = (IntArrayCachedObject) rootStore.get(position, true);
 
             // integrity check
             if (getBlockIndexLimit() == 0) {
@@ -141,8 +143,29 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
             }
 
             initialiseSpaceList();
-            initialiseTableSpace(directorySpaceManager);
-            initialiseTableSpace(defaultSpaceManager);
+
+            int blockPos = rootBlock.getValue(dirBlockSize - 1);
+
+            if (blockPos == 0) {
+
+                // create
+                lastBlocks = new DoubleIntArrayCachedObject(lastBlockListSize);
+
+                initialiseTableSpace(directorySpaceManager);
+
+                lastBlockStore.add(lastBlocks, true);
+
+                blockPos = getFileBlockPosFromPosition(lastBlocks.getPos());
+
+                rootBlock.setValue(dirBlockSize - 1, blockPos);
+            } else {
+                position = getPositionFromFileBlock(blockPos);
+                lastBlocks =
+                    (DoubleIntArrayCachedObject) lastBlockStore.get(position,
+                        true);
+
+                initialiseTableSpace(directorySpaceManager);
+            }
         }
     }
 
@@ -154,13 +177,12 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                                     * dirBlockSize, dirBlockSize);
     }
 
-    BlockObjectStore getDirectoryStore(boolean force240) {
+    BlockObjectStore getDirectoryStore() {
 
-        return new BlockObjectStore(
-            cache, directorySpaceManager,
-            DirectoryBlockCachedObject.class,
-            DirectoryBlockCachedObject.fileSizeFactor * dirBlockSize,
-            dirBlockSize);
+        return new BlockObjectStore(cache, directorySpaceManager,
+                                    DirectoryBlockCachedObject.class,
+                                    DirectoryBlockCachedObject.fileSizeFactor
+                                    * dirBlockSize, dirBlockSize);
     }
 
     BlockObjectStore getBitMapStore() {
@@ -169,61 +191,36 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                                     BitMapCachedObject.class,
                                     bitmapStorageSize, bitmapIntSize);
     }
+
+    BlockObjectStore getLastBlockStore() {
+
+        return new BlockObjectStore(cache, directorySpaceManager,
+                                    DoubleIntArrayCachedObject.class,
+                                    DoubleIntArrayCachedObject.fileSizeFactor
+                                    * lastBlockListSize, lastBlockListSize);
+    }
+
     private void initialiseNewSpaceDirectory() {
 
-        long currentSize = cache.getFileFreePos();
-        long totalBlocks = (currentSize / fileBlockSize) + 1;
-        long lastFreePosition = cache.enlargeFileSpace(totalBlocks
-            * fileBlockSize - currentSize);
+        long filePosition       = DataFileCache.Positions.MAX_INITIAL_FREE_POS;
+        int  dirSpaceBlockCount = 1;
 
-        defaultSpaceManager.initialiseFileBlock(null, lastFreePosition,
-                cache.getFileFreePos());
+        cache.enlargeFileSpace(fileBlockSize);
+        directorySpaceManager.initialiseFileBlock(null, filePosition,
+                fileBlockSize);
 
-        long defaultSpaceBlockCount = totalBlocks;
-        long directorySpaceBlockCount =
-            calculateDirectorySpaceBlocks(totalBlocks);
+        rootBlock = new IntArrayCachedObject(dirBlockSize);
 
-        lastFreePosition = cache.enlargeFileSpace(directorySpaceBlockCount
-                * fileBlockSize);
+        rootStore.add(rootBlock, true);
 
-        // file block is empty
-        directorySpaceManager.initialiseFileBlock(null, lastFreePosition,
-                cache.getFileFreePos());
+        lastBlocks = new DoubleIntArrayCachedObject(lastBlockListSize);
 
-        IntArrayCachedObject root = new IntArrayCachedObject(dirBlockSize);
+        lastBlockStore.add(lastBlocks, true);
 
-        rootStore.add(root, true);
+        int blockPos = getFileBlockPosFromPosition(lastBlocks.getPos());
 
-        rootBlock = root;
-
-        createFileBlocksInDirectory((int) defaultSpaceBlockCount,
-                                    (int) directorySpaceBlockCount,
-                                    tableIdDirectory);
-        createFileBlocksInDirectory(0, (int) defaultSpaceBlockCount,
-                                    tableIdDefault);
-    }
-
-    private long calculateDirectorySpaceBlocks(long blockCount) {
-
-        long currentSize   = calculateDirectorySpaceSize(blockCount);
-        long currentBlocks = currentSize / fileBlockSize + 1;
-
-        currentSize   += calculateDirectorySpaceSize(currentBlocks);
-        currentBlocks = currentSize / fileBlockSize + 1;
-
-        return currentBlocks;
-    }
-
-    private long calculateDirectorySpaceSize(long blockCount) {
-
-        long blockLimit = ArrayUtil.getBinaryMultipleCeiling(blockCount + 1,
-            dirBlockSize);
-        long currentSize = IntArrayCachedObject.fileSizeFactor * blockLimit;    // root
-
-        currentSize += DirectoryBlockCachedObject.fileSizeFactor * blockLimit;    // directory
-        currentSize += bitmapStorageSize * (blockCount + 1);                      // bitmaps
-
-        return currentSize;
+        rootBlock.setValue(dirBlockSize - 1, blockPos);
+        createFileBlocksInDirectory(0, dirSpaceBlockCount, tableIdDirectory);
     }
 
     /**
@@ -233,7 +230,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
      */
     private void ensureDirectorySpaceAvailable(int blockCount) {
 
-        int dirObjectSize = bitmapStorageSize * blockCount;
+        long dirObjectSize = bitmapStorageSize * blockCount;
 
         dirObjectSize += DirectoryBlockCachedObject.fileSizeFactor
                          * dirBlockSize;
@@ -241,28 +238,16 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         boolean hasRoom = directorySpaceManager.hasFileRoom(dirObjectSize);
 
         if (!hasRoom) {
-            long cacheFreePos;
-            int  index         = getBlockIndexLimit();
-            int  dirBlockCount = dirObjectSize / fileBlockSize + 1;
-            long filePosition = cache.enlargeFileSpace((long) dirBlockCount
-                * fileBlockSize);
+            int  index              = getBlockIndexLimit();
+            long filePosition       = (long) index * fileBlockSize;
+            long  dirSpaceBlockCount = dirObjectSize / fileBlockSize + 1;
+            long delta = dirSpaceBlockCount * fileBlockSize;
 
+            cache.enlargeFileSpace(filePosition + delta);
             directorySpaceManager.addFileBlock(filePosition,
-                                               filePosition
-                                               + (long) dirBlockCount
-                                                 * fileBlockSize);
-            createFileBlocksInDirectory(index, dirBlockCount,
+                                               filePosition + delta);
+            createFileBlocksInDirectory(index, (int) dirSpaceBlockCount,
                                         tableIdDirectory);
-
-            // integrity check
-            cacheFreePos = cache.getFileFreePos();
-            index        = getBlockIndexLimit();
-
-            if ((long) index * fileBlockSize != cacheFreePos) {
-                cache.logSevereEvent(
-                    "space manager end file pos different from data file: "
-                    + (index * fileBlockSize) + ", " + cacheFreePos, null);
-            }
         }
     }
 
@@ -295,16 +280,12 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
     private long getNewFileBlocksNoCheck(int tableId, int blockCount) {
 
-        long index        = getBlockIndexLimit();
-        long filePosition = index * fileBlockSize;
-        long delta = filePosition + ((long) blockCount * fileBlockSize)
-                     - cache.getFileFreePos();
+        int  index        = getBlockIndexLimit();
+        long filePosition = (long) index * fileBlockSize;
+        long delta        = (long) blockCount * fileBlockSize;
 
-        if (delta > 0) {
-            cache.enlargeFileSpace(delta);
-        }
-
-        createFileBlocksInDirectory((int) index, blockCount, tableId);
+        cache.enlargeFileSpace(filePosition + delta);
+        createFileBlocksInDirectory(index, blockCount, tableId);
 
         return filePosition;
     }
@@ -324,9 +305,8 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         bitMapStore.add(bitMap, false);
 
         //
-        int bitmapBlockPos = (int) (bitMap.getPos() * dataFileScale
-                                    / DataSpaceManager.fixedBlockSizeUnit);
-        int blockOffset = fileBlockIndex % dirBlockSize;
+        int bitmapBlockPos = getFileBlockPosFromPosition(bitMap.getPos());
+        int blockOffset    = fileBlockIndex % dirBlockSize;
         DirectoryBlockCachedObject directory = getDirectory(fileBlockIndex,
             true);
 
@@ -352,18 +332,14 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
     private DirectoryBlockCachedObject getDirectoryByIndex(int indexInRoot,
             boolean keep) {
 
-        long                       position = rootBlock.getValue(indexInRoot);
-        DirectoryBlockCachedObject directory;
+        int  blockPos = rootBlock.getValue(indexInRoot);
+        long position = getPositionFromFileBlock(blockPos);
 
         if (position == 0) {
             return null;
         }
 
-        position *= (DataSpaceManager.fixedBlockSizeUnit / dataFileScale);
-        directory = (DirectoryBlockCachedObject) directoryStore.get(position,
-                keep);
-
-        return directory;
+        return (DirectoryBlockCachedObject) directoryStore.get(position, keep);
     }
 
     private void createDirectory(int fileBlockIndex) {
@@ -374,9 +350,8 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
         directoryStore.add(directory, false);
 
-        int indexInRoot = fileBlockIndex / dirBlockSize;
-        int blockPosition = (int) (directory.getPos() * dataFileScale
-                                   / DataSpaceManager.fixedBlockSizeUnit);
+        int indexInRoot   = fileBlockIndex / dirBlockSize;
+        int blockPosition = getFileBlockPosFromPosition(directory.getPos());
 
         rootBlock.setValue(indexInRoot, blockPosition);
     }
@@ -506,7 +481,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
     public TableSpaceManager getTableSpace(int spaceId) {
 
-        if (spaceId == DataSpaceManager.tableIdDefault) {
+        if (spaceId == tableIdDefault) {
             return defaultSpaceManager;
         }
 
@@ -521,12 +496,9 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                 (TableSpaceManagerBlocks) spaceManagerList.get(spaceId);
 
             if (manager == null) {
-                int minReuse = cache.database.logger.propMinReuse;
-
                 manager = new TableSpaceManagerBlocks(
                     this, spaceId, fileBlockSize,
-                    cache.database.logger.propMaxFreeBlocks, dataFileScale,
-                    minReuse);
+                    cache.database.logger.propMaxFreeBlocks, dataFileScale);
 
                 spaceManagerList.put(spaceId, manager);
             }
@@ -555,9 +527,12 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
             if (tableSpace != null) {
                 tableSpace.reset();
+
                 // do not remove from spaceManagerList - can be truncate
             }
 
+            lastBlocks.removeKey(spaceId);
+
             IntIndex list = new IntIndex(16, false);
 
             ba.initialise(true);
@@ -578,47 +553,31 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         }
     }
 
-    public void freeTableSpace(int spaceId, DoubleIntIndex spaceList,
-                               long offset, long limit, boolean full) {
+    public void freeTableSpace(int spaceId, LongLookup spaceList, long offset,
+                               long limit) {
 
         if (spaceList.size() == 0 && offset == limit) {
             return;
         }
 
-        // sorts by keys
-        spaceList.compactLookupAsIntervals();
-
-        if (!full) {
-            int available = spaceList.capacity() - spaceList.size();
-
-            if (available > spaceList.capacity() / 4) {
-                spaceList.setValuesSearchTarget();
-                spaceList.sort();
-
-                return;
-            }
-        }
-
         cache.writeLock.lock();
 
         try {
             ba.initialise(true);
 
             try {
-
-                // spaceId may be the tableIdDefault for moved spaces
-                int[] keys   = spaceList.getKeys();
-                int[] values = spaceList.getValues();
+                long position;
+                int  units;
 
                 for (int i = 0; i < spaceList.size(); i++) {
-                    int position = keys[i];
-                    int units    = values[i];
+                    position = spaceList.getLongKey(i);
+                    units    = (int) spaceList.getLongValue(i);
 
                     freeTableSpacePart(position, units);
                 }
 
-                long position = offset / dataFileScale;
-                int  units    = (int) ((limit - offset) / dataFileScale);
+                position = offset / dataFileScale;
+                units    = (int) ((limit - offset) / dataFileScale);
 
                 freeTableSpacePart(position, units);
             } finally {
@@ -627,9 +586,6 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         } finally {
             cache.writeLock.unlock();
         }
-
-        spaceList.clear();
-        spaceList.setValuesSearchTarget();
     }
 
     private void freeTableSpacePart(long position, int units) {
@@ -717,7 +673,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                     boolean result = ba.moveToBlock(block);
 
                     if (result) {
-                        ba.setTable(DataSpaceManager.tableIdSetAside);
+                        ba.setTable(tableIdSetAside);
                     }
                 }
             } finally {
@@ -784,9 +740,8 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                 TableSpaceManagerBlocks tableSpace =
                     (TableSpaceManagerBlocks) it.next();
 
-                if (tableSpace.getSpaceID() == DataSpaceManager
-                        .tableIdDirectory || tableSpace
-                        .getFileBlockIndex() != -1) {
+                if (tableSpace.getSpaceID() == tableIdDirectory
+                        || tableSpace.getFileBlockIndex() != -1) {
                     initialiseTableSpace(tableSpace);
                 }
             }
@@ -807,6 +762,15 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                     (TableSpaceManagerBlocks) it.next();
 
                 tableSpace.reset();
+
+                int lastBlockIndex = tableSpace.getFileBlockIndex();
+                int spaceId;
+
+                if (lastBlockIndex >= 0) {
+                    spaceId = tableSpace.getSpaceID();
+
+                    lastBlocks.addKey(spaceId, lastBlockIndex);
+                }
             }
         } finally {
             cache.writeLock.unlock();
@@ -823,18 +787,13 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
 
     public DirectoryBlockCachedObject[] getDirectoryList() {
 
-        int                          count = 0;
+        int                          count = rootBlock.getNonZeroSize();
         DirectoryBlockCachedObject[] directoryList;
-        int[]                        rootArray = rootBlock.getIntArray();
-
-        while (rootArray[count] != 0) {
-            count++;
-        }
 
         directoryList = new DirectoryBlockCachedObject[count];
 
         for (int i = 0; i < directoryList.length; i++) {
-            directoryList[i] = getDirectory(i * dirBlockSize, false);
+            directoryList[i] = getDirectoryByIndex(i, false);
         }
 
         return directoryList;
@@ -846,14 +805,9 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
      */
     DoubleIntIndex checkDirectorySpaces() {
 
-        DirectoryBlockCachedObject[] directoryList = getDirectoryList();
-        DoubleIntIndex offspaceBitmaps = new DoubleIntIndex(8, false);
-
-        offspaceBitmaps.setKeysSearchTarget();
-
-        DoubleIntIndex positionBitmaps = new DoubleIntIndex(8, false);
-
-        positionBitmaps.setKeysSearchTarget();
+        DirectoryBlockCachedObject[] directoryList   = getDirectoryList();
+        DoubleIntIndex               offspaceBitmaps = new DoubleIntIndex(8);
+        DoubleIntIndex               positionBitmaps = new DoubleIntIndex(8);
 
         for (int i = 0; i < directoryList.length; i++) {
             DirectoryBlockCachedObject dir        = directoryList[i];
@@ -862,8 +816,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
             int                        blockIndex = i;
             int blockPos = rootBlock.getValue(blockIndex);
             boolean                    result;
-            int count = dir.getStorageSize()
-                        / DataSpaceManager.fixedBlockSizeUnit;
+            int count = dir.getStorageSize() / fixedDiskBlockSize;
 
             for (int j = 0; j < count; j++) {
                 result = positionBitmaps.addUnique(blockPos, blockIndex);
@@ -878,13 +831,11 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                     break;
                 }
 
-                position = blockPos
-                           * (DataSpaceManager.fixedBlockSizeUnit
-                              / dataFileScale);
+                position   = getPositionFromFileBlock(blockPos);
                 spaceId    = findTableSpace(position);
                 blockIndex = i * dirBlockSize + j;
 
-                if (spaceId != DataSpaceManager.tableIdDirectory) {
+                if (spaceId != tableIdDirectory) {
                     offspaceBitmaps.add(blockIndex, spaceId);
                 } else {
                     result = positionBitmaps.addUnique(blockPos, blockIndex);
@@ -911,12 +862,9 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
      */
     DoubleIntIndex checkDirectoryBitmaps(DirectoryBlockCachedObject mismatch) {
 
-        DirectoryBlockCachedObject[] directoryList = getDirectoryList();
-        DoubleIntIndex offspaceBitmaps = new DoubleIntIndex(8, false);
-
-        offspaceBitmaps.setKeysSearchTarget();
-
-        int mismatchCount = 0;
+        DirectoryBlockCachedObject[] directoryList   = getDirectoryList();
+        DoubleIntIndex               offspaceBitmaps = new DoubleIntIndex(8);
+        int                          mismatchCount   = 0;
 
         for (int i = 0; i < directoryList.length; i++) {
             DirectoryBlockCachedObject dir = directoryList[i];
@@ -929,11 +877,9 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                     break;
                 }
 
-                long position = blockPos
-                                * (DataSpaceManager.fixedBlockSizeUnit
-                                   / dataFileScale);
-                int spaceId    = findTableSpace(position);
-                int blockIndex = i * dirBlockSize + j;
+                long position   = getPositionFromFileBlock(blockPos);
+                int  spaceId    = findTableSpace(position);
+                int  blockIndex = i * dirBlockSize + j;
                 BitMapCachedObject currentBitMap =
                     (BitMapCachedObject) bitMapStore.get(position, false);
 
@@ -962,27 +908,39 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         return offspaceBitmaps;
     }
 
-    private void initialiseTableSpace(TableSpaceManagerBlocks tableSpace) {
+    private int findLastFreeSpace(int spaceId) {
+        return lastBlocks.getValue(spaceId, -1);
+    }
+
+    public void initialiseTableSpace(TableSpaceManagerBlocks tableSpace) {
 
         int spaceId        = tableSpace.getSpaceID();
-        int blockIndex     = -1;
         int lastBlockIndex = tableSpace.getFileBlockIndex();
 
+        if (lastBlockIndex < 0) {
+            lastBlockIndex = findLastFreeSpace(spaceId);
+        }
+
         if (lastBlockIndex >= 0) {
-            if (hasFreeSpace(spaceId, lastBlockIndex)) {
-                blockIndex = lastBlockIndex;
+            long position = lastBlockIndex * fileBlockItemCount;
+            int  id       = findTableSpace(position);
+
+            if (id != spaceId) {
+                lastBlockIndex = -1;
             }
         }
 
-        if (blockIndex < 0) {
-            blockIndex = findLargestFreeSpace(spaceId);
+        if (lastBlockIndex < 0) {
+            lastBlockIndex = findLargestFreeSpace(spaceId);
         }
 
-        if (blockIndex < 0) {
+        if (lastBlockIndex < 0) {
             return;
         }
 
-        initialiseTableSpace(tableSpace, blockIndex);
+        if (hasFreeSpace(spaceId, lastBlockIndex)) {
+            initialiseTableSpace(tableSpace, lastBlockIndex);
+        }
     }
 
     private boolean hasFreeSpace(int spaceId, int blockIndex) {
@@ -1058,6 +1016,18 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
         }
     }
 
+    /**
+     * input is in units of fixedBlockSizeUnit
+     * output is in units of dataFileScale
+     */
+    long getPositionFromFileBlock(int fixedBlockPos) {
+        return fixedBlockPos * (long) (fixedDiskBlockSize / dataFileScale);
+    }
+
+    int getFileBlockPosFromPosition(long position) {
+        return (int) (position / (fixedDiskBlockSize / dataFileScale));
+    }
+
     private class BlockAccessor {
 
         boolean                    currentKeep;
@@ -1126,8 +1096,7 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                 }
 
                 if (currentKeep) {
-                    position *= (DataSpaceManager.fixedBlockSizeUnit
-                                 / dataFileScale);
+                    position *= (fixedDiskBlockSize / dataFileScale);
                     currentBitMap =
                         (BitMapCachedObject) bitMapStore.get(position, true);
                 }
@@ -1186,8 +1155,8 @@ public class DataSpaceManagerBlocks implements DataSpaceManager {
                 int currentId =
                     currentDir.getTableIdArray()[currentBlockOffset];
 
-                if (currentId != DataSpaceManager.tableIdSetAside) {
-                    setTable(DataSpaceManager.tableIdEmpty);
+                if (currentId != tableIdSetAside) {
+                    setTable(tableIdEmpty);
                     emptySpaceList.addUnique(currentBlockIndex);
 
                     released++;
