@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.lib.StringUtil;
  * Manages a .properties file for a database.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.0
  */
 public class HsqlDatabaseProperties extends HsqlProperties {
@@ -133,14 +133,16 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final int SQL_PROPERTY    = 2;
 
     // db files modified
-    public static final int     FILES_NOT_MODIFIED = 0;
-    public static final int     FILES_MODIFIED     = 1;
-    public static final int     FILES_MODIFIED_NEW = 2;
-    public static final int     FILES_NEW          = 3;
-    private static final String MODIFIED_NO        = "no";
-    private static final String MODIFIED_YES       = "yes";
-    private static final String MODIFIED_YES_NEW   = "yes-new-files";
-    private static final String MODIFIED_NO_NEW    = "no-new-files";
+    public static final int     FILES_NOT_MODIFIED      = 0;
+    public static final int     FILES_MODIFIED          = 1;
+    public static final int     FILES_MODIFIED_NEW      = 2;
+    public static final int     FILES_MODIFIED_NEW_DATA = 3;
+    public static final int     FILES_NEW               = 4;
+    private static final String MODIFIED_NO             = "no";
+    private static final String MODIFIED_YES            = "yes";
+    private static final String MODIFIED_YES_NEW        = "yes-new-files";
+    private static final String MODIFIED_YES_NEW_DATA   = "yes-new-files-data";
+    private static final String MODIFIED_NO_NEW         = "no-new-files";
 
     // allowed property metadata
     private static final HashMap dbMeta   = new HashMap(67);
@@ -152,19 +154,19 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
 //#ifdef JAVA8
 
-    public static final String THIS_VERSION         = "2.5.0";
-    public static final String THIS_FULL_VERSION    = "2.5.0";
+    public static final String THIS_VERSION         = "2.5.1";
+    public static final String THIS_FULL_VERSION    = "2.5.1";
     public static final int    MAJOR                = 2,
                                MINOR                = 5,
-                               REVISION             = 0;
+                               REVISION             = 1;
 
 //#else
 /*
-    public static final String THIS_VERSION      = "2.3.7";
-    public static final String THIS_FULL_VERSION = "2.3.7";
+    public static final String THIS_VERSION      = "2.3.8";
+    public static final String THIS_FULL_VERSION = "2.3.8";
     public static final int    MAJOR             = 2,
                                MINOR             = 3,
-                               REVISION          = 7;
+                               REVISION          = 8;
 */
 
 //#endif JAVA8
@@ -306,6 +308,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final String textdb_vs           = "textdb.vs";
     public static final String textdb_lvs          = "textdb.lvs";
     public static final String textdb_qc           = "textdb.qc";
+    public static final String textdb_null_def     = "textdb.null_def";
 
     //
     public static final String hsqldb_min_reuse = "hsqldb.min_reuse";
@@ -325,6 +328,9 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         textMeta.put(textdb_ignore_first,
                      HsqlProperties.getMeta(textdb_ignore_first, SQL_PROPERTY,
                                             false));
+        textMeta.put(textdb_null_def,
+                     HsqlProperties.getMeta(textdb_null_def, SQL_PROPERTY,
+                                            false));
         textMeta.put(textdb_fs,
                      HsqlProperties.getMeta(textdb_fs, SQL_PROPERTY, ","));
         textMeta.put(textdb_vs,
@@ -663,8 +669,8 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                 Long.toString(database.logger.getFilesTimestamp()));
             props.setProperty(hsqldb_modified, getProperty(hsqldb_modified));
             props.save(fileName + ".properties" + ".new");
-            fa.renameElement(fileName + ".properties" + ".new",
-                             fileName + ".properties", true);
+            fa.renameElementOrCopy(fileName + ".properties" + ".new",
+                                   fileName + ".properties");
         } catch (Throwable t) {
             database.logger.logSevereEvent("save failed", t);
 
@@ -847,6 +853,10 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                 value = MODIFIED_YES_NEW;
                 break;
 
+            case FILES_MODIFIED_NEW_DATA :
+                value = MODIFIED_YES_NEW_DATA;
+                break;
+
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500,
                                          "HsqlDatabaseProperties");
@@ -864,6 +874,8 @@ public class HsqlDatabaseProperties extends HsqlProperties {
             return FILES_MODIFIED;
         } else if (MODIFIED_YES_NEW.equals(value)) {
             return FILES_MODIFIED_NEW;
+        } else if (MODIFIED_YES_NEW_DATA.equals(value)) {
+            return FILES_MODIFIED_NEW_DATA;
         } else if (MODIFIED_NO_NEW.equals(value)) {
             return FILES_NEW;
         }
