@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,10 +34,6 @@ package org.hsqldb.persist;
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.io.LineNumberReader;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
@@ -48,18 +44,15 @@ import java.util.zip.Inflater;
 import org.hsqldb.Database;
 import org.hsqldb.DatabaseType;
 import org.hsqldb.HsqlException;
-import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.Session;
 import org.hsqldb.SessionInterface;
 import org.hsqldb.Statement;
-import org.hsqldb.Table;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlByteArrayInputStream;
 import org.hsqldb.lib.LineGroupReader;
-import org.hsqldb.lib.java.JavaSystem;
 import org.hsqldb.map.ValuePool;
 import org.hsqldb.navigator.RowSetNavigator;
 import org.hsqldb.result.Result;
@@ -75,14 +68,13 @@ import org.hsqldb.types.Types;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class LobManager {
 
     static final String resourceFileName =
         "/org/hsqldb/resources/lob-schema.sql";
-    static final String[] starters = new String[]{ "/*" };
 
     //
     Database         database;
@@ -124,7 +116,7 @@ public class LobManager {
     Statement createPart;
 
     //
-    boolean usageChanged;
+    long usageChanged;
 
     //
     ReadWriteLock lock      = new ReentrantReadWriteLock();
@@ -258,33 +250,17 @@ public class LobManager {
 
         sysLobSession = database.sessionManager.getSysLobSession();
 
-        InputStream fis = AccessController.doPrivileged(
-                new PrivilegedAction<InputStream>() {
-
-                    public InputStream run() {
-                        return getClass().getResourceAsStream(resourceFileName);
-                    }
-                });
-        InputStreamReader reader = new InputStreamReader(fis, JavaSystem.CS_ISO_8859_1);
-        LineNumberReader lineReader = new LineNumberReader(reader);
-        LineGroupReader lg = new LineGroupReader(lineReader, starters);
-        HashMappedList map = lg.getAsMap();
-
-        lg.close();
-
-        String    sql       = (String) map.get("/*lob_schema_definition*/");
-        Statement statement = sysLobSession.compileStatement(sql);
-        Result    result    = statement.execute(sysLobSession);
+        HashMappedList map = LineGroupReader.getStatementMap(resourceFileName);
+        String         sql = (String) map.get("/*lob_schema_definition*/");
+        Statement      statement = sysLobSession.compileStatement(sql);
+        Result         result    = statement.execute(sysLobSession);
 
         if (result.isError()) {
             throw result.getException();
         }
 
-        HsqlName name =
-            database.schemaManager.getSchemaHsqlName("SYSTEM_LOBS");
-        Table table = database.schemaManager.getUserTable("BLOCKS",
-            "SYSTEM_LOBS");
-
+        // throws if schema not created
+        database.schemaManager.getSchemaHsqlName("SYSTEM_LOBS");
         compileStatements();
     }
 
@@ -359,7 +335,7 @@ public class LobManager {
         compressLobs = database.logger.propCompressLobs;
 
         if (compressLobs || cryptLobs) {
-            int largeBufferBlockSize = largeLobBlockSize + 4 * 1024;
+            int largeBufferBlockSize = largeLobBlockSize + 32 * 1024;
 
             inflater   = new Inflater();
             deflater   = new Deflater(Deflater.BEST_SPEED);
@@ -519,8 +495,6 @@ public class LobManager {
             Result result = sysLobSession.executeCompiledStatement(createLob,
                 params, 0);
 
-            usageChanged = true;
-
             return lobID.longValue();
         } finally {
             writeLock.unlock();
@@ -544,8 +518,6 @@ public class LobManager {
             Result result = sysLobSession.executeCompiledStatement(createLob,
                 params, 0);
 
-            usageChanged = true;
-
             return lobID.longValue();
         } finally {
             writeLock.unlock();
@@ -567,7 +539,7 @@ public class LobManager {
                 sysLobSession.executeCompiledStatement(deleteLobCall, params,
                     0);
 
-            usageChanged = true;
+            usageChanged += lobBlockSize;
 
             return result;
         } finally {
@@ -575,8 +547,8 @@ public class LobManager {
         }
     }
 
-    public void setUsageChanged() {
-        usageChanged = true;
+    public long getUsageChanged() {
+        return usageChanged;
     }
 
     public Result deleteUnusedLobs() {
@@ -584,29 +556,15 @@ public class LobManager {
         writeLock.lock();
 
         try {
-            if (lobStore == null || byteBuffer == null || !usageChanged) {
+            if (lobStore == null || byteBuffer == null || usageChanged == 0) {
                 return Result.updateZeroResult;
             }
 
-            Session[] sessions   = database.sessionManager.getAllSessions();
-            long      firstLobID = Long.MAX_VALUE;
+            long           limitLobID = Long.MAX_VALUE;
+            ResultMetaData meta = deleteUnusedLobs.getParametersMetaData();
+            Object[]       params     = new Object[meta.getColumnCount()];
 
-            for (int i = 0; i < sessions.length; i++) {
-                if (sessions[i].isClosed()) {
-                    continue;
-                }
-
-                long sessionLobID = sessions[i].sessionData.getFirstLobID();
-
-                if (sessionLobID != 0 && sessionLobID < firstLobID) {
-                    firstLobID = sessionLobID;
-                }
-            }
-
-            ResultMetaData meta   = deleteUnusedLobs.getParametersMetaData();
-            Object[]       params = new Object[meta.getColumnCount()];
-
-            params[0] = Long.valueOf(firstLobID);
+            params[0] = Long.valueOf(limitLobID);
 
             Result result =
                 sysLobSession.executeCompiledStatement(deleteUnusedLobs,
@@ -638,8 +596,8 @@ public class LobManager {
                 return result;
             }
 
-            // result is empty when there is no lob, or one row
-            usageChanged = false;
+            // result is empty when there is no lob, or it has one row
+            usageChanged = 0;
 
             long            sizeLimit = 0;
             RowSetNavigator navigator = result.getNavigator();
@@ -1105,8 +1063,6 @@ public class LobManager {
                 return result;
             }
 
-            usageChanged = true;
-
             if (newLength == 0) {
                 return ResultLob.newLobSetResponse(newLobID.longValue(),
                                                    newLength);
@@ -1125,7 +1081,12 @@ public class LobManager {
                 newBlockCount++;
             }
 
-            createBlockAddresses(newLobID.longValue(), 0, newBlockCount);
+            result = createBlockAddresses(newLobID.longValue(), 0,
+                                          newBlockCount);
+
+            if (result.isError()) {
+                return result;
+            }
 
             // copy the contents
             int[][] sourceBlocks = getBlockAddresses(lobID, 0,
@@ -1137,6 +1098,8 @@ public class LobManager {
                 copyBlockSet(sourceBlocks, targetBlocks);
             } catch (HsqlException e) {
                 return Result.newErrorResult(e);
+            } catch (Throwable e) {
+                return Result.newErrorResult(e);
             }
 
             // clear the end block unused space
@@ -1496,8 +1459,7 @@ public class LobManager {
             blockLimit++;
         }
 
-        createBlockAddresses(lobID, 0, blockLimit);
-
+        Result  actionResult   = createBlockAddresses(lobID, 0, blockLimit);
         int[][] blockAddresses = getBlockAddresses(lobID, 0, blockLimit);
 
         for (int i = 0; i < blockAddresses.length; i++) {
@@ -1519,7 +1481,7 @@ public class LobManager {
                                                     localLength);
 
                         if (read == -1) {
-                            return Result.newErrorResult(new EOFException());
+                            break;
                         }
 
                         localLength -= read;
@@ -1598,21 +1560,38 @@ public class LobManager {
         }
     }
 
+    /**
+     * for new blob only
+     *
+     * blob length is set to inputStream size (not given length)
+     */
     public Result setBytesForNewBlob(long lobID, InputStream inputStream,
                                      long length) {
 
-        if (length == 0) {
-            return ResultLob.newLobSetResponse(lobID, 0);
-        }
-
         if (byteBuffer == null) {
-            throw Error.error(ErrorCode.DATA_IS_READONLY);
+            return Result.newErrorResult(
+                Error.error(ErrorCode.DATA_IS_READONLY));
         }
 
         writeLock.lock();
 
         try {
             Result result = setBytesIS(lobID, inputStream, length, false);
+            Result actionResult = Result.updateZeroResult;
+
+            if (result.isError()) {
+                return result;
+            }
+
+            long newLength = ((ResultLob) result).getBlockLength();
+
+            if (newLength < length) {
+                actionResult = truncate(lobID, newLength);
+            }
+
+            if (actionResult.isError()) {
+                return actionResult;
+            }
 
             return result;
         } finally {
@@ -1671,10 +1650,6 @@ public class LobManager {
     public Result setCharsForNewClob(long lobID, InputStream inputStream,
                                      long length) {
 
-        if (length == 0) {
-            return ResultLob.newLobSetResponse(lobID, 0);
-        }
-
         if (byteBuffer == null) {
             throw Error.error(ErrorCode.DATA_IS_READONLY);
         }
@@ -1775,7 +1750,10 @@ public class LobManager {
 
         Result result = updateLobUsage.execute(session);
 
-        usageChanged = true;
+        // todo - return a value in result and check - UPDATE decrements (to zero) then increments usage
+        if (delta < 0) {
+            usageChanged += lobBlockSize;
+        }
 
         session.sessionContext.pop();
 
@@ -2116,14 +2094,8 @@ public class LobManager {
     private Result setBytesISCompressed(long lobID, InputStream inputStream,
                                         long length, boolean isClob) {
 
-        long   offset = 0;
-        byte[] tempBuffer;
-
-        if (length < largeLobBlockSize) {
-            tempBuffer = new byte[(int) length];
-        } else {
-            tempBuffer = new byte[largeLobBlockSize];
-        }
+        long   offset     = 0;
+        byte[] tempBuffer = new byte[largeLobBlockSize];
 
         while (true) {
             int localLength = tempBuffer.length;
