@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -86,7 +86,7 @@ import org.hsqldb.types.Type;
  *  storage.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.0
  */
 public class Logger implements EventLogInterface {
@@ -105,15 +105,15 @@ public class Logger implements EventLogInterface {
 
     //
     private boolean propIsFileDatabase;
-    boolean         propIncrementBackup;
     boolean         propNioDataFile;
     long            propNioMaxSize    = 256 * 1024 * 1024L;
     int             propMaxFreeBlocks = 512;
     int             propMinReuse      = 0;
     private int     propCacheMaxRows;
     private long    propCacheMaxSize;
-    int             propCacheDefragLimit;
+    int             propDataFileDefragLimit;
     private int     propDataFileScale;
+    int             propDataFileSpace;
     String          propTextSourceDefault = "";
     boolean         propTextAllowFullPath;
     private int     propWriteDelay;
@@ -128,8 +128,7 @@ public class Logger implements EventLogInterface {
     boolean         propCompressLobs;
     int             propScriptFormat = 0;
     boolean         propLargeData;
-    int             propFileSpaceValue;
-    long            propFileTimestamp;
+    long            propFilesTimestamp;
 
     //
     Log               log;
@@ -153,6 +152,9 @@ public class Logger implements EventLogInterface {
     AtomicInteger backupState     = new AtomicInteger();
     AtomicInteger checkpointState = new AtomicInteger();
 
+    //
+    long maxLogSize;
+
     //
     static final int largeDataFactor = 128;
 
@@ -209,25 +211,38 @@ public class Logger implements EventLogInterface {
             hasScript = fileAccess.isStreamElement(database.getPath()
                                                    + scriptFileExtension);
 
-            boolean exists;
+            boolean version18 = database.databaseProperties.isVersion18();
 
-            if (database.databaseProperties.isVersion18()) {
-                exists = hasFileProps;
+            if (version18) {
+                throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION,
+                                  "unsupported old database version");
+            }
 
-                database.databaseProperties.setProperty(
-                    HsqlDatabaseProperties.hsqldb_inc_backup, false);
-            } else {
-                exists = hasScript;
+            boolean exists = hasScript;
 
-                if (!exists) {
-                    exists =
-                        fileAccess.isStreamElement(database.getPath()
-                                                   + scriptFileExtension
-                                                   + Logger.newFileExtension);
+            if (!exists) {
+                exists = fileAccess.isStreamElement(database.getPath()
+                                                    + scriptFileExtension
+                                                    + Logger.newFileExtension);
+
+                if (exists) {
+                    database.databaseProperties.setDBModified(
+                        HsqlDatabaseProperties.FILES_MODIFIED_NEW);
+                } else {
+                    exists = fileAccess.isStreamElement(database.getPath()
+                                                        + dataFileExtension);
 
                     if (exists) {
-                        database.databaseProperties.setDBModified(
-                            HsqlDatabaseProperties.FILES_MODIFIED_NEW);
+                        throw Error.error(ErrorCode.DATA_FILE_ERROR,
+                                          "database files not complete");
+                    }
+
+                    exists = fileAccess.isStreamElement(database.getPath()
+                                                        + backupFileExtension);
+
+                    if (exists) {
+                        throw Error.error(ErrorCode.DATA_FILE_ERROR,
+                                          "database files not complete");
                     }
                 }
             }
@@ -315,18 +330,6 @@ public class Logger implements EventLogInterface {
             acquireLock(database.getPath());
         }
 
-        boolean version18 = database.databaseProperties.isVersion18();
-
-        if (version18) {
-            database.setDatabaseName(newUniqueName());
-            database.schemaManager.createPublicSchema();
-
-            HsqlName name = database.schemaManager.findSchemaHsqlName(
-                SqlInvariants.PUBLIC_SCHEMA);
-
-            database.schemaManager.setDefaultSchemaHsqlName(name);
-        }
-
         log = new Log(database);
 
         log.open();
@@ -334,10 +337,6 @@ public class Logger implements EventLogInterface {
         logsStatements = true;
         loggingEnabled = propLogData && !database.isFilesReadOnly();
 
-        if (version18) {
-            checkpoint(null, false, false);
-        }
-
         if (database.getNameString().length() == 0) {
             database.setDatabaseName(newUniqueName());
         }
@@ -403,9 +402,6 @@ public class Logger implements EventLogInterface {
 
         propScriptFormat = database.databaseProperties.getIntegerProperty(
             HsqlDatabaseProperties.hsqldb_script_format);
-
-        boolean version18 = database.databaseProperties.isVersion18();
-
         propMaxFreeBlocks = database.databaseProperties.getIntegerProperty(
             HsqlDatabaseProperties.hsqldb_cache_free_count);
         propMaxFreeBlocks = ArrayUtil.getTwoPowerFloor(propMaxFreeBlocks);
@@ -425,15 +421,10 @@ public class Logger implements EventLogInterface {
 
         database.granteeManager.setDigestAlgo(temp);
 
-        if (!isNewDatabase && !version18) {
+        if (!isNewDatabase) {
             return;
         }
 
-        temp = database.databaseProperties.getStringProperty(
-            HsqlDatabaseProperties.hsqldb_digest);
-
-        database.granteeManager.setDigestAlgo(temp);
-
         if (tempDirectoryPath != null) {
             int rows = database.databaseProperties.getIntegerProperty(
                 HsqlDatabaseProperties.hsqldb_result_max_memory_rows);
@@ -563,8 +554,6 @@ public class Logger implements EventLogInterface {
             database.setReadOnly();
         }
 
-        propIncrementBackup = database.databaseProperties.isPropertyTrue(
-            HsqlDatabaseProperties.hsqldb_inc_backup);
         propNioDataFile = database.databaseProperties.isPropertyTrue(
             HsqlDatabaseProperties.hsqldb_nio_data_file);
         propNioMaxSize =
@@ -579,7 +568,7 @@ public class Logger implements EventLogInterface {
         setLobFileScaleNoCheck(
             database.databaseProperties.getIntegerProperty(
                 HsqlDatabaseProperties.hsqldb_lob_file_scale));
-        this.setLobFileCompressedNoCheck(
+        setLobFileCompressedNoCheck(
             database.databaseProperties.isPropertyTrue(
                 HsqlDatabaseProperties.hsqldb_lob_file_compressed));
         setDataFileScaleNoCheck(
@@ -591,11 +580,12 @@ public class Logger implements EventLogInterface {
             HsqlDatabaseProperties.hsqldb_files_space, 0);
 
         if (fileSpace != 0) {
-            setDataFileSpaces(fileSpace);
+            setDataFileSpace(fileSpace);
         }
 
-        propCacheDefragLimit = database.databaseProperties.getIntegerProperty(
-            HsqlDatabaseProperties.hsqldb_defrag_limit);
+        propDataFileDefragLimit =
+            database.databaseProperties.getIntegerProperty(
+                HsqlDatabaseProperties.hsqldb_defrag_limit);
         propWriteDelay = database.databaseProperties.getIntegerProperty(
             HsqlDatabaseProperties.hsqldb_write_delay_millis);
 
@@ -604,8 +594,10 @@ public class Logger implements EventLogInterface {
             propWriteDelay = 0;
         }
 
-        propLogSize = database.databaseProperties.getIntegerProperty(
-            HsqlDatabaseProperties.hsqldb_log_size);
+        setLogSize(
+            database.databaseProperties.getIntegerProperty(
+                HsqlDatabaseProperties.hsqldb_log_size));
+
         propLogData = database.databaseProperties.isPropertyTrue(
             HsqlDatabaseProperties.hsqldb_log_data);
         propGC = database.databaseProperties.getIntegerProperty(
@@ -658,7 +650,7 @@ public class Logger implements EventLogInterface {
             switch (closemode) {
 
                 case Database.CLOSEMODE_IMMEDIATELY :
-                    log.shutdown();
+                    log.closeImmediately();
                     break;
 
                 case Database.CLOSEMODE_NORMAL :
@@ -715,7 +707,7 @@ public class Logger implements EventLogInterface {
     }
 
     public boolean isAllowedFullPath() {
-        return this.propTextAllowFullPath;
+        return propTextAllowFullPath;
     }
 
     /**
@@ -962,6 +954,11 @@ public class Logger implements EventLogInterface {
 
         if (loggingEnabled) {
             log.writeCommitStatement(session);
+        } else {
+            if (maxLogSize > 0
+                    && database.lobManager.getUsageChanged() > maxLogSize) {
+                setCheckpointRequired();
+            }
         }
     }
 
@@ -1028,6 +1025,7 @@ public class Logger implements EventLogInterface {
     public synchronized void setLogSize(int megas) {
 
         propLogSize = megas;
+        maxLogSize  = propLogSize * 1024L * 1024;
 
         if (log != null) {
             log.setLogSize(propLogSize);
@@ -1100,24 +1098,6 @@ public class Logger implements EventLogInterface {
         return propLobBlockSize;
     }
 
-    public synchronized void setIncrementBackup(boolean val) {
-
-        if (val == propIncrementBackup) {
-            return;
-        }
-
-        if (log != null) {
-            log.setIncrementBackup(val);
-
-            if (log.hasCache()) {
-                checkpointState.compareAndSet(stateCheckpointNormal,
-                                              stateCheckpointRequired);
-            }
-        }
-
-        propIncrementBackup = val;
-    }
-
     public void setCacheMaxRows(int value) {
         propCacheMaxRows = value;
     }
@@ -1135,26 +1115,16 @@ public class Logger implements EventLogInterface {
     }
 
     public void setCacheMinReuseSize(int value) {
-        this.propMinReuse = ArrayUtil.getTwoPowerFloor(value);
+        propMinReuse = ArrayUtil.getTwoPowerFloor(value);
     }
 
     public void setDataFileScale(int value) {
 
-        if (propDataFileScale == value) {
-            return;
-        }
-
-        checkPower(value, 10);
-
-        if (value < 8 && value != 1) {
+        if (value < 16) {
             throw Error.error(ErrorCode.X_42556);
         }
 
-        if (hasCache()) {
-            throw Error.error(ErrorCode.DATA_FILE_IN_USE);
-        }
-
-        propDataFileScale = value;
+        setDataFileScaleNoCheck(value);
     }
 
     public void setDataFileScaleNoCheck(int value) {
@@ -1166,6 +1136,10 @@ public class Logger implements EventLogInterface {
         }
 
         propDataFileScale = value;
+
+        if (propDataFileSpace > 0) {
+            propDataFileSpace = propDataFileScale / 16;    // reduce if too large
+        }
     }
 
     public int getDataFileScale() {
@@ -1177,20 +1151,16 @@ public class Logger implements EventLogInterface {
                              : 1;
     }
 
-    public void setDataFileSpaces(boolean value) {
+    public void setDataFileSpace(boolean value) {
 
         if (value) {
-            setDataFileSpaces(propDataFileScale / 16);
+            setDataFileSpace(propDataFileScale / 16);
         } else {
-            setDataFileSpaces(0);
+            setDataFileSpace(0);
         }
     }
 
-    public void setDataFileSpaces(int value) {
-
-        if (propFileSpaceValue == value) {
-            return;
-        }
+    public void setDataFileSpace(int value) {
 
         if (value != 0) {
             checkPower(value, 6);
@@ -1200,30 +1170,19 @@ public class Logger implements EventLogInterface {
             value = propDataFileScale / 16;
         }
 
-        propFileSpaceValue = value;
-
-        if (hasCache()) {
-            DataFileCache dataCache = getCache();
-            boolean       result    = dataCache.setDataSpaceManager();
-
-            if (!result) {
-                return;
-            }
-
-            database.persistentStoreCollection.setNewTableSpaces();
-        }
+        propDataFileSpace = value;
     }
 
-    public int getDataFileSpaces() {
-        return propFileSpaceValue;
+    public int getDataFileSpace() {
+        return propDataFileSpace;
     }
 
     public long getFilesTimestamp() {
-        return propFileTimestamp;
+        return propFilesTimestamp;
     }
 
     public void setFilesTimestamp(long value) {
-        propFileTimestamp = value;
+        propFilesTimestamp = value;
     }
 
     public void setLobFileScale(int value) {
@@ -1276,11 +1235,16 @@ public class Logger implements EventLogInterface {
     }
 
     public void setDefagLimit(int value) {
-        propCacheDefragLimit = value;
+
+        if (value > 0 && value < 25) {
+            value = 25;
+        }
+
+        propDataFileDefragLimit = value;
     }
 
     public int getDefragLimit() {
-        return propCacheDefragLimit;
+        return propDataFileDefragLimit;
     }
 
     public void setDefaultTextTableProperties(String source,
@@ -1289,7 +1253,7 @@ public class Logger implements EventLogInterface {
         props.setProperty(HsqlDatabaseProperties.url_check_props, true);
         database.getProperties().setURLProperties(props);
 
-        this.propTextSourceDefault = source;
+        propTextSourceDefault = source;
     }
 
     public void setNioDataFile(boolean value) {
@@ -1521,11 +1485,11 @@ public class Logger implements EventLogInterface {
         }
 
         if (HsqlDatabaseProperties.hsqldb_defrag_limit.equals(name)) {
-            return String.valueOf(propCacheDefragLimit);
+            return String.valueOf(propDataFileDefragLimit);
         }
 
         if (HsqlDatabaseProperties.hsqldb_files_space.equals(name)) {
-            return String.valueOf(propFileSpaceValue);
+            return String.valueOf(propDataFileSpace);
         }
 
         if (HsqlDatabaseProperties.hsqldb_files_readonly.equals(name)) {
@@ -1533,10 +1497,6 @@ public class Logger implements EventLogInterface {
                 HsqlDatabaseProperties.hsqldb_files_readonly);
         }
 
-        if (HsqlDatabaseProperties.hsqldb_inc_backup.equals(name)) {
-            return String.valueOf(propIncrementBackup);
-        }
-
         if (HsqlDatabaseProperties.hsqldb_large_data.equals(name)) {
             return String.valueOf(propLargeData);
         }
@@ -1705,7 +1665,7 @@ public class Logger implements EventLogInterface {
         }
 
         if (HsqlDatabaseProperties.hsqldb_min_reuse.equals(name)) {
-            return String.valueOf(this.propMinReuse);
+            return String.valueOf(propMinReuse);
         }
 
         if (HsqlDatabaseProperties.sql_sys_index_names.equals(name)) {
@@ -1820,8 +1780,8 @@ public class Logger implements EventLogInterface {
 
         list.add(sb.toString());
         sb.setLength(0);
-        sb.append("SET DATABASE ").append(Tokens.T_TRANSACTION);
-        sb.append(' ').append(Tokens.T_ROLLBACK).append(' ');
+        sb.append("SET DATABASE ").append(Tokens.T_TRANSACTION).append(' ');
+        sb.append(Tokens.T_ROLLBACK).append(' ');
         sb.append(Tokens.T_ON).append(' ');
         sb.append(Tokens.T_CONFLICT).append(' ');
         sb.append(database.txConflictRollback ? Tokens.T_TRUE
@@ -2108,8 +2068,7 @@ public class Logger implements EventLogInterface {
         sb.setLength(0);
         sb.append("SET FILES ").append(Tokens.T_BACKUP);
         sb.append(' ').append(Tokens.T_INCREMENT).append(' ');
-        sb.append(propIncrementBackup ? Tokens.T_TRUE
-                                      : Tokens.T_FALSE);
+        sb.append(Tokens.T_TRUE);
         list.add(sb.toString());
         sb.setLength(0);
         sb.append("SET FILES ").append(Tokens.T_CACHE);
@@ -2136,33 +2095,40 @@ public class Logger implements EventLogInterface {
             sb.setLength(0);
         }
 
-        sb.append("SET FILES ").append(Tokens.T_LOB).append(' ').append(
-            Tokens.T_SCALE);
+        if (propDataFileSpace != 0) {
+            sb.append("SET FILES ").append(Tokens.T_SPACE).append(' ');
+            sb.append(propDataFileSpace);
+            list.add(sb.toString());
+            sb.setLength(0);
+        }
+
+        sb.append("SET FILES ").append(Tokens.T_LOB).append(' ');
+        sb.append(Tokens.T_SCALE);
         sb.append(' ').append(getLobFileScale());
         list.add(sb.toString());
         sb.setLength(0);
 
         if (propCompressLobs) {
-            sb.append("SET FILES ").append(Tokens.T_LOB).append(' ').append(
-                Tokens.T_COMPRESSED);
-            sb.append(' ').append(propCompressLobs ? Tokens.T_TRUE
-                                                   : Tokens.T_FALSE);
+            sb.append("SET FILES ").append(Tokens.T_LOB).append(' ');
+            sb.append(Tokens.T_COMPRESSED).append(' ');
+            sb.append(propCompressLobs ? Tokens.T_TRUE
+                                       : Tokens.T_FALSE);
             list.add(sb.toString());
             sb.setLength(0);
         }
 
-        sb.append("SET FILES ").append(Tokens.T_DEFRAG);
-        sb.append(' ').append(propCacheDefragLimit);
+        sb.append("SET FILES ").append(Tokens.T_DEFRAG).append(' ');
+        sb.append(propDataFileDefragLimit);
         list.add(sb.toString());
         sb.setLength(0);
-        sb.append("SET FILES ").append(Tokens.T_NIO);
-        sb.append(' ').append(propNioDataFile ? Tokens.T_TRUE
-                                              : Tokens.T_FALSE);
+        sb.append("SET FILES ").append(Tokens.T_NIO).append(' ');
+        sb.append(propNioDataFile ? Tokens.T_TRUE
+                                  : Tokens.T_FALSE);
         list.add(sb.toString());
         sb.setLength(0);
-        sb.append("SET FILES ").append(Tokens.T_NIO).append(' ').append(
-            Tokens.T_SIZE);
-        sb.append(' ').append(propNioMaxSize / (1024 * 1024));
+        sb.append("SET FILES ").append(Tokens.T_NIO).append(' ');
+        sb.append(Tokens.T_SIZE).append(' ');
+        sb.append(propNioMaxSize / (1024 * 1024));
         list.add(sb.toString());
         sb.setLength(0);
         sb.append("SET FILES ").append(Tokens.T_LOG).append(' ');
@@ -2174,20 +2140,10 @@ public class Logger implements EventLogInterface {
         sb.append(Tokens.T_SIZE).append(' ').append(propLogSize);
         list.add(sb.toString());
         sb.setLength(0);
-
-        if (propFileTimestamp != 0) {
-            sb.append("SET FILES ").append(Tokens.T_CHECK).append(' ');
-            sb.append(propFileTimestamp);
-            list.add(sb.toString());
-            sb.setLength(0);
-        }
-
-        if (propFileSpaceValue != 0) {
-            sb.append("SET FILES ").append(Tokens.T_SPACE).append(' ');
-            sb.append(propFileSpaceValue);
-            list.add(sb.toString());
-            sb.setLength(0);
-        }
+        sb.append("SET FILES ").append(Tokens.T_CHECK).append(' ');
+        sb.append(propFilesTimestamp);
+        list.add(sb.toString());
+        sb.setLength(0);
 
         String[] array = new String[list.size()];
 
@@ -2353,24 +2309,16 @@ public class Logger implements EventLogInterface {
                         RAShadowFile shadowFile =
                             dataFileCache.getShadowFile();
 
-                        if (shadowFile == null) {
+                        file = new File(dataFileCache.dataFileName);
+                        isw = new InputStreamWrapper(
+                            new FileInputStream(file));
 
-                            // non-incremental backup - ignore .data file
-                            backup.setFileIgnore(dataFileExtension);
-                        } else {
-                            file = new File(dataFileCache.dataFileName);
-                            isw = new InputStreamWrapper(
-                                new FileInputStream(file));
+                        isw.setSizeLimit(dataFileCache.fileStartFreePosition);
+                        backup.setStream(dataFileExtension, isw);
 
-                            isw.setSizeLimit(
-                                dataFileCache.fileStartFreePosition);
-                            backup.setStream(dataFileExtension, isw);
+                        InputStreamInterface isi = shadowFile.getInputStream();
 
-                            InputStreamInterface isi =
-                                shadowFile.getInputStream();
-
-                            backup.setStream(backupFileExtension, isi);
-                        }
+                        backup.setStream(backupFileExtension, isi);
                     }
 
                     // log
