@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,19 +49,21 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.index.Index;
 import org.hsqldb.index.IndexAVL;
+import org.hsqldb.index.IndexAVLCheck.IndexAVLProbe;
+import org.hsqldb.index.IndexStats;
 import org.hsqldb.index.NodeAVL;
 import org.hsqldb.lib.ArrayUtil;
+import org.hsqldb.lib.LongKeyHashMap;
 import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.rowio.RowInputInterface;
 import org.hsqldb.types.LobData;
 import org.hsqldb.types.Type;
-import org.hsqldb.lib.LongKeyHashMap;
 
 /*
  * Base implementation of PersistentStore for different table types.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public abstract class RowStoreAVL implements PersistentStore {
@@ -115,8 +117,6 @@ public abstract class RowStoreAVL implements PersistentStore {
 
     public void setMemory(boolean mode) {}
 
-    public abstract void set(CachedObject object);
-
     public abstract CachedObject get(long key, boolean keep);
 
     public abstract CachedObject get(CachedObject object, boolean keep);
@@ -480,7 +480,9 @@ public abstract class RowStoreAVL implements PersistentStore {
             readLock();
 
             try {
-                elementCount.set(index.getNodeCount(null, this));
+                long count = index.size(null, this);
+
+                elementCount.set(count);
             } finally {
                 readUnlock();
             }
@@ -506,7 +508,7 @@ public abstract class RowStoreAVL implements PersistentStore {
                     readLock();
 
                     try {
-                        return index.getNodeCount(session, this);
+                        return index.size(session, this);
                     } finally {
                         readUnlock();
                     }
@@ -529,8 +531,6 @@ public abstract class RowStoreAVL implements PersistentStore {
         return false;
     }
 
-    public void moveDataToSpace(Session session) {}
-
     /**
      * Moves the data from an old store to new after changes to table
      * The colIndex argument is the indexes of the columns that were
@@ -633,14 +633,22 @@ public abstract class RowStoreAVL implements PersistentStore {
         }
     }
 
-    public void reindex(Session session, Index index) {
+    public void reindex(Session session, Index index, Index useIndex) {
 
         writeLock();
 
         try {
-            setAccessor(index, null);
 
-            RowIterator it = table.rowIterator(this);
+            // get the iterator first in case the index set null
+            RowIterator it;
+
+            if (useIndex == null) {
+                it = table.rowIterator(this);
+            } else {
+                it = useIndex.firstRow(this);
+            }
+
+            setAccessor(index, null);
 
             while (it.next()) {
                 RowAVL row = (RowAVL) it.getCurrentRow();
@@ -653,6 +661,24 @@ public abstract class RowStoreAVL implements PersistentStore {
         }
     }
 
+    public IndexStats[] checkIndexes(Session session, int mode) {
+
+        IndexStats[] indexStats = new IndexStats[accessorList.length];
+
+        for (int i = 0; i < indexList.length; i++) {
+            IndexAVL index = (IndexAVL) indexList[i];
+            NodeAVL  node  = (NodeAVL) accessorList[i];
+            IndexAVLProbe probe = new IndexAVLProbe(session, this, index,
+                node);
+
+            probe.probe();
+
+            indexStats[i] = probe.getStats();
+        }
+
+        return indexStats;
+    }
+
     public void setReadOnly(boolean readOnly) {}
 
     public void readLock() {}
@@ -722,12 +748,19 @@ public abstract class RowStoreAVL implements PersistentStore {
             while (it.next()) {
                 Row     row      = it.getCurrentRow();
                 NodeAVL backnode = ((RowAVL) row).getNode(0);
-                int     j        = position;
+                int     j        = 0;
 
-                while (--j > 0) {
+                while (backnode != null) {
                     backnode = backnode.nNext;
+
+                    j++;
                 }
 
+                if (j < indexList.length) {
+                    break;
+                }
+
+                backnode = ((RowAVL) row).getNode(position - 1);
                 backnode.nNext = backnode.nNext.nNext;
             }
 
@@ -742,7 +775,7 @@ public abstract class RowStoreAVL implements PersistentStore {
     /**
      * Used with memory indexes
      */
-    void destroy() {
+    void destroyIndexes() {
 
         if (indexList.length == 0) {
             return;
