@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,9 @@
 package org.hsqldb.persist;
 
 import java.io.IOException;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
-import org.hsqldb.HsqlException;
 import org.hsqldb.Row;
 import org.hsqldb.RowAVL;
 import org.hsqldb.RowAVLDisk;
@@ -50,6 +51,7 @@ import org.hsqldb.index.NodeAVL;
 import org.hsqldb.index.NodeAVLDisk;
 import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.DoubleIntIndex;
+import org.hsqldb.lib.DoubleLongIndex;
 import org.hsqldb.lib.LongKeyHashMap;
 import org.hsqldb.lib.LongLookup;
 import org.hsqldb.navigator.RowIterator;
@@ -60,7 +62,7 @@ import org.hsqldb.rowio.RowOutputInterface;
  * Implementation of PersistentStore for CACHED tables.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class RowStoreAVLDisk extends RowStoreAVL {
@@ -71,49 +73,39 @@ public class RowStoreAVLDisk extends RowStoreAVL {
 
     public RowStoreAVLDisk(DataFileCache cache, Table table) {
 
-        this(table);
-
-        this.cache = cache;
-        rowOut     = cache.rowOut.duplicate();
+        this.table        = table;
+        this.database     = table.database;
+        this.indexList    = table.getIndexList();
+        this.accessorList = new CachedObject[indexList.length];
+        this.cache        = cache;
+        rowOut            = cache.rowOut.duplicate();
+        largeData         = database.logger.getDataFileFactor() > 1;
 
         cache.adjustStoreCount(1);
 
         rowActionMap = new LongKeyHashMap(8);
         largeData    = database.logger.propLargeData;
         tableSpace   = cache.spaceManager.getTableSpace(table.getSpaceID());
-        readLock     = rowActionMap.getReadLock();
-        writeLock    = rowActionMap.getWriteLock();
-    }
-
-    protected RowStoreAVLDisk(Table table) {
-
-        this.database     = table.database;
-        this.table        = table;
-        this.indexList    = table.getIndexList();
-        this.accessorList = new CachedObject[indexList.length];
-        largeData         = database.logger.getDataFileFactor() > 1;
+        lock         = new ReentrantReadWriteLock(true);
+        readLock     = lock.readLock();
+        writeLock    = lock.writeLock();
     }
 
     public boolean isMemory() {
         return false;
     }
 
-    public void set(CachedObject object) {
+    private void set(CachedObject object) {
 
         if (database.txManager.isMVRows()) {
-            RowAction rowact = (RowAction) rowActionMap.get(object.getPos());
+            RowAction action = (RowAction) rowActionMap.get(object.getPos());
 
-            ((Row) object).rowAction = rowact;
+            if (action != null) {
+                ((Row) object).rowAction = action;
+            }
         }
     }
 
-    public CachedObject get(long key) {
-
-        CachedObject object = cache.get(key, this, false);
-
-        return object;
-    }
-
     public CachedObject get(long key, boolean keep) {
 
         CachedObject object = cache.get(key, this, keep);
@@ -137,15 +129,16 @@ public class RowStoreAVLDisk extends RowStoreAVL {
 
         object.setStorageSize(size);
 
-        long pos = tableSpace.getFilePosition(size, false);
+        long pos = tableSpace.getFilePosition(size);
 
         object.setPos(pos);
 
         if (tx) {
-            RowAction.addInsertAction(session, table, (Row) object);
+            RowAction action = RowAction.addInsertAction(session, table,
+                (Row) object);
 
             if (database.txManager.isMVRows()) {
-                rowActionMap.put(object.getPos(), ((Row) object).rowAction);
+                rowActionMap.put(object.getPos(), action);
             }
         }
 
@@ -184,11 +177,17 @@ public class RowStoreAVLDisk extends RowStoreAVL {
     public CachedObject get(RowInputInterface in) {
 
         try {
+            Row row;
+
             if (largeData) {
-                return new RowAVLDiskLarge(this, in);
+                row = new RowAVLDiskLarge(this, in);
             } else {
-                return new RowAVLDisk(this, in);
+                row = new RowAVLDisk(this, in);
             }
+
+            set(row);
+
+            return row;
         } catch (IOException e) {
             throw Error.error(ErrorCode.DATA_FILE_ERROR, e);
         }
@@ -210,9 +209,18 @@ public class RowStoreAVLDisk extends RowStoreAVL {
         return row;
     }
 
-    public void indexRow(Session session, Row row) {
+    public void delete(Session session, Row row) {
 
-        writeLock();
+        try {
+            row = (Row) get(row, true);
+
+            super.delete(session, row);
+        } finally {
+            row.keepInMemory(false);
+        }
+    }
+
+    public void indexRow(Session session, Row row) {
 
         try {
             row = (Row) get(row, true);
@@ -220,7 +228,6 @@ public class RowStoreAVLDisk extends RowStoreAVL {
             super.indexRow(session, row);
         } finally {
             row.keepInMemory(false);
-            writeUnlock();
         }
     }
 
@@ -244,7 +251,9 @@ public class RowStoreAVLDisk extends RowStoreAVL {
     public void postCommitAction(Session session, RowAction action) {
 
         if (action.getType() == RowAction.ACTION_NONE) {
-            writeLock();
+            Lock mapLock = rowActionMap.getWriteLock();
+
+            mapLock.lock();
 
             try {
 
@@ -253,7 +262,7 @@ public class RowStoreAVLDisk extends RowStoreAVL {
                     rowActionMap.remove(action.getPos());
                 }
             } finally {
-                writeUnlock();
+                mapLock.unlock();
             }
         } else if (action.getType() == RowAction.ACTION_DELETE_FINAL
                    && !action.isDeleteComplete()) {
@@ -261,10 +270,6 @@ public class RowStoreAVLDisk extends RowStoreAVL {
 
             Row row = action.getRow();
 
-            if (row == null) {
-                row = (Row) get(action.getPos(), false);
-            }
-
             delete(session, row);
 
             // remove info after delete but before removing persistence
@@ -374,25 +379,21 @@ public class RowStoreAVLDisk extends RowStoreAVL {
             }
         }
 
-        writeLock();
+        row = (Row) cache.get(row, this, true);
+
+        Lock mapLock = rowActionMap.getWriteLock();
+
+        mapLock.lock();
 
         try {
-            action = (RowAction) rowActionMap.get(row.getPos());
+            action = RowAction.addDeleteAction(session, table, row, colMap);
 
-            if (action == null) {
-                action = RowAction.addDeleteAction(session, table, row,
-                                                   colMap);
-
-                if (action != null) {
-                    rowActionMap.put(action.getPos(), action);
-                }
-            } else {
-                row.rowAction = action;
-                action = RowAction.addDeleteAction(session, table, row,
-                                                   colMap);
+            if (action != null) {
+                rowActionMap.put(action.getPos(), action);
             }
         } finally {
-            writeUnlock();
+            mapLock.unlock();
+            row.keepInMemory(false);
         }
 
         return action;
@@ -424,7 +425,7 @@ public class RowStoreAVLDisk extends RowStoreAVL {
         int position = key.getPosition();
 
         if (position >= accessorList.length) {
-            throw Error.runtimeError(ErrorCode.U_S0500, "RowStoreAVL");
+            throw Error.runtimeError(ErrorCode.U_S0500, "RowStoreAVLDisk");
         }
 
         NodeAVL node = (NodeAVL) accessorList[position];
@@ -468,6 +469,19 @@ public class RowStoreAVLDisk extends RowStoreAVL {
 
     public void setReadOnly(boolean readOnly) {}
 
+    public void setSpaceManager(Session session) {
+
+        if (table.getSpaceID() == DataSpaceManager.tableIdDefault) {
+            return;
+        }
+
+        TableSpaceManager tableSpace =
+            cache.spaceManager.getTableSpace(table.getSpaceID());
+
+        setSpaceManager(tableSpace);
+        moveDataToSpace(session);
+    }
+
     public void moveDataToSpace(Session session) {
 
         Table table    = (Table) this.table;
@@ -483,33 +497,27 @@ public class RowStoreAVLDisk extends RowStoreAVL {
             return;
         }
 
-        DoubleIntIndex pointerLookup = new DoubleIntIndex((int) rowCount);
-
         writeLock();
 
         try {
-            moveDataToSpace(cache, pointerLookup);
+            LongLookup pointerLookup = getPointerList();
+            LongLookup removeList    = pointerLookup.duplicate();
+
+            moveDataToNewSpace(cache, pointerLookup);
 
             CachedObject[] newAccessorList =
                 new CachedObject[accessorList.length];
 
             for (int i = 0; i < accessorList.length; i++) {
-                long pos = pointerLookup.lookup(accessorList[i].getPos());
+                Index   key  = indexList[i];
+                long    pos  = pointerLookup.lookup(accessorList[i].getPos());
+                RowAVL  row  = (RowAVL) cache.get(pos, this, false);
+                NodeAVL node = row.getNode(key.getPosition());
 
-                newAccessorList[i] = cache.get(pos, this, false);
+                newAccessorList[key.getPosition()] = node;
             }
 
-            // using the old index
-            RowIterator it = rowIterator();
-
-            // todo - check this - must remove from old space, not new one
-            // it works but the rows are removed from new space manager
-            while (it.next()) {
-                Row row = it.getCurrentRow();
-
-                cache.remove(row);
-                tableSpace.release(row.getPos(), row.getStorageSize());
-            }
+            removeDefaultSpaces(removeList);
 
             accessorList = newAccessorList;
         } finally {
@@ -522,10 +530,22 @@ public class RowStoreAVLDisk extends RowStoreAVL {
 
     public void moveDataToSpace(DataFileCache targetCache,
                                 LongLookup pointerLookup) {
+        populatePointerList(pointerLookup);
+        moveDataToNewSpace(targetCache, pointerLookup);
+    }
+
+    private DoubleLongIndex getPointerList() {
+
+        DoubleLongIndex pointerLookup =
+            new DoubleLongIndex((int) elementCount());
+
+        populatePointerList(pointerLookup);
+
+        return pointerLookup;
+    }
+
+    private void populatePointerList(LongLookup pointerLookup) {
 
-        int spaceId = table.getSpaceID();
-        TableSpaceManager targetSpace =
-            targetCache.spaceManager.getTableSpace(spaceId);
         RowIterator it = indexList[0].firstRow(this);
 
         while (it.next()) {
@@ -535,15 +555,23 @@ public class RowStoreAVLDisk extends RowStoreAVL {
         }
 
         pointerLookup.sort();
+    }
+
+    private void moveDataToNewSpace(DataFileCache targetCache,
+                                    LongLookup pointerLookup) {
+
+        int spaceId = table.getSpaceID();
+        TableSpaceManager targetSpace =
+            targetCache.spaceManager.getTableSpace(spaceId);
 
         for (int i = 0; i < pointerLookup.size(); i++) {
             long newPos = targetSpace.getFilePosition(
-                (int) pointerLookup.getLongValue(i), false);
+                (int) pointerLookup.getLongValue(i));
 
             pointerLookup.setLongValue(i, newPos);
         }
 
-        it = indexList[0].firstRow(this);
+        RowIterator it = indexList[0].firstRow(this);
 
         while (it.next()) {
             CachedObject row    = it.getCurrentRow();
@@ -556,6 +584,24 @@ public class RowStoreAVLDisk extends RowStoreAVL {
         }
     }
 
+    private void removeDefaultSpaces(LongLookup removeList) {
+
+        DataSpaceManager manager = cache.spaceManager;
+        int              scale   = cache.getDataFileScale();
+
+        for (int i = 0; i < removeList.size(); i++) {
+            long pos  = removeList.getLongKey(i);
+            long size = removeList.getLongValue(i) / scale;
+
+            removeList.setLongValue(i, size);
+            cache.release(pos);
+        }
+
+        removeList.compactLookupAsIntervals();
+        manager.freeTableSpace(DataSpaceManager.tableIdDefault, removeList, 0,
+                               0);
+    }
+
     long getStorageSizeEstimate() {
 
         if (elementCount.get() == 0) {
@@ -563,7 +609,7 @@ public class RowStoreAVLDisk extends RowStoreAVL {
         }
 
         CachedObject accessor = getAccessor(indexList[0]);
-        CachedObject row      = get(accessor.getPos());
+        CachedObject row      = get(accessor.getPos(), false);
 
         return row.getStorageSize() * elementCount.get();
     }
