@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,7 @@ import org.hsqldb.rowio.RowInputInterface;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 2.3.0
  */
 public abstract class SimpleStore implements PersistentStore {
@@ -56,12 +56,6 @@ public abstract class SimpleStore implements PersistentStore {
         return false;
     }
 
-    public void set(CachedObject object) {}
-
-    public CachedObject get(long i) {
-        return cache.get(i, this, false);
-    }
-
     public CachedObject get(CachedObject object, boolean keep) {
         return cache.get(object, this, keep);
     }
@@ -116,7 +110,7 @@ public abstract class SimpleStore implements PersistentStore {
         return cache == null;
     }
 
-    public void reindex(Session session, Index index) {}
+    public void reindex(Session session, Index index, Index useIndex) {}
 
     public void setCache(DataFileCache cache) {
         this.cache = cache;
@@ -186,8 +180,6 @@ public abstract class SimpleStore implements PersistentStore {
         return null;
     }
 
-    public void moveDataToSpace(Session session) {}
-
     public void moveData(Session session, PersistentStore other,
                          int[] colIndex, int adjust) {}
 
