@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,13 +31,13 @@
 
 package org.hsqldb.persist;
 
-import org.hsqldb.lib.DoubleIntIndex;
+import org.hsqldb.lib.LongLookup;
 
 /**
  * Manages allocation of space for rows.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.0
+ * @version 2.5.1
  * @since 2.3.0
  */
 public interface TableSpaceManager {
@@ -52,13 +52,13 @@ public interface TableSpaceManager {
     /**
      * Allocates a block
      */
-    long getFilePosition(int rowSize, boolean asBlocks);
+    long getFilePosition(int rowSize);
 
     boolean hasFileRoom(long blockSize);
 
     void addFileBlock(long blockFreePos, long blockLimit);
 
-    void initialiseFileBlock(DoubleIntIndex lookup, long blockFreePos,
+    void initialiseFileBlock(LongLookup lookup, long blockFreePos,
                              long blockLimit);
 
     void reset();
