@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,11 +33,11 @@ package org.hsqldb.persist;
 
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
-import org.hsqldb.lib.DoubleIntIndex;
+import org.hsqldb.lib.LongLookup;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.0
+ * @version 2.5.1
  * @since 2.3.0
  */
 public class TableSpaceManagerSimple implements TableSpaceManager {
@@ -57,18 +57,19 @@ public class TableSpaceManagerSimple implements TableSpaceManager {
     public void release(long pos, int rowSize) {}
 
     /**
-     * Returns the position of a free block or 0.
+     * Returns the position of a free block or throws.
      */
-    public long getFilePosition(int rowSize, boolean asBlocks) {
+    public long getFilePosition(int rowSize) {
 
         cache.writeLock.lock();
 
         try {
+            long filePosition = cache.getFileFreePos();
             long position;
             long newFreePosition;
 
-            position        = cache.getFileFreePos() / scale;
-            newFreePosition = cache.getFileFreePos() + rowSize;
+            position        = filePosition / scale;
+            newFreePosition = filePosition + rowSize;
 
             if (newFreePosition > cache.maxDataFileSize) {
                 cache.logSevereEvent("data file reached maximum size "
@@ -91,7 +92,7 @@ public class TableSpaceManagerSimple implements TableSpaceManager {
 
     public void addFileBlock(long blockFreePos, long blockLimit) {}
 
-    public void initialiseFileBlock(DoubleIntIndex lookup, long blockFreePos,
+    public void initialiseFileBlock(LongLookup lookup, long blockFreePos,
                                     long blockLimit) {}
 
     public void reset() {}
