@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@ import org.hsqldb.types.Type;
  *
  * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class Result {
@@ -78,6 +78,8 @@ public class Result {
     public static final ResultMetaData sessionAttributesMetaData;
 
     static {
+
+        // calls to initialise some global statics
         SqlInvariants.isSystemSchemaName(SqlInvariants.SYSTEM_SCHEMA);
         Charset.getDefaultInstance();
         Collation.getDefaultInstance();
@@ -243,13 +245,16 @@ public class Result {
         return result;
     }
 
-    public static Result newResult(DataInput dataInput,
+    public static Result newResult(DataInputStream dataInput,
                                    RowInputInterface in) throws IOException {
-        return newResult(null, dataInput.readByte(), dataInput, in);
+
+        int readMode = dataInput.readByte();
+
+        return newResult(null, readMode, dataInput, in);
     }
 
     public static Result newResult(Session session, int mode,
-                                   DataInput dataInput,
+                                   DataInputStream dataInput,
                                    RowInputInterface in) throws IOException {
 
         try {
@@ -287,12 +292,11 @@ public class Result {
         }
     }
 
-    public void readLobResults(SessionInterface session,
-                               DataInputStream inputStream,
-                               RowInputInterface in) throws IOException {
+    public Result readLobResults(Session session,
+                                 DataInputStream inputStream)
+                                 throws IOException {
 
-        Result  currentResult = this;
-        boolean hasLob        = false;
+        boolean hasLob = false;
 
         setSession(session);
 
@@ -300,12 +304,11 @@ public class Result {
             int addedResultMode = inputStream.readByte();
 
             if (addedResultMode == ResultConstants.LARGE_OBJECT_OP) {
-                ResultLob resultLob = ResultLob.newLob(inputStream, false);
+                ResultLob resultLob    = ResultLob.newLob(inputStream, false);
+                Result    actionResult = session.allocateResultLob(resultLob);
 
-                if (session instanceof Session) {
-                    session.allocateResultLob(resultLob, inputStream);
-                } else {
-                    currentResult.addLobResult(resultLob);
+                if (actionResult.isError()) {
+                    return actionResult;
                 }
 
                 hasLob = true;
@@ -319,8 +322,10 @@ public class Result {
         }
 
         if (hasLob) {
-            ((Session) session).registerResultLobs(currentResult);
+            session.registerResultLobs(this);
         }
+
+        return Result.updateZeroResult;
     }
 
     private static Result newResult(Session session, DataInput dataInput,
@@ -1072,6 +1077,12 @@ public class Result {
             result.mainString = result.exception.getMessage();
             result.subString  = result.exception.getSQLState();
             result.errorCode  = result.exception.getErrorCode();
+        } else if (t instanceof IOException) {
+            result.exception = Error.error(ErrorCode.GENERAL_IO_ERROR);
+            result.mainString = result.exception.getMessage() + ' '
+                                + t.getMessage();
+            result.subString = result.exception.getSQLState();
+            result.errorCode = result.exception.getErrorCode();
         } else {
             result.exception  = Error.error(ErrorCode.GENERAL_ERROR);
             result.mainString = result.exception.getMessage();
@@ -1352,6 +1363,11 @@ public class Result {
     }
 
     public HsqlException getException() {
+
+        if (exception == null && mode == ResultConstants.ERROR) {
+            exception = Error.error(this);
+        }
+
         return exception;
     }
 
