@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,6 @@
 
 package org.hsqldb.rights;
 
-import org.hsqldb.Expression;
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.NumberSequence;
 import org.hsqldb.Routine;
@@ -73,7 +72,7 @@ import org.hsqldb.types.Type;
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
  *
- * @version 2.3.4
+ * @version 2.5.1
  * @since 1.8.0
  */
 public class Grantee implements SchemaObject {
@@ -224,7 +223,7 @@ public class Grantee implements SchemaObject {
         return set;
     }
 
-    public boolean isAccessible(HsqlName name, int privilegeType) {
+    public boolean isAccessible(HsqlName name, int action) {
 
         if (isFullyAccessibleByRole(name)) {
             return true;
@@ -236,7 +235,7 @@ public class Grantee implements SchemaObject {
             return false;
         }
 
-        return right.canAccess(privilegeType);
+        return right.canAccess(action);
     }
 
     /**
@@ -578,105 +577,105 @@ public class Grantee implements SchemaObject {
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkInsert(SchemaObject object, boolean[] checkList) {
+    public Right checkInsert(SchemaObject object, boolean[] checkList) {
 
         if (object instanceof Table) {
             Table table = (Table) object;
 
             if (isFullyAccessibleByRole(table.getName())) {
-                return;
+                return Right.fullRights;
             }
 
             Right right = (Right) fullRightsMap.get(table.getName());
 
             if (right != null && right.canInsert(table, checkList)) {
-                return;
+                return right;
             }
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkUpdate(SchemaObject object, boolean[] checkList) {
+    public Right checkUpdate(SchemaObject object, boolean[] checkList) {
 
         if (object instanceof Table) {
             Table table = (Table) object;
 
             if (isFullyAccessibleByRole(table.getName())) {
-                return;
+                return Right.fullRights;
             }
 
             Right right = (Right) fullRightsMap.get(table.getName());
 
             if (right != null && right.canUpdate(table, checkList)) {
-                return;
+                return right;
             }
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkReferences(SchemaObject object, boolean[] checkList) {
+    public Right checkReferences(SchemaObject object, boolean[] checkList) {
 
         if (object instanceof Table) {
             Table table = (Table) object;
 
             if (isFullyAccessibleByRole(table.getName())) {
-                return;
+                return Right.fullRights;
             }
 
             Right right = (Right) fullRightsMap.get(table.getName());
 
             if (right != null && right.canReference(table, checkList)) {
-                return;
+                return right;
             }
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkTrigger(SchemaObject object, boolean[] checkList) {
+    public Right checkTrigger(SchemaObject object, boolean[] checkList) {
 
         if (object instanceof Table) {
             Table table = (Table) object;
 
             if (isFullyAccessibleByRole(table.getName())) {
-                return;
+                return Right.fullRights;
             }
 
             Right right = (Right) fullRightsMap.get(table.getName());
 
             if (right != null && right.canReference(table, checkList)) {
-                return;
+                return right;
             }
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkDelete(SchemaObject object) {
+    public Right checkDelete(SchemaObject object) {
 
         if (object instanceof Table) {
             Table table = (Table) object;
 
             if (isFullyAccessibleByRole(table.getName())) {
-                return;
+                return Right.fullRights;
             }
 
             Right right = (Right) fullRightsMap.get(table.getName());
 
             if (right != null && right.canDelete()) {
-                return;
+                return right;
             }
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkAccess(SchemaObject object) {
+    public Right checkAccess(SchemaObject object) {
 
         if (isFullyAccessibleByRole(object.getName())) {
-            return;
+            return Right.fullRights;
         }
 
         HsqlName name = object.getName();
@@ -688,7 +687,7 @@ public class Grantee implements SchemaObject {
         Right right = (Right) fullRightsMap.get(name);
 
         if (right != null && !right.isEmpty()) {
-            return;
+            return right;
         }
 
         throw Error.error(ErrorCode.X_42501, object.getName().name);
@@ -1157,16 +1156,6 @@ public class Grantee implements SchemaObject {
                             sb.append(Tokens.T_TABLE).append(' ');
                             sb.append(
                                 hsqlname.getSchemaQualifiedStatementName());
-
-                            Expression expr = right.getFilterExpression();
-
-                            if (expr != null) {
-                                sb.append(' ').append(Tokens.T_FILTER);
-                                sb.append(Tokens.T_OPENBRACKET);
-                                sb.append(Tokens.T_WHERE).append(' ');
-                                sb.append(expr.getSQL());
-                                sb.append(Tokens.T_CLOSEBRACKET);
-                            }
                         }
                         break;
 
