@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,6 +61,8 @@ import org.hsqldb.resources.ResourceBundleHandler;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
 
+import java.util.Properties;
+
 // fredt@users 20020215 - patch 1.7.0
 // methods reorganised to use new HsqlProperties class
 // fredt@users 20020424 - patch 1.7.0 - shutdown without exit
@@ -232,7 +234,7 @@ import org.hsqldb.result.ResultConstants;
  * is started as part of a larger framework. <p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.7.2
  */
 public class Server implements HsqlSocketRequestHandler, Notified {
@@ -1049,6 +1051,23 @@ public class Server implements HsqlSocketRequestHandler, Notified {
      * @throws IOException
      *          ACL list was requested but I/O problem loading ACL.
      */
+    public void setProperties(Properties props)
+    throws IOException, ServerAcl.AclFormatException {
+
+        HsqlProperties serverProps = new HsqlProperties(props);
+
+        setProperties(serverProps);
+    }
+
+    /**
+     * Sets server properties using the specified HsqlProperties object
+     *
+     * @param props The HsqlProperties object containing properties to set
+     * @throws ServerAcl.AclFormatException
+     *          ACL list was requested but problem loading ACL.
+     * @throws IOException
+     *          ACL list was requested but I/O problem loading ACL.
+     */
     public void setProperties(HsqlProperties props)
     throws IOException, ServerAcl.AclFormatException {
 
@@ -1402,7 +1421,7 @@ public class Server implements HsqlSocketRequestHandler, Notified {
      * @param cid the connection identifier
      * @param r the request whose description is to be printed
      */
-    final void printRequest(int cid, Result r) {
+    final void printRequest(long cid, Result r) {
 
         if (isSilent()) {
             return;
@@ -2161,32 +2180,38 @@ public class Server implements HsqlSocketRequestHandler, Notified {
 
         // Be nice and let applications exit if there are no
         // running connection threads - wait at most 100 ms per active thread
-        for (int count = serverConnectionThreadGroup.activeCount();
-                count > 0 && serverConnectionThreadGroup.activeCount() > 0;
-                count--) {
-            try {
-                Thread.sleep(100);
-            } catch (Exception e) {
+        if (serverConnectionThreadGroup != null) {
+            if (!serverConnectionThreadGroup.isDestroyed()) {
+                int count = serverConnectionThreadGroup.activeCount();
 
-                // e.getMessage();
+                for (; count > 0
+                        && serverConnectionThreadGroup.activeCount() > 0;
+                        count--) {
+                    try {
+                        Thread.sleep(100);
+                    } catch (Exception e) {
+
+                        // e.getMessage();
+                    }
+                }
+
+                try {
+                    serverConnectionThreadGroup.destroy();
+                    printWithThread(serverConnectionThreadGroup.getName()
+                                    + " destroyed");
+                } catch (Throwable t) {
+                    printWithThread(serverConnectionThreadGroup.getName()
+                                    + " not destroyed");
+                    printWithThread(t.toString());
+                }
             }
+
+            serverConnectionThreadGroup = null;
         }
 
-        try {
-            serverConnectionThreadGroup.destroy();
-            printWithThread(serverConnectionThreadGroup.getName()
-                            + " destroyed");
-        } catch (Throwable t) {
-            printWithThread(serverConnectionThreadGroup.getName()
-                            + " not destroyed");
-            printWithThread(t.toString());
-        }
+        serverThread = null;
 
         setState(ServerConstants.SERVER_STATE_SHUTDOWN);
-
-        serverConnectionThreadGroup = null;
-        serverThread                = null;
-
         print(sw.elapsedTimeToMessage("Shutdown sequence completed"));
 
         if (isNoSystemExit()) {
@@ -2356,5 +2381,4 @@ public class Server implements HsqlSocketRequestHandler, Notified {
             DriverManager.setLogWriter(newPrintWriter);
         } catch (Exception e) {}
     }
-
 }
