@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2019, The HSQL Development Group
+ * Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,7 +77,7 @@ import java.io.EOFException;
 import java.io.IOException;
 import java.net.Socket;
 import java.net.SocketException;
-import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicLong;
 
 import org.hsqldb.ClientConnection;
 import org.hsqldb.ColumnBase;
@@ -118,9 +118,11 @@ import org.hsqldb.types.Type;
  *
  * ODBC support added for version 2.0.0 by Blaine Simpson.<p>
  *
+ * ODBC support updated for version 2.5.x. (fredt@users)<p>
+ *
  * @author Blaine Simpson (unsaved@users dot sourceforge.net
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.1
  * @since 1.6.2
  */
 class ServerConnection implements Runnable {
@@ -134,15 +136,16 @@ class ServerConnection implements Runnable {
     private Server           server;
     private DataInputStream  dataInput;
     private DataOutputStream dataOutput;
-    private int              mThread;
+    private long             mThread;
     static final int         BUFFER_SIZE = 0x1000;
     final byte[]             mainBuffer  = new byte[BUFFER_SIZE];
     RowOutputInterface       rowOut;
     RowInputBinary           rowIn;
     Thread                   runnerThread;
+    InResultProcessor        processor;
 
     //
-    private static AtomicInteger mCurrentThread = new AtomicInteger(0);
+    private static AtomicLong mCurrentThread = new AtomicLong(0);
 
     //
     protected static String TEXTBANNER_PART1 = null;
@@ -191,11 +194,9 @@ class ServerConnection implements Runnable {
         rowOut = rowOutTemp;
 
         //
-        Thread runnerThread;
-
-        this.socket = socket;
-        this.server = server;
-        mThread     = mCurrentThread.getAndIncrement();
+        this.socket  = socket;
+        this.server  = server;
+        this.mThread = mCurrentThread.getAndIncrement();
 
         synchronized (server.serverConnSet) {
             server.serverConnSet.add(this);
@@ -277,30 +278,13 @@ class ServerConnection implements Runnable {
             switch (streamProtocol) {
 
                 case HSQL_STREAM_PROTOCOL :
-                    if (firstInt
-                            != ClientConnection
-                                .NETWORK_COMPATIBILITY_VERSION_INT) {
-                        if (firstInt == -1900000) {
-                            firstInt = -2000000;
-                        }
-
-                        String verString =
-                            ClientConnection.toNetCompVersionString(firstInt);
-
-                        throw Error.error(
-                            null, ErrorCode.SERVER_VERSIONS_INCOMPATIBLE, 0,
-                            new String[] {
-                            HsqlDatabaseProperties.THIS_VERSION, verString
-                        });
-                    }
-
                     int msgType = dataInput.readByte();
 
-                    receiveResult(msgType);
+                    processor.receiveConnection(msgType);
                     break;
 
                 case ODBC_STREAM_PROTOCOL :
-                    odbcConnect(firstInt);
+                    processor.receiveConnection(firstInt);
                     break;
 
                 default :
@@ -315,7 +299,7 @@ class ServerConnection implements Runnable {
             // Expected failures will have been handled (by sending feedback
             // to user-- with an output Result for normal protocols), then
             // continuing.
-            StringBuilder sb = new StringBuilder(mThread);
+            StringBuilder sb = new StringBuilder(String.valueOf(mThread));
 
             sb.append(":Failed to connect client.");
 
@@ -329,8 +313,6 @@ class ServerConnection implements Runnable {
         }
     }
 
-    private static class CleanExit extends Exception {}
-
     private static class ClientFailure extends Exception {
 
         private String clientMessage = null;
@@ -355,7 +337,7 @@ class ServerConnection implements Runnable {
         Result resultIn = Result.newResult(session, resultMode, dataInput,
                                            rowIn);
 
-        resultIn.readLobResults(session, dataInput, rowIn);
+        resultIn.readLobResults(session, dataInput);
         server.printRequest(mThread, resultIn);
 
         Result resultOut = null;
@@ -629,6 +611,11 @@ class ServerConnection implements Runnable {
                             interposedStatement = suffix;
                             sql                 = sql.substring(0, lastSemi);
                         }
+
+                        if (suffix.startsWith("show transaction_isolation")) {
+                            interposedStatement = null;
+                            sql = "values session_isolation_level()";
+                        }
                     }
 
                     /** ******************************************* */
@@ -655,14 +642,19 @@ class ServerConnection implements Runnable {
                      * SIMPLE mode), so we are more capable than our client is.
                      */
                     if (normalized.startsWith("select current_schema()")) {
-                        server.printWithThread(
-                            "Implement 'select current_schema() emulation!");
 
-                        throw new RecoverableOdbcFailure(
-                            "current_schema() not supported yet", "0A000");
+                        //ok in PGS compatibility mode
+                        // but transalate for other modes
+                        sql        = "values current_schema()";
+                        normalized = sql;
                     }
 
-                    if (normalized.startsWith("select n.nspname,")) {
+                    if (normalized.startsWith("show ")) {
+                        sql = "values " + sql.substring("show ".length());
+                        normalized = sql;
+                    }
+
+                    if (normalized.startsWith("values n.nspname,")) {
 
                         // Executed by psqlodbc after every user-specified query.
                         server.printWithThread(
@@ -722,7 +714,8 @@ class ServerConnection implements Runnable {
                         break;
                     }
 
-                    if (normalized.startsWith("select ")) {
+                    if (normalized.startsWith("select ")
+                            || normalized.startsWith("values ")) {
                         server.printWithThread(
                             "Performing a real non-prepared SELECT...");
 
@@ -732,6 +725,7 @@ class ServerConnection implements Runnable {
                             sql, 0, 0, StatementTypes.RETURN_RESULT, 0,
                             ResultProperties.defaultPropsValue,
                             java.sql.Statement.NO_GENERATED_KEYS, null, null);
+                        server.printWithThread(sql);
 
                         rOut = session.execute(r);
 
@@ -765,8 +759,7 @@ class ServerConnection implements Runnable {
                         pgTypes  = new PgType[columnCount];
 
                         for (int i = 0; i < pgTypes.length; i++) {
-                            pgTypes[i] = PgType.getPgType(colTypes[i],
-                                                          md.isTableColumn(i));
+                            pgTypes[i] = PgType.getPgType(colTypes[i]);
                         }
 
                         // fredt : colLabels may not contain some column names
@@ -1086,8 +1079,7 @@ class ServerConnection implements Runnable {
 
                         for (int i = 0; i < paramTypes.length; i++) {
                             outPacket.writeInt(
-                                PgType.getPgType(
-                                    paramTypes[i], true).getOid());
+                                PgType.getPgType(paramTypes[i]).getOid());
 
                             // TODO:  Determine whether parameter typing works
                             // better for Strings when try to match table column
@@ -1133,8 +1125,7 @@ class ServerConnection implements Runnable {
                     ColumnBase[] colDefs = md.columns;
 
                     for (int i = 0; i < pgTypes.length; i++) {
-                        pgTypes[i] = PgType.getPgType(colTypes[i],
-                                                      md.isTableColumn(i));
+                        pgTypes[i] = PgType.getPgType(colTypes[i]);
                     }
 
                     if (colNames.length != colDefs.length) {
@@ -1352,9 +1343,7 @@ class ServerConnection implements Runnable {
                         pgTypes  = new PgType[colCount];
 
                         for (int i = 0; i < pgTypes.length; i++) {
-                            pgTypes[i] = PgType.getPgType(
-                                colTypes[i],
-                                portal.ackResult.metaData.isTableColumn(i));
+                            pgTypes[i] = PgType.getPgType(colTypes[i]);
                         }
 
                         for (int i = 0; i < colCount; i++) {
@@ -1382,7 +1371,7 @@ class ServerConnection implements Runnable {
                         outPacket.xmit('D', dataOutput);
                     }
 
-                    if (navigator.afterLast()) {
+                    if (navigator.getSize() == 0 || navigator.afterLast()) {
                         outPacket.write("SELECT");
                         outPacket.xmit('C', dataOutput);
 
@@ -1527,11 +1516,7 @@ class ServerConnection implements Runnable {
                 while (keepAlive) {
                     msgType = dataInput.readByte();
 
-                    if (msgType < ResultConstants.MODE_UPPER_LIMIT) {
-                        receiveResult(msgType);
-                    } else {
-                        receiveOdbcPacket((char) msgType);
-                    }
+                    processor.receiveResult((char) msgType);
                 }
             } catch (CleanExit ce) {
                 keepAlive = false;
@@ -1696,10 +1681,29 @@ class ServerConnection implements Runnable {
                 // size.  The size can never be large enough that the first
                 // byte will be non-zero.
                 streamProtocol = ODBC_STREAM_PROTOCOL;
+                processor      = new OdbcInResultProcessor();
                 break;
 
             default :
                 streamProtocol = HSQL_STREAM_PROTOCOL;
+                processor      = new HsqlInResultProcessor();
+
+                if (firstInt
+                        != ClientConnection
+                            .NETWORK_COMPATIBILITY_VERSION_INT) {
+                    if (firstInt == -1900000) {
+                        firstInt = -2000000;
+                    }
+
+                    String verString =
+                        ClientConnection.toNetCompVersionString(firstInt);
+
+                    throw Error.error(null,
+                                      ErrorCode.SERVER_VERSIONS_INCOMPATIBLE,
+                                      0, new String[] {
+                        HsqlDatabaseProperties.THIS_VERSION, verString
+                    });
+                }
 
             // HSQL protocol client
         }
@@ -1967,6 +1971,7 @@ class ServerConnection implements Runnable {
             statement, 0, 0, StatementTypes.RETURN_COUNT, 0,
             ResultProperties.defaultPropsValue,
             ResultConstants.RETURN_NO_GENERATED_KEYS, null, null);
+        server.printWithThread(statement);
 
         Result rOut = session.execute(r);
 
@@ -2002,4 +2007,37 @@ class ServerConnection implements Runnable {
             }
         }
     }
+
+    class OdbcInResultProcessor implements InResultProcessor {
+
+        public void receiveConnection(int type) throws CleanExit, IOException {
+            ServerConnection.this.odbcConnect(type);
+        }
+
+        public void receiveResult(int type) throws CleanExit, IOException {
+            ServerConnection.this.receiveOdbcPacket((char) type);
+        }
+    }
+
+    class HsqlInResultProcessor implements InResultProcessor {
+
+        public void receiveConnection(int type) throws CleanExit, IOException {
+            receiveResult(type);
+        }
+
+        public void receiveResult(int type) throws CleanExit, IOException {
+            ServerConnection.this.receiveResult(type);
+        }
+    }
 }
+
+
+interface InResultProcessor {
+
+    void receiveConnection(int type) throws CleanExit, IOException;
+
+    void receiveResult(int type) throws CleanExit, IOException;
+}
+
+
+class CleanExit extends Exception {}
