@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ public class TestCascade extends TestCase {
             Class.forName("org.hsqldb.jdbc.JDBCDriver");
             createDatabase();
 
-            con = DriverManager.getConnection("jdbc:hsqldb:testdb", "sa", "");
+            con = DriverManager.getConnection("jdbc:hsqldb:testdb/testdb", "sa", "");
         } catch (Exception e) {
             e.printStackTrace();
             System.out.println(this + ".setUp() error: " + e.getMessage());
@@ -112,12 +112,9 @@ public class TestCascade extends TestCase {
 
     private static void createDatabase() throws SQLException {
 
-        new File("testdb.backup").delete();
-        new File("testdb.data").delete();
-        new File("testdb.properties").delete();
-        new File("testdb.script").delete();
+        TestUtil.deleteDatabase("testdb/testdb");
 
-        Connection con = DriverManager.getConnection("jdbc:hsqldb:testdb",
+        Connection con = DriverManager.getConnection("jdbc:hsqldb:testdb/testdb",
             "sa", "");
         String[] saDDL = {
             "CREATE CACHED TABLE XB (EIACODXA VARCHAR(10) NOT NULL, LSACONXB VARCHAR(18) NOT NULL, ALTLCNXB VARCHAR(2) NOT NULL, LCNTYPXB VARCHAR(1) NOT NULL, LCNINDXB VARCHAR(1), LCNAMEXB VARCHAR(19), UPDT_BY VARCHAR(32), LST_UPDT TIMESTAMP, CONSTRAINT XPKXB PRIMARY KEY (EIACODXA, LSACONXB, ALTLCNXB, LCNTYPXB));",
