@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,9 @@
 package org.hsqldb.test;
 
 import java.sql.Connection;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.sql.Statement;
 
 import org.hsqldb.Scanner;
 import org.hsqldb.Session;
@@ -49,7 +52,7 @@ import junit.framework.Assert;
  * TODO: Add tests with intervals, other date types/time and time zone.
  *
  * @author Pascal-Eric Servais (peservais@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.5.0
  * @since 2.3.4
  */
 public class TestPeriodPredicates extends TestBase {
@@ -57,10 +60,11 @@ public class TestPeriodPredicates extends TestBase {
     private Scanner    scanner;
     private Connection conn;
     private Session    session;
+    private Statement  statement;
 
     public TestPeriodPredicates(String name) {
 
-        super(name,  "jdbc:hsqldb:mem:test", false, false);
+        super(name, "jdbc:hsqldb:mem:test", false, false);
 
         scanner = new Scanner();
     }
@@ -69,8 +73,9 @@ public class TestPeriodPredicates extends TestBase {
 
         super.setUp();
 
-        conn    = super.newConnection();
-        session = (Session) ((JDBCConnection) conn).getSession();
+        conn      = super.newConnection();
+        session   = (Session) ((JDBCConnection) conn).getSession();
+        statement = conn.createStatement();
     }
 
     /**
@@ -98,8 +103,24 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
-        Assert.assertTrue(DateTimeType.overlaps(session, b, tb, a, ta));
+        assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertTrue(DateTimeType.overlaps(session, b, tb, a, ta));
+    }
+
+    public void testSQLFirstPeriodOverlapsSecondPeriod() throws SQLException {
+
+        String sql =
+            "CALL"
+            + "(TIMESTAMP '1999-12-01 01:02:03', TIMESTAMP '2000-01-12 01:02:03')"
+            + "OVERLAPS"
+            + "(TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')";
+        ResultSet r = statement.executeQuery(sql);
+
+        r.next();
+        Boolean v = r.getBoolean(1);
+        boolean value = v.booleanValue();
+        assertTrue(value);
+        r.close();
     }
 
     /**
@@ -127,8 +148,8 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.overlapsRelaxed(session, a, ta, b, tb));
-        Assert.assertTrue(DateTimeType.overlapsRelaxed(session, b, tb, a, ta));
+        assertTrue(DateTimeType.overlapsRelaxed(session, a, ta, b, tb));
+        assertTrue(DateTimeType.overlapsRelaxed(session, b, tb, a, ta));
     }
 
     /**
@@ -137,7 +158,7 @@ public class TestPeriodPredicates extends TestBase {
      * (TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')
      * is false
      */
-    public void testFirstPeriodDoesNotOverlapsSecondPeriod() {
+    public void testFirstPeriodDoesNotOverlapSecondPeriod() {
 
         Object[] a = {
             scanner.newTimestamp("1999-12-01 01:02:03"),
@@ -156,8 +177,24 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
-        Assert.assertFalse(DateTimeType.overlaps(session, b, tb, a, ta));
+        assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertFalse(DateTimeType.overlaps(session, b, tb, a, ta));
+    }
+
+    public void testSQLFirstPeriodDoesNotOverlapSecondPeriod() throws SQLException {
+
+        String sql =
+            "CALL "
+            + "PERIOD(TIMESTAMP '1999-12-01 01:02:03', TIMESTAMP '1999-12-31 01:02:03')"
+            + "OVERLAPS "
+            + "PERIOD(TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')";
+        ResultSet r = statement.executeQuery(sql);
+
+        r.next();
+        Boolean v = r.getBoolean(1);
+        boolean value = v.booleanValue();
+        assertFalse(value);
+        r.close();
     }
 
     /**
@@ -166,7 +203,7 @@ public class TestPeriodPredicates extends TestBase {
      * (TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')
      * is false
      */
-    public void testFirstPeriodDoesNotOverlapsSecondPeriod2() {
+    public void testFirstPeriodDoesNotOverlapSecondPeriod2() {
 
         Object[] a = {
             scanner.newTimestamp("1999-12-01 01:02:03"),
@@ -185,8 +222,24 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
-        Assert.assertFalse(DateTimeType.overlaps(session, b, tb, a, ta));
+        assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertFalse(DateTimeType.overlaps(session, b, tb, a, ta));
+    }
+
+    public void testSQLFirstPeriodDoesNotOverlapSecondPeriod2() throws SQLException {
+
+        String sql =
+            "CALL "
+            + "PERIOD(TIMESTAMP '1999-12-01 01:02:03', TIMESTAMP '2000-01-01 01:02:03')"
+            + "OVERLAPS "
+            + "PERIOD(TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')";
+        ResultSet r = statement.executeQuery(sql);
+
+        r.next();
+        Boolean v = r.getBoolean(1);
+        boolean value = v.booleanValue();
+        assertFalse(value);
+        r.close();
     }
 
     /**
@@ -214,8 +267,8 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
-        Assert.assertTrue(DateTimeType.overlaps(session, b, tb, a, ta));
+        assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertTrue(DateTimeType.overlaps(session, b, tb, a, ta));
     }
 
     /**
@@ -245,7 +298,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertTrue(DateTimeType.overlaps(session, a, ta, b, tb));
     }
 
     /**
@@ -275,7 +328,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertTrue(DateTimeType.overlaps(session, b2, tb, a2, ta));
+        assertTrue(DateTimeType.overlaps(session, b2, tb, a2, ta));
     }
 
     /**
@@ -284,7 +337,7 @@ public class TestPeriodPredicates extends TestBase {
      * (TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:03')
      * is false
      */
-    public void testFirstPeriodDoesNotOverlapsSecondPeriodWithInterval() {
+    public void testFirstPeriodDoesNotOverlapSecondPeriodWithInterval() {
 
         Object[] a = {
             scanner.newTimestamp("1999-12-01 01:02:03"),
@@ -305,7 +358,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
+        assertFalse(DateTimeType.overlaps(session, a, ta, b, tb));
     }
 
     /**
@@ -337,7 +390,7 @@ public class TestPeriodPredicates extends TestBase {
 
         try {
             DateTimeType.overlaps(null, a, ta, b, tb);
-            Assert.fail("It is illegal to start a period with an interval");
+            fail("It is illegal to start a period with an interval");
         } catch (Exception e) {}
     }
 
@@ -370,7 +423,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertTrue(DateTimeType.precedes(session, a, ta, b, tb));
+        assertTrue(DateTimeType.precedes(session, a, ta, b, tb));
     }
 
     /**
@@ -398,7 +451,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.precedes(session, a, ta, b, tb));
+        assertTrue(DateTimeType.precedes(session, a, ta, b, tb));
     }
 
     /**
@@ -426,7 +479,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
+        assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
     }
 
     /**
@@ -454,7 +507,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
+        assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
     }
 
     /**
@@ -486,7 +539,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
+        assertFalse(DateTimeType.precedes(session, a, ta, b, tb));
     }
 
     /**
@@ -514,7 +567,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.succeeds(session, a, ta, b, tb));
+        assertTrue(DateTimeType.succeeds(session, a, ta, b, tb));
     }
 
     /**
@@ -542,7 +595,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
+        assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
     }
 
     /**
@@ -570,7 +623,23 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
+        assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
+    }
+
+    public void testSQLFirstPeriodDoesNotSucceedsSecondPeriod2() throws SQLException {
+
+        String sql =
+            "CALL "
+            + "PERIOD(TIMESTAMP '2000-02-01 01:02:03', TIMESTAMP '2000-03-01 01:02:03')"
+            + "SUCCEEDS "
+            + "PERIOD(TIMESTAMP '2000-01-01 01:02:03', TIMESTAMP '2000-02-01 01:02:04')";
+        ResultSet r = statement.executeQuery(sql);
+
+        r.next();
+        Boolean v = r.getBoolean(1);
+        boolean value = v.booleanValue();
+        assertFalse(value);
+        r.close();
     }
 
     /**
@@ -602,7 +671,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertTrue(DateTimeType.succeeds(session, a, ta, b, tb));
+        assertTrue(DateTimeType.succeeds(session, a, ta, b, tb));
     }
 
     /**
@@ -634,7 +703,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
+        assertFalse(DateTimeType.succeeds(session, a, ta, b, tb));
     }
 
     /**
@@ -662,8 +731,8 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.equals(session, a, ta, b, tb));
-        Assert.assertTrue(DateTimeType.equals(session, b, tb, a, ta));
+        assertTrue(DateTimeType.equals(session, a, ta, b, tb));
+        assertTrue(DateTimeType.equals(session, b, tb, a, ta));
     }
 
     /**
@@ -693,7 +762,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.equals(session, a, ta, b, tb));
+        assertTrue(DateTimeType.equals(session, a, ta, b, tb));
     }
 
     /**
@@ -725,7 +794,7 @@ public class TestPeriodPredicates extends TestBase {
             IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY, 2, 0)
         };
 
-        Assert.assertTrue(DateTimeType.equals(session, a, ta, b, tb));
+        assertTrue(DateTimeType.equals(session, a, ta, b, tb));
     }
 
     /**
@@ -740,11 +809,10 @@ public class TestPeriodPredicates extends TestBase {
             scanner.newTimestamp("1999-12-01 01:02:01"),
             scanner.newTimestamp("1999-12-01 01:02:03")
         };
-        Object[] b = {
-            scanner.newTimestamp("1999-12-01 01:02:03"),
-            null
+        Object[] b  = {
+            scanner.newTimestamp("1999-12-01 01:02:03"), null
         };
-        Type[] ta = {
+        Type[]   ta = {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0),
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
@@ -753,8 +821,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.contains(session, a, ta, b, tb,
-                true));
+        assertFalse(DateTimeType.contains(session, a, ta, b, tb, true));
     }
 
     /**
@@ -769,11 +836,10 @@ public class TestPeriodPredicates extends TestBase {
             scanner.newTimestamp("1999-12-01 01:02:03"),
             scanner.newTimestamp("1999-12-31 01:02:03")
         };
-        Object[] b = {
-            scanner.newTimestamp("1999-12-01 01:02:03"),
-            null
+        Object[] b  = {
+            scanner.newTimestamp("1999-12-01 01:02:03"), null
         };
-        Type[] ta = {
+        Type[]   ta = {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0),
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
@@ -782,7 +848,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.contains(session, a, ta, b, tb, true));
+        assertTrue(DateTimeType.contains(session, a, ta, b, tb, true));
     }
 
     /**
@@ -797,11 +863,10 @@ public class TestPeriodPredicates extends TestBase {
             scanner.newTimestamp("1999-12-01 01:02:03"),
             scanner.newTimestamp("1999-12-31 01:02:03")
         };
-        Object[] b = {
-            scanner.newTimestamp("1999-12-31 01:02:03"),
-            null
+        Object[] b  = {
+            scanner.newTimestamp("1999-12-31 01:02:03"), null
         };
-        Type[] ta = {
+        Type[]   ta = {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0),
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
@@ -810,8 +875,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.contains(session, a, ta, b, tb,
-                true));
+        assertFalse(DateTimeType.contains(session, a, ta, b, tb, true));
     }
 
     /**
@@ -839,7 +903,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
+        assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
     }
 
     /**
@@ -867,7 +931,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
+        assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
     }
 
     /**
@@ -895,7 +959,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
+        assertTrue(DateTimeType.contains(session, a, ta, b, tb, false));
     }
 
     /**
@@ -923,8 +987,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.contains(session, a, ta, b, tb,
-                false));
+        assertFalse(DateTimeType.contains(session, a, ta, b, tb, false));
     }
 
     /**
@@ -952,8 +1015,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.immediatelyPrecedes(session, a, ta, b,
-                tb));
+        assertTrue(DateTimeType.immediatelyPrecedes(session, a, ta, b, tb));
     }
 
     /**
@@ -981,8 +1043,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertFalse(DateTimeType.immediatelyPrecedes(session, a, ta, b,
-                tb));
+        assertFalse(DateTimeType.immediatelyPrecedes(session, a, ta, b, tb));
     }
 
     /**
@@ -1010,8 +1071,7 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.immediatelySucceeds(session, a, ta, b,
-                tb));
+        assertTrue(DateTimeType.immediatelySucceeds(session, a, ta, b, tb));
     }
 
     /**
@@ -1039,7 +1099,6 @@ public class TestPeriodPredicates extends TestBase {
             new DateTimeType(Types.SQL_TIMESTAMP, Types.SQL_TIMESTAMP, 0)
         };
 
-        Assert.assertTrue(DateTimeType.immediatelySucceeds(session, a, ta, b,
-                tb));
+        assertTrue(DateTimeType.immediatelySucceeds(session, a, ta, b, tb));
     }
 }
