@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,6 +70,7 @@ public class TestUtil {
      * The executing scripts do have state.  This class should be
      * redesigned with OOD.
      */
+    static private final String dbPath = "testdb/test1";
     static private final SimpleDateFormat sdfYMDHMS =
         new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
     static private boolean      abortOnErr        = false;
@@ -128,12 +129,12 @@ public class TestUtil {
 
     static void testScripts(String directory, StopWatch sw) {
 
-        TestUtil.deleteDatabase("test1");
+        TestUtil.deleteDatabase(dbPath);
 
         try {
             Class.forName("org.hsqldb.jdbc.JDBCDriver");
 
-            String     url = "jdbc:hsqldb:test1;sql.enforce_strict_size=true";
+            String     url = "jdbc:hsqldb:file:" + dbPath + ";sql.enforce_strict_size=true";
             String     user        = "sa";
             String     password    = "";
             Connection cConnection = null;
@@ -222,7 +223,7 @@ public class TestUtil {
          */
         try {
             while (true) {
-                HsqlArrayList section = sqlReader.getSection();
+                HsqlArrayList section = sqlReader.getNextSection();
 
                 startLine = sqlReader.getStartLineNumber();
 
