@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@ package org.hsqldb.types;
 
 import java.io.InputStream;
 
+import org.hsqldb.HsqlException;
 import org.hsqldb.SessionInterface;
 import org.hsqldb.error.Error;
 import org.hsqldb.result.Result;
@@ -42,7 +43,7 @@ import org.hsqldb.result.ResultLob;
  * Locator for BLOB.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.5
+ * @version 2.5.1
  * @since 1.9.0
  */
 public class BlobDataID implements BlobData {
@@ -60,7 +61,13 @@ public class BlobDataID implements BlobData {
         Result    resultIn  = session.execute(resultOut);
 
         if (resultIn.isError()) {
-            throw resultIn.getException();
+            HsqlException ex = resultIn.getException();
+
+            if (ex == null) {
+                ex = new HsqlException(resultIn);
+            }
+
+            throw ex;
         }
 
         long lobID = ((ResultLob) resultIn).getLobID();
@@ -148,6 +155,7 @@ public class BlobDataID implements BlobData {
         Result    resultIn  = session.execute(resultOut);
 
         if (resultIn.isError()) {
+
             throw resultIn.getException();
         }
 
