@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,6 @@ package org.hsqldb.types;
 
 import java.math.BigDecimal;
 
-import org.hsqldb.HsqlDateTime;
 import org.hsqldb.OpTypes;
 import org.hsqldb.Session;
 import org.hsqldb.SessionInterface;
@@ -384,28 +383,28 @@ public class CharacterType extends Type {
         } else if (la > lb) {
             if (collation.isPadSpace()
                     && opType != OpTypes.GREATER_EQUAL_PRE) {
-                char[] buffer = new char[la];
-
-                bs.getChars(0, lb, buffer, 0);
-                ArrayUtil.fillArray(buffer, lb, ' ');
-
-                bs = String.valueOf(buffer);
+                bs = padString(bs, la);
             }
         } else {
             if (collation.isPadSpace()
                     && opType != OpTypes.GREATER_EQUAL_PRE) {
-                char[] buffer = new char[lb];
-
-                as.getChars(0, la, buffer, 0);
-                ArrayUtil.fillArray(buffer, la, ' ');
-
-                as = String.valueOf(buffer);
+                as = padString(as, lb);
             }
         }
 
         return collation.compare(as, bs);
     }
 
+    public static String padString(String s, int length) {
+
+        char[] buffer = new char[length];
+
+        s.getChars(0, s.length(), buffer, 0);
+        ArrayUtil.fillArray(buffer, s.length(), ' ');
+
+        return String.valueOf(buffer);
+    }
+
     public Object convertToTypeLimits(SessionInterface session, Object a) {
 
         if (a == null) {
@@ -648,8 +647,8 @@ public class CharacterType extends Type {
         } else if (a instanceof java.sql.Timestamp) {
             s = a.toString();
         } else if (a instanceof java.util.Date) {
-            s = HsqlDateTime.getTimestampString(
-                ((java.util.Date) a).getTime());
+            a = new java.sql.Timestamp(((java.util.Date) a).getTime());
+            s = a.toString();
         } else if (a instanceof java.util.UUID) {
             s = a.toString();
         } else {
@@ -736,7 +735,7 @@ public class CharacterType extends Type {
     }
 
     public boolean canConvertFrom(Type otherType) {
-        return !otherType.isObjectType() && !otherType.isArrayType();
+        return !otherType.isObjectType();
     }
 
     public int canMoveFrom(Type otherType) {
@@ -745,10 +744,6 @@ public class CharacterType extends Type {
             return 0;
         }
 
-        if (!otherType.isCharacterType()) {
-            return -1;
-        }
-
         switch (typeCode) {
 
             case Types.SQL_VARCHAR : {
@@ -1173,7 +1168,8 @@ public class CharacterType extends Type {
                     return charArray[(int) length];
                 }
 
-            // fall through
+                return new CharacterType(type, (int) length);
+
             case Types.SQL_VARCHAR :
                 return new CharacterType(type, (int) length);
 
