@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,7 @@ import org.hsqldb.rights.Grantee;
  *
  * @author Frand Schoenheit (frank.schoenheit@sun dot com)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.8.0
  */
 public class Collation implements SchemaObject {
@@ -471,14 +471,14 @@ public class Collation implements SchemaObject {
         return isFinal;
     }
 
-    public HsqlName getName() {
-        return name;
-    }
-
     public int getType() {
         return SchemaObject.COLLATION;
     }
 
+    public HsqlName getName() {
+        return name;
+    }
+
     public HsqlName getSchemaName() {
         return name.schema;
     }
@@ -532,7 +532,9 @@ public class Collation implements SchemaObject {
         sb.append(sourceName.statementName);
         sb.append(' ');
 
-        if (!padSpace) {
+        if (padSpace) {
+            sb.append(Tokens.T_PAD).append(' ').append(Tokens.T_SPACE);
+        } else {
             sb.append(Tokens.T_NO).append(' ').append(Tokens.T_PAD);
         }
 
@@ -568,7 +570,9 @@ public class Collation implements SchemaObject {
         sb.append(getName().statementName);
         sb.append(' ');
 
-        if (!padSpace) {
+        if (padSpace) {
+            sb.append(Tokens.T_PAD).append(' ').append(Tokens.T_SPACE);
+        } else {
             sb.append(Tokens.T_NO).append(' ').append(Tokens.T_PAD);
         }
 
