@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,7 +48,7 @@ import org.hsqldb.map.ValuePool;
  * Type subclass for all NUMBER types.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.5.0
+ * @version 2.5.1
  * @since 1.9.0
  */
 public final class NumberType extends Type {
@@ -56,7 +56,7 @@ public final class NumberType extends Type {
     static final int        tinyintPrecision             = 3;
     static final int        smallintPrecision            = 5;
     static final int        integerPrecision             = 10;
-    static final int        bigintPrecision              = 19;
+    public static final int bigintPrecision              = 19;
     static final int        doublePrecision              = 0;
     public static final int defaultNumericPrecision      = 128;
     public static final int defaultNumericScale          = 32;
@@ -893,7 +893,7 @@ public final class NumberType extends Type {
                     break;
                 }
 
-            // fall through
+                throw Error.error(ErrorCode.X_42561);
             default :
                 throw Error.error(ErrorCode.X_42561);
         }
@@ -1170,20 +1170,7 @@ public final class NumberType extends Type {
 
         double value;
 
-        if (a instanceof java.lang.Double) {
-            return ((Double) a).doubleValue();
-        } else if (a instanceof BigDecimal) {
-            BigDecimal bd = (BigDecimal) a;
-
-            value = bd.doubleValue();
-
-            int        signum = bd.signum();
-            BigDecimal bdd    = new BigDecimal(value + signum);
-
-            if (bdd.compareTo(bd) != signum) {
-                throw Error.error(ErrorCode.X_22003);
-            }
-        } else if (a instanceof Number) {
+        if (a instanceof Number) {
             value = ((Number) a).doubleValue();
         } else {
             throw Error.error(ErrorCode.X_22501);
@@ -1303,30 +1290,23 @@ public final class NumberType extends Type {
         switch (typeCode) {
 
             case Types.TINYINT :
-                if (otherType.typeCode == Types.SQL_SMALLINT
-                        || otherType.typeCode == Types.SQL_INTEGER) {
+                if (otherType.typeCode == Types.SQL_SMALLINT) {
                     return 1;
                 }
                 break;
 
             case Types.SQL_SMALLINT :
-                if (otherType.typeCode == Types.TINYINT) {
-                    return 0;
-                }
-
-                if (otherType.typeCode == Types.SQL_INTEGER) {
-                    return 1;
-                }
-                break;
-
-            case Types.SQL_INTEGER :
-                if (otherType.typeCode == Types.SQL_SMALLINT
+                if (otherType.typeCode == typeCode
                         || otherType.typeCode == Types.TINYINT) {
                     return 0;
                 }
                 break;
 
+            case Types.SQL_INTEGER :
             case Types.SQL_BIGINT :
+                if (otherType.typeCode == typeCode) {
+                    return 0;
+                }
                 break;
 
             case Types.SQL_DECIMAL :
