@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2019, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ import org.hsqldb.rights.Grantee;
  * Class for DOMAIN and DISTINCT objects.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.5.0
  * @since 1.9.0
  */
 public class UserTypeModifier {
@@ -165,6 +165,12 @@ public class UserTypeModifier {
             }
         }
 
+        Collation collation = dataType.getCollation();
+
+        if (collation != null && collation.isObjectCollation()) {
+            set.add(collation.getName());
+        }
+
         return set;
     }
 
@@ -190,7 +196,7 @@ public class UserTypeModifier {
 
     public String getSQL() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         if (schemaObjectType == SchemaObject.TYPE) {
             sb.append(Tokens.T_CREATE).append(' ').append(
@@ -199,8 +205,9 @@ public class UserTypeModifier {
             sb.append(' ').append(Tokens.T_AS).append(' ');
             sb.append(dataType.getDefinition());
 
-            if (dataType.isCharacterType() ) {
+            if (dataType.isCharacterType()) {
                 Collation collation = dataType.getCollation();
+
                 if (collation.isObjectCollation()) {
                     sb.append(' ').append(collation.getCollateSQL());
                 }
@@ -212,6 +219,14 @@ public class UserTypeModifier {
             sb.append(' ').append(Tokens.T_AS).append(' ');
             sb.append(dataType.getDefinition());
 
+            if (dataType.isCharacterType()) {
+                Collation collation = dataType.getCollation();
+
+                if (collation.isObjectCollation()) {
+                    sb.append(' ').append(collation.getCollateSQL());
+                }
+            }
+
             if (defaultExpression != null) {
                 sb.append(' ').append(Tokens.T_DEFAULT).append(' ');
                 sb.append(defaultExpression.getSQL());
