@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2019, The HSQL Development Group
+/* Copyright (c) 2001-2020, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +43,6 @@ import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
 import java.util.ArrayList;
-
 import java.awt.BorderLayout;
 import java.awt.Button;
 import java.awt.Color;
@@ -102,7 +101,7 @@ import org.hsqldb.lib.RCData;
  * Originally in HypersonicSQL. Extended in various versions of HSQLDB.
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
- * @version 2.5.0
+ * @version 2.5.1
  * @since Hypersonic SQL
  */
 public class DatabaseManager extends Applet
@@ -125,13 +124,13 @@ implements ActionListener, WindowListener, KeyListener {
         "See the forums, mailing lists, and HSQLDB User Guide\n"
         + "at http://hsqldb.org.\n\n"
         + "Please paste the following version identifier with any\n"
-        + "problem reports or help requests:  $Revision: 5982 $"
+        + "problem reports or help requests:  $Revision: 6139 $"
         + (TT_AVAILABLE ? ""
                         : ("\n\nTransferTool classes are not in CLASSPATH.\n"
                            + "To enable the Tools menu, add 'transfer.jar' to your class path."));
     ;
     private static final String ABOUT_TEXT =
-        "$Revision: 5982 $ of DatabaseManager\n\n"
+        "$Revision: 6139 $ of DatabaseManager\n\n"
         + "Copyright (c) 1995-2000, The Hypersonic SQL Group.\n"
         + "Copyright (c) 2001-2019, The HSQL Development Group.\n"
         + "http://hsqldb.org  (User Guide available at this site).\n\n\n"
@@ -162,7 +161,7 @@ implements ActionListener, WindowListener, KeyListener {
     String           ifHuge = "";
 
     // (ulrivo): variables set by arguments from the commandline
-    static String defDriver   = "org.hsqldb.jdbcDriver";
+    static String defDriver   = "org.hsqldb.jdbc.JDBCDriver";
     static String defURL      = "jdbc:hsqldb:mem:.";
     static String defUser     = "SA";
     static String defPassword = "";
@@ -862,6 +861,22 @@ implements ActionListener, WindowListener, KeyListener {
             if (r == -1) {
                 ResultSet rs = sStatement.getResultSet();
 
+                try {
+                    formatResultSet(rs);
+                } catch (Throwable t) {
+                    g[0] = "Error displaying the ResultSet";
+
+                    gResult.setHead(g);
+
+                    String s = t.getMessage();
+
+                    g[0] = s;
+
+                    gResult.addRow(g);
+                }
+            } else if (sStatement.getMoreResults()) {    // repeated for if a procedure returns a result set
+                ResultSet rs = sStatement.getResultSet();
+
                 try {
                     formatResultSet(rs);
                 } catch (Throwable t) {
@@ -1099,12 +1114,12 @@ implements ActionListener, WindowListener, KeyListener {
 
     void showResultInText() {
 
-        String[] col   = gResult.getHead();
-        int      width = col.length;
-        int[]    size  = new int[width];
-        ArrayList<String[]> data = gResult.getData();
-        String[] row;
-        int      height = data.size();
+        String[]  col   = gResult.getHead();
+        int       width = col.length;
+        int[]     size  = new int[width];
+        ArrayList<String[]> data  = gResult.getData();
+        String[]  row;
+        int       height = data.size();
 
         for (int i = 0; i < width; i++) {
             size[i] = col[i].length();
@@ -1256,6 +1271,10 @@ implements ActionListener, WindowListener, KeyListener {
         tTree.removeAll();
 
         try {
+            if (cConn == null) {
+                throw new SQLException("no connection");
+            }
+
             wasAutoCommit = cConn.getAutoCommit();
 
             cConn.setAutoCommit(false);
@@ -1289,14 +1308,14 @@ implements ActionListener, WindowListener, KeyListener {
             }
 
             for (int i = 0; i < tables.size(); i++) {
-                String name   = (String) tables.get(i);
-                String schema = (String) schemas.get(i);
+                String name   = tables.get(i);
+                String schema = schemas.get(i);
                 String key    = "tab-" + name + "-";
 
                 tTree.addRow(key, name, "+", color_table);
 
                 // sqlbob@users Added remarks.
-                String remark = (String) remarks.get(i);
+                String remark = remarks.get(i);
 
                 if ((schema != null) && !schema.trim().equals("")) {
                     tTree.addRow(key + "s", "schema: " + schema);
@@ -1370,7 +1389,9 @@ implements ActionListener, WindowListener, KeyListener {
             tTree.addRow("-", e.getSQLState());
         } finally {
             try {
-                cConn.setAutoCommit(wasAutoCommit);
+                if (cConn != null) {
+                    cConn.setAutoCommit(wasAutoCommit);
+                }
             } catch (SQLException e) {}
         }
 
@@ -1386,5 +1407,4 @@ implements ActionListener, WindowListener, KeyListener {
             DriverManager.setLogWriter(newPrintWriter);
         } catch (Exception e) {}
     }
-
 }
